#===============================================================
#
#   bld.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
package bld;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
BLD_init_build
BLD_init_audit
BLD_get_commands
);
}




use glo::env;
use gbsenv;
use swa;
use target;
use audit;
use bldopt;
use gbsbg;
use fspec;




sub BLD_init_build();
sub BLD_init_audit($);
sub BLD_get_commands($$$);




my $GBS_SCRIPTS_PATH = GBSENV_scripts_path();
my $GBS_ROOT_PATH = GBSENV_gbs_root_path();
my $GBS_SUBSYS = ENV_getenv( 'GBS_SUBSYS');
my $GBS_COMP_PATH = "$GBS_ROOT_PATH/dev/$GBS_SUBSYS/comp";
my $GBS_TARGET = ENV_getenv( 'GBS_TARGET');
my $GBS_TARGET_PLUGIN = ENV_getenv( 'GBS_TARGET_PLUGIN');

my $GBS_AUDIT;
my $GBS_AUDIT_PLUGIN;
my $SCADEF_IS_PRESENT;

my $VIA_FORMAT;

my $OPTIONS_BANNER;





sub BLD_init_build()
{
$GBS_AUDIT = undef;

$OPTIONS_BANNER = BLDOPT_init( undef);






{
my $plugin_rel = GBSENV_get_plugin_data( $GBS_TARGET_PLUGIN);
ENV_whisper( 1, "$GBS_TARGET [$GBS_TARGET_PLUGIN ($plugin_rel)]");
}

($VIA_FORMAT, undef) = TARGET_get_section_items( $GBS_TARGET, SETUP => 'VIA_FORMAT');

return $OPTIONS_BANNER;
}




sub BLD_init_audit($)
{
($GBS_AUDIT) = @_;	# can have multiple audit-runs in one execution of gbsaudit command

$OPTIONS_BANNER = BLDOPT_init( $GBS_AUDIT);



$GBS_AUDIT_PLUGIN = SWA_set( $GBS_AUDIT, 0);

AUDIT_set_envs( $GBS_AUDIT);
$SCADEF_IS_PRESENT = (-e FSPEC_scadef( $GBS_AUDIT_PLUGIN)) ? 1 : 0;




{
my $plugin_rel = GBSENV_get_plugin_data( $GBS_TARGET_PLUGIN);
ENV_whisper( 1, "$GBS_TARGET [$GBS_TARGET_PLUGIN ($plugin_rel)]");
}




{
my $plugin_rel = GBSENV_get_plugin_data( $GBS_AUDIT_PLUGIN);
my @src_types = AUDIT_get_src_types( $GBS_AUDIT);
my $sca_text = ($SCADEF_IS_PRESENT) ? ' SCA' : '';
ENV_whisper( 1, "$GBS_AUDIT [$GBS_AUDIT_PLUGIN ($plugin_rel)$sca_text] (@src_types)");
}

($VIA_FORMAT, undef) = AUDIT_get_section_items( $GBS_AUDIT, SETUP => 'VIA_FORMAT');

return $OPTIONS_BANNER;
}







sub BLD_get_commands($$$)
{
my ($component,
$in_file,
$command_line_flags_ref,
) = @_;
my @lines;

my ($file_name, $src_type) = ENV_split_spec_nt( $in_file);


my $src_dir = "$GBS_COMP_PATH/$component/src";

my $out_file_path;
my $out_file_base;
my $gen_type;
my $out_filetype;
my $map_file;
my $src_abs_path;	    # bool
my $gen_type_verb;
my @out_types;
my @opt_out_types;
my @out_files;
my $out_keep_file;
if (defined $GBS_AUDIT)
{
$out_file_path	= "../aud/$GBS_AUDIT/$GBS_TARGET";
$out_file_base	= "$out_file_path/$in_file";
$gen_type	= '';
$out_filetype	= AUDIT_get_primary_out_type( $GBS_AUDIT, $src_type);
$src_abs_path	= 0;
$gen_type_verb	= 'Auditing';
@out_types	= (AUDIT_get_src_items( $GBS_AUDIT, $src_type, 'OUT_TYPES'),
qw( .garg .gerr .gmet .gout .html ));
@opt_out_types	= (qw( .log.html ));
@out_files	= ();

$out_keep_file	= "$out_file_base.gout";
} else
{
$out_file_path	= "../bld/$GBS_TARGET";
$out_file_base	= "$out_file_path/$file_name";
$gen_type	= TARGET_get_src_items( $GBS_TARGET, $src_type, 'TYPE')->[0];
$out_filetype	= TARGET_get_primary_bld_type( $GBS_TARGET, $src_type);
$src_abs_path	= TARGET_get_src_items( $GBS_TARGET, $src_type, 'SRC_ABS_PATH');
$gen_type_verb	= TARGET_get_type_verb( $gen_type);	# E.g.: Compiling, Linking
@out_types	= TARGET_get_src_items( $GBS_TARGET, $src_type, 'OUT_TYPES');
@opt_out_types	= TARGET_get_src_items( $GBS_TARGET, $src_type, 'OPT_OUT_TYPES');
@out_files	= TARGET_get_src_items( $GBS_TARGET, $src_type, 'OUT_FILES');

if ($gen_type eq 'glk')
{
$map_file = "$out_file_base.map";
$out_keep_file = $map_file;
} elsif ($gen_type eq 'glt')
{
$out_keep_file = "$out_file_base$out_filetype";	    # .log file
} else
{

}
}




my @out_del_files  = map { "$out_file_base$_" } (@out_types, @opt_out_types);
push @out_del_files, map { "$out_file_path/$_" } @out_files;
unlink @out_del_files;




@out_del_files = grep( $_ ne $out_keep_file, @out_del_files)
if (defined $out_keep_file);




my @env_def_refs = BLDOPT_get_envs_settings( $component, $out_file_base, $src_type, $map_file);




my $in_filespec = ($src_abs_path) ? "$src_dir/$in_file" : $in_file;
my $out_filespec = "$out_file_base$out_filetype";
my @pos_args = ($file_name, $in_filespec, $out_filespec, $out_file_path, $out_filetype);
my @command_data_refs;
if (defined $GBS_AUDIT)
{
@command_data_refs = AUDIT_get_src_command_data_refs_exp(  $GBS_AUDIT,  $src_type, \@pos_args, $command_line_flags_ref);
} else
{
@command_data_refs = TARGET_get_src_command_data_refs_exp( $GBS_TARGET, $src_type, \@pos_args, $command_line_flags_ref);
}




{



push @lines, GBSBG_echo( "** $gen_type_verb ${GBS_SUBSYS}:$component:$in_file to $GBS_TARGET:$file_name$out_filetype...");



map { push @lines, GBSBG_set( $_->[0], $_->[1]) } @env_def_refs;




if (defined $GBS_AUDIT)
{
if ($SCADEF_IS_PRESENT)
{



my $unix_style_search = (TARGET_get_src_items( $GBS_TARGET, $src_type, 'INC_SEARCH_STYLE') eq 'UNIX') ? 1 : 0;
push @lines, GBSBG_sca( $in_file, $unix_style_search, undef, @command_data_refs);  # out_files
} else
{



push @lines, GBSBG_exec( $in_file, undef, \@out_del_files, @command_data_refs); # $must_print_commands, out_files
}
} else
{
my ($glkb_type, undef) = TARGET_get_src_items( $GBS_TARGET, $src_type, 'GLKB');
if (defined $glkb_type)
{
my $glkb_args_ref = [ $glkb_type, $in_file, $out_filespec, $VIA_FORMAT ];
push @lines, GBSBG_glkb( $in_file, undef, $glkb_args_ref, \@out_del_files, @command_data_refs);	    # $must_print_commands
} else
{
push @lines, GBSBG_exec( $in_file, undef, \@out_del_files, @command_data_refs);			    # $must_print_commands
}
}

}

return @lines;
}

1;

