#===============================================================
#
#   bldopt.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
package bldopt;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
BLDOPT_init
BLDOPT_get_envs_settings
);
}




use glo::env;
use glo::spit;
use gbsenv;
use sys;
use target;
use audit;
use flincs;




sub BLDOPT_init($);
sub BLDOPT_get_envs_settings($$$$);

sub get_incs($$);
sub format_incs($$);
sub get_flags($$$);
sub format_flags($$);




my $GBS_ROOT_PATH = GBSENV_gbs_root_path();
my $GBS_SYSBUILD_PATH = "$GBS_ROOT_PATH/sysbuild";
my $GBS_SUBSYS = ENV_getenv( 'GBS_SUBSYS');
my $GBS_COMP_PATH = "$GBS_ROOT_PATH/dev/$GBS_SUBSYS/comp";
my $GBS_TARGET = ENV_getenv( 'GBS_TARGET');

my $GBS_AUDIT;
my $FOR_AUDIT;	# bool

my $VIA_FORMAT;
my $VIA_FILE_TYPE;

my @OPTION_REFS;    #  MODE OPT DEBUGGER MAP


my %FLAGS_CACHE;


my %INCS_CACHE;





my $SUBS_TOKEN_1 = (ENV_is_win32()) ? '%1' : '\\$1';	    # %1 or $1

my @FLAGS_INCS_ORDER = qw( sysflags flags sysincs incs);




sub BLDOPT_init($)
{
($GBS_AUDIT) = @_;		# undef or '' if for build

my $banner;

$FOR_AUDIT = ($GBS_AUDIT) ? 1 : 0;


if ($FOR_AUDIT)
{





($VIA_FORMAT, $VIA_FILE_TYPE) = AUDIT_get_section_items( $GBS_AUDIT, SETUP => 'VIA_FORMAT');

%FLAGS_CACHE = ();
%INCS_CACHE = ();
} else
{



TARGET_set_envs( $GBS_TARGET);	# Also sets: GBS_BLD_*, GBS_MODE, GBS_OPT, etc
SYS_set_envs();			# Checks and sets GBS_APP_*

($VIA_FORMAT, $VIA_FILE_TYPE) = TARGET_get_section_items( $GBS_TARGET, SETUP => 'VIA_FORMAT');
}

@OPTION_REFS = TARGET_get_options( $GBS_TARGET, $FOR_AUDIT);    #  MODE OPT DEBUGGER MAP





my @banner;
foreach my $opt_ref (@OPTION_REFS)    #  MODE OPT DEBUGGER MAP
{
my ($opt_name, $default_value_name, undef, $choice) = @{$opt_ref};
my $banner = "$opt_name=$choice";
$banner .= " ($default_value_name)"
if ($choice ne $default_value_name);
push @banner, $banner;
}
$banner = join( ', ', @banner);

return $banner;
}




sub BLDOPT_get_envs_settings($$$$)
{
my ($component,	    # '' if non-gbs subsys
$out_file_base,
$src_type,
$map_filespec,
) = @_;
my @env_def_refs;	    # [ $envvar_name => $envvar_value ]

my $env_name_postfix = '';
if ($component eq '')
{
$env_name_postfix = '_' . uc( substr( $src_type, 1));
}




my @flinc_filespecs;
my $flincs_filetype = ($VIA_FILE_TYPE) ? $VIA_FILE_TYPE : '.gbs';
foreach my $flinc (@FLAGS_INCS_ORDER)   # = qw( sysflags flags sysincs incs)
{
push @flinc_filespecs, "$out_file_base.$flinc$flincs_filetype";
}

unlink @flinc_filespecs;




my @flincs;

push @flincs, get_flags( $component, $src_type, $map_filespec);
push @flincs, get_incs(  $component, $src_type);

for (my $i = 0; $i < @FLAGS_INCS_ORDER; $i++) # = qw( sysflags flags sysincs incs)
{
my $flinc_type = $flincs[$i*2];
my $flincs_ref = $flincs[$i*2+1];

my $envvar_value;
if ($flinc_type eq 'FILE')
{




my $flinc_filespec = $flinc_filespecs[$i];
SPIT_file_nl( $flinc_filespec, $flincs_ref);
$envvar_value = sprintf( $VIA_FORMAT, $flinc_filespec);
} else	#$flincs_type eq 'VARIABLE'
{
$envvar_value = "@{$flincs_ref}";

}
my $env_name = $FLAGS_INCS_ORDER[$i];	# = qw( sysflags flags sysincs incs)
my $flinc_uc = uc $env_name;
push @env_def_refs, [ "GBS_$flinc_uc$env_name_postfix" => $envvar_value ];
}




if ($FOR_AUDIT)
{
my $audit_flags = ENV_getenv( 'GBS_FLAGS_'. uc $GBS_AUDIT);
push @env_def_refs, [ 'GBS_AUDIT_FLAGS', $audit_flags];
}





return @env_def_refs;
}




sub get_incs($$)
{
my ($component,
$src_type,
) = @_;
my ( $sysincs_type, $sysincs_ref, $incs_type, $incs_ref);

my $incs_src_ref = ($component) ? $INCS_CACHE{$component}->{$src_type} : undef;
if (defined $incs_src_ref)
{
( $sysincs_type, $sysincs_ref, $incs_type, $incs_ref ) = @{$incs_src_ref};
} else
{
my ( @sysincs, @incs );




my ($inc_format, $sysinc_format, $inc_abs_path);
if ($FOR_AUDIT)
{
my ($inc_format_ref, $sysinc_format_ref) =
AUDIT_get_src_items( $GBS_AUDIT, $src_type, qw( INC_FORMAT SYSINC_FORMAT));
($incs_type, $inc_format) = ENV_undef2empty( @{$inc_format_ref});
($sysincs_type, $sysinc_format) = ENV_undef2empty( @{$sysinc_format_ref});
$inc_abs_path = 0;
} else
{
(my $inc_format_ref, my $sysinc_format_ref, $inc_abs_path) =
TARGET_get_src_items( $GBS_TARGET, $src_type, qw( INC_FORMAT SYSINC_FORMAT INC_ABS_PATH));
($incs_type, $inc_format) = ENV_undef2empty( @{$inc_format_ref});
($sysincs_type, $sysinc_format) = ENV_undef2empty( @{$sysinc_format_ref});
}
my $src_dir = "$GBS_ROOT_PATH/dev/$GBS_SUBSYS/comp/$component/src";




if ($sysinc_format)
{
@sysincs = FLINCS_get_bld_incs( $src_type);
@sysincs = ($inc_abs_path) ?
ENV_abs_path( $src_dir, [ @sysincs ]) :
ENV_shortest_path( $src_dir, [ @sysincs ]);
format_incs( \@sysincs, $sysinc_format);
}
if ($inc_format)
{
@incs = FLINCS_get_incs( $src_type, $component);
@incs = ($inc_abs_path) ?
ENV_abs_path( $src_dir, [ @incs ]) :
ENV_shortest_path( $src_dir, [ @incs ]);
format_incs( \@incs, $inc_format);
}


$sysincs_ref = [ @sysincs ];
$incs_ref    = [ @incs ];
$INCS_CACHE{$component}->{$src_type} = [ $sysincs_type, $sysincs_ref, $incs_type, $incs_ref ]
if ($component);
}

return ( $sysincs_type, $sysincs_ref, $incs_type, $incs_ref );
}




sub get_flags($$$)
{
my ($component,
$src_type,
$map_filespec,
) = @_;
my ( $sysflags_type, $sysflags_ref, $flags_type, $flags_ref, $map_option_index );

my $flags_src_ref = ($component) ? $FLAGS_CACHE{$component}->{$src_type} : undef;
if (defined $flags_src_ref)
{
( $sysflags_type, $sysflags_ref, $flags_type, $flags_ref, $map_option_index ) = @{$flags_src_ref};
} else
{
my ( @sysflags, @flags );




my ($flag_format, $sysflag_format);
if ($FOR_AUDIT)
{
my ($flag_format_ref, $sysflag_format_ref) =
AUDIT_get_src_items( $GBS_AUDIT, $src_type, qw( FLAG_FORMAT SYSFLAG_FORMAT));
($flags_type, $flag_format) = ENV_undef2empty( @{$flag_format_ref});
($sysflags_type, $sysflag_format) = ENV_undef2empty( @{$sysflag_format_ref});
} else
{
my ($flag_format_ref, $sysflag_format_ref) =
TARGET_get_src_items( $GBS_TARGET, $src_type, qw( FLAG_FORMAT SYSFLAG_FORMAT));
($flags_type, $flag_format) = ENV_undef2empty( @{$flag_format_ref});
($sysflags_type, $sysflag_format) = ENV_undef2empty( @{$sysflag_format_ref});
}




if ($sysflag_format)
{
@sysflags = FLINCS_get_bld_flags( $src_type);




@sysflags = grep( /^"?-D/, @sysflags)
if ($FOR_AUDIT);
}




{



push @flags, FLINCS_get_flags( $src_type, $component);




{
my $src_type_name = substr( $src_type, 1);
my $key = 'GBS_FLAGS_'. uc $src_type_name;
my $gbs_flags = ENV_getenv( $key);
push @flags, $gbs_flags
if ($gbs_flags);
}





$map_option_index = -1;
foreach my $opt_ref (@OPTION_REFS)
{
my ($opt_name, $default_value_name, undef, $choice) = @{$opt_ref};    #  MODE OPT DEBUGGER MAP
my @values = TARGET_get_src_subitems( $GBS_TARGET, $src_type, $opt_name, $choice);
if ($opt_name eq 'MAP')
{
foreach my $value (@values)
{
push @flags, $value;
$map_option_index = $#flags
if ($value =~ /$SUBS_TOKEN_1/);	    # $SUBS_TOKEN_1 = (ENV_is_win32()) ? '%1' : '\\$1';
}
} else
{
push @flags, @values;
}
}




@flags = grep( /^"?-D/, @flags)
if ($FOR_AUDIT);
}




format_flags( \@sysflags, $sysflag_format);
$sysflags_ref = [ @sysflags ];

format_flags( \@flags, $flag_format);
$flags_ref = [ @flags ];

$FLAGS_CACHE{$component}->{$src_type} = [ $sysflags_type, $sysflags_ref, $flags_type, $flags_ref, $map_option_index ]
if ($component);
}




if ($map_option_index >= 0)
{



my @flags = @{$flags_ref};					# we must copy the flags for the MAP


$flags[$map_option_index] =~ s/$SUBS_TOKEN_1/$map_filespec/g;	# $SUBS_TOKEN_1 = (ENV_is_win32()) ? '%1' : '\\$1';
$flags_ref = \@flags;
}

return ( $sysflags_type, $sysflags_ref, $flags_type, $flags_ref);
}





sub format_flags($$)
{
my ($flags_ref,	# In/OUT
$flag_format,
) = @_;

my $quoted_format;
if ($flag_format =~ / /)
{
$quoted_format = $flag_format;
$quoted_format =~ s/(.*?) (.*)/$1 "$2"/;
} else
{
$quoted_format = "\"$flag_format\"";
}

foreach my $flag (@{$flags_ref})
{

if (substr( $flag, 0, 2) eq '-D')
{

if ($flag =~ / / && $flag !~ /"/)
{
$flag = sprintf( $quoted_format, substr( $flag, 2));
} else
{
$flag = sprintf( $flag_format, substr( $flag, 2));
}
} elsif (substr( $flag, 0, 3) eq '"-D')
{

$flag = '"' . sprintf( $flag_format, substr( $flag, 3));
} else
{

}

}
}





sub format_incs($$)
{
my ($incs_ref,	# In/OUT
$inc_format,
) = @_;

my $quoted_format;
if ($inc_format =~ / /)
{
$quoted_format = $inc_format;
$quoted_format =~ s/(.*?) (.*)/$1 "$2"/;
} else
{
$quoted_format =  "\"$inc_format\"";
}

foreach my $inc (@{$incs_ref})
{
if ($inc =~ / / && $inc !~ /"/)
{
$inc = sprintf( $quoted_format, $inc);
} else
{
$inc = sprintf( $inc_format, $inc);
}

}
}

1;

