#===============================================
#
#   cfo.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
package cfo;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
CFO_parse_gen
CFO_parse_make
CFO_parse_audit
CFO_pre_options
CFO_get_item
CFO_get_components
);
}




use glo::env;
use glo::format;
use gbsenv;
use gbsglo;
use target;
use audit;




sub CFO_parse_gen($);
sub CFO_parse_make($$);
sub CFO_parse_audit($$);
sub CFO_pre_options();
sub CFO_get_item($);
sub CFO_get_components();

sub pre_parse($);
sub cfo_parse();
sub store_component($$$);
sub store_make_component($);
sub wild_components($);
sub select_src_files($$@);
sub select_target_files($$@);
sub select_gen_and_audit();
sub select_make();
sub validate_component($);
sub validate_file($$);
sub validate_type($$);
sub get_wild_sources($$);
sub is_option($);




my $GBS_ROOT_PATH = GBSENV_gbs_root_path();
my $GBS_SUBSYS = ENV_getenv( 'GBS_SUBSYS');
my $GBS_TARGET;
my $CUR_GBS_AUDIT;		# not taken from EnvVars!

my $GBS_SUBSYS_PATH = "$GBS_ROOT_PATH/dev/$GBS_SUBSYS";
my $GBS_COMP_PATH = "$GBS_SUBSYS_PATH/comp";

my @ALLOWED_COMPS;
my %COMPONENTS;		# per parse


my %SRCTYPES;


my %ALLOWED_TYPES;


my @ALLOWED_TYPES;

my $FUNCTION;		# 'audit', 'generate' or 'make'
my $FUNCTION_TEXT;
my $FOR_AUDIT = 0;
my $FOR_GEN = 0;
my $FOR_MAKE = 0;

my $REVERSE_TYPE = 0;	# make only

my @PRE_OPTIONS;
my @PARSED_REFS;


my @CFO_REFS;


my @TMP_CFO_REFS;






sub CFO_parse_gen($)
{
my ($comp_file_opts_ref,	    # positional parameters
) = @_;





$GBS_TARGET = ENV_getenv( 'GBS_TARGET');
$FUNCTION = 'generate';
$FOR_GEN = 1;
@ALLOWED_TYPES = TARGET_get_src_types( $GBS_TARGET);
map { $ALLOWED_TYPES{$_} = 1 } @ALLOWED_TYPES;

pre_parse( $comp_file_opts_ref)
if (!@PARSED_REFS);

return cfo_parse();
}




sub CFO_parse_make($$)
{
my ($comp_file_opts_ref,	    # positional parameters
$reverse_type) = @_;	    # Specify source-types instead of build-types





$GBS_TARGET = ENV_getenv( 'GBS_TARGET');
$FUNCTION = 'make';
$FOR_MAKE = 1;
$REVERSE_TYPE = $reverse_type;
@ALLOWED_TYPES = ($REVERSE_TYPE) ? TARGET_get_src_types( $GBS_TARGET) : TARGET_get_bld_types( $GBS_TARGET);
map { $ALLOWED_TYPES{$_} = 1 } @ALLOWED_TYPES;

pre_parse( $comp_file_opts_ref)
if (!@PARSED_REFS);

return cfo_parse();
}




sub CFO_parse_audit($$)
{
my ($audit,			    # audit
$comp_file_opts_ref,	    # positional parameters
) = @_;





$GBS_TARGET = ENV_getenv( 'GBS_TARGET');
$CUR_GBS_AUDIT = $audit;
$FUNCTION = 'audit';
$FOR_AUDIT = 1;
my %target_types = map { $_ => 1 } TARGET_get_src_types( $GBS_TARGET);
my @audit_src_types = AUDIT_get_src_types( $audit);
@ALLOWED_TYPES = ();
%ALLOWED_TYPES = ();
foreach my $audit_src_type (@audit_src_types)
{
if (exists $target_types{$audit_src_type})
{
push @ALLOWED_TYPES, $audit_src_type;
}
}
ENV_sig( EE => "No Audits defined")
if (!@ALLOWED_TYPES);
map { $ALLOWED_TYPES{$_} = 1 } @ALLOWED_TYPES;


pre_parse( $comp_file_opts_ref)
if (!@PARSED_REFS);

return cfo_parse();
}




sub pre_parse($)
{
my ($comp_file_opts_ref,	    # positional parameters
) = @_;






$FUNCTION_TEXT = ucfirst $FUNCTION;

map { $SRCTYPES{$_} = TARGET_get_primary_bld_type( undef, $_) } TARGET_get_src_types( $GBS_TARGET);
ENV_sig( EE => "No Builders defined for this Target '$GBS_TARGET'")
if (!%SRCTYPES);

my @comp_file_opts = @{$comp_file_opts_ref};





while (@comp_file_opts && is_option( $comp_file_opts[0]))
{
push @PRE_OPTIONS, shift @comp_file_opts;
}




my $component = ENV_getenv( 'GBS_COMPONENT');
my @files;
my @options;
foreach my $item (@comp_file_opts)
{

if (is_option( $item))
{
if ($FOR_MAKE)
{
push @PRE_OPTIONS, $item;
} else
{
push @options, $item;
}
} else
{
my $contains_colon = $item =~ /\:/;
my $plain_name = ($FOR_MAKE) ? $item !~ /[:.]/ : 0;     # no colon or dot
if (scalar @options || $contains_colon || $plain_name)
{
push @PARSED_REFS, [ $component, [@files], [@options] ]
if (@files);
@files = ();
@options = ();
}
if ($plain_name)	    # make only
{
push @PARSED_REFS, [ $item, undef, undef ]
} elsif ($contains_colon)
{
my ($this_comp, $files) = split ':', $item;
$component = $this_comp if ($this_comp ne '.' && $this_comp ne '');
if ($FOR_MAKE)
{
if ($files eq '' || $files eq '*.*' || $files eq '*')
{
push @PARSED_REFS, [ $component, undef, undef ]
} else
{
@files = split ',', $files;
}
} else
{
@files = split ',', $files;
@files = ('*.*') if (!@files);
}
} else
{

push @files, split ',', $item;
}
}
}

push @PARSED_REFS, [ $component, [@files], [@options] ]
if (@files);

}




sub cfo_parse()
{
my $nr_cfo = 0;

@ALLOWED_COMPS = ();
%COMPONENTS = ();




foreach my $ref (@PARSED_REFS)
{
my ($component, $files_ref, $options_ref) = @{$ref};
if (defined $files_ref)
{
store_component( $component, $files_ref, $options_ref);
} else
{
store_make_component( $component);	    # or $item
}
}




@CFO_REFS = ();
if ($FOR_MAKE)
{
select_make();
} else
{
select_gen_and_audit();
}
@TMP_CFO_REFS = ();







{
ENV_say( 1, "Selected:");
if (@CFO_REFS > 0)
{
my @row_refs;
push @row_refs, [ 0, "ALL: @PRE_OPTIONS" ]
if (@PRE_OPTIONS);
foreach my $ref (@CFO_REFS)
{
my ($sel_order, $component, $files_ref, $opts_ref) = @{$ref};

$COMPONENTS{$component} = 1;
push @row_refs, [ 0, "$sel_order:$component:" ];
push @row_refs, [ 3, "@{$files_ref}" ];
push @row_refs, [ 5, "(@{$opts_ref})" ]
if (@{$opts_ref});
}
ENV_say( 2, FORMAT_indent( -10, 0, @row_refs));
} else
{
ENV_say( 2, 'None');
}
}

$nr_cfo = @CFO_REFS;
return $nr_cfo;
}




sub store_component($$$)
{
my ($component,
$files_ref,
$options_ref) = @_;


my @file_refs;
foreach my $file (@{$files_ref})
{
if (GBSGLO_validate_spec( $file))
{
my ($name, $type) = ENV_split_spec_nt( $file);
$type = '.*' if ($type eq '');
if (validate_type( $file, $type))
{
push @file_refs, [ $file, $name, $type ];
}
} else
{
ENV_say( 1, "File '$component:$file' skipped");
}
}
my @components;
my $is_multi_component;
if (ENV_is_wildcard( $component))
{
@components = wild_components( $component);
$is_multi_component = 1;
} else
{
validate_component( $component);
@components = ($component);
$is_multi_component = 0;
}

my $select_files_func = ($FOR_MAKE && !$REVERSE_TYPE) ? \&select_target_files : \&select_src_files;
foreach my $comp (@components)
{
my $this_files_ref = &$select_files_func( $is_multi_component, $comp,
@file_refs);
push @TMP_CFO_REFS, [ $comp, $this_files_ref, $options_ref ];
}
}




sub store_make_component($)
{
my ($component) = @_;


$component = 'ALL' if ($component eq '*');

if ($component eq 'ALL')
{
push @TMP_CFO_REFS, [ $component, undef, undef ];
} else
{
if (ENV_is_wildcard( $component))
{
my @components = wild_components( $component);
map { push @TMP_CFO_REFS, [ $_, undef, undef ] } @components;
} else
{
validate_component( $component);
push @TMP_CFO_REFS, [ $component, undef, undef ];
}
}
}




sub wild_components($)
{
my ($wild_component) = @_;
my @components;

if (!@ALLOWED_COMPS)
{
if ($FOR_AUDIT)
{
@ALLOWED_COMPS = GBSGLO_components_for_audit( $GBS_SUBSYS, $CUR_GBS_AUDIT, $GBS_TARGET);
} else
{
@ALLOWED_COMPS = GBSGLO_components_for_target( $GBS_SUBSYS, $GBS_TARGET);
}
}
@components = ENV_wildcard( $wild_component, [ @ALLOWED_COMPS ]);

if (!@components)
{
if ($FOR_AUDIT)
{
ENV_sig( W => "No valid wild Component '$wild_component' for Audit '$CUR_GBS_AUDIT' and Target '$GBS_TARGET'");
} else
{
ENV_sig( W => "No valid Wild Component '$wild_component' Target '$GBS_TARGET'");
}
}

return @components;
}




sub select_src_files($$@)
{
my ($is_wild_component,
$component,
@generic_file_refs	     # [ $file, $name, $type ]
) = @_;
my @files;


my $src_dir = "$GBS_COMP_PATH/$component/src";
foreach my $generic_file_ref (@generic_file_refs)
{

my $generic_file = $generic_file_ref->[0];
if (ENV_is_wildcard( $generic_file))
{
push @files, get_wild_sources( $src_dir, $generic_file);
} else
{
if (-f "$src_dir/$generic_file")
{
push @files, $generic_file
if (validate_file( $src_dir, $generic_file));
} else
{
ENV_sig( EE => "No File '$generic_file' in Component '$component'")
if (!$is_wild_component);
}
}
}

if ($REVERSE_TYPE)
{
foreach my $file (@files)
{
my ($name,$type) = ENV_split_spec_nt( $file);
my $bld_type = $SRCTYPES{$type};
$file = "$name$bld_type";
}
}

return [ @files ];
}





sub select_target_files($$@)
{
my ($is_wild_component,
$component,
@generic_file_refs	     # [ $file, $name, $type ]
) = @_;
my @files;

my %srctypes;


my $src_dir = "$GBS_COMP_PATH/$component/src";
foreach my $generic_file_ref (@generic_file_refs)
{

my ($generic_file, $generic_name, $generic_type) = @{$generic_file_ref};
$generic_name = '*' if ($generic_name eq '');
$generic_type = '.*' if ($generic_type eq '');
$generic_file = "$generic_name$generic_type";
my @src_files = get_wild_sources( $src_dir, "$generic_name.*");
my $is_wild_file = ENV_is_wildcard( $generic_file);
if (@src_files)
{
if ($is_wild_file)
{
foreach my $src_filespec (@src_files)
{
my ($dir, $src_name, $src_type) = ENV_split_spec_dnt( $src_filespec);
my $bld_type = $SRCTYPES{$src_type};

if (defined $bld_type &&
ENV_wildcard( $generic_type, [ $bld_type ]))
{

push @files, "$src_name$bld_type";
}
}
} else
{
my $bld_file;
foreach my $src_file (@src_files)
{
my $src_type = ENV_split_spec_t( $src_file);
my $bld_type = $SRCTYPES{$src_type};
if (defined $bld_type &&
$bld_type eq $generic_type)
{
$bld_file = $generic_file;
last;
}
}
if (defined $bld_file)
{
push @files, $bld_file;
} else
{
ENV_sig( EE => "No src-file for $generic_file found")
if (!$is_wild_component);
}
}
} else
{
ENV_sig( EE => "No src-file for $generic_file found")
if ((!$is_wild_file) && (!$is_wild_component));
}
}

return [ @files ];
}




sub select_gen_and_audit()
{



my $sel_order = 1;
foreach my $order_ref (TARGET_get_rule_gen_order( $GBS_TARGET)) # per order

{
my %types;
map { $types{$_} = 1 } @{$order_ref};


my $file_count = 0;
foreach my $tmp_cfo_ref( @TMP_CFO_REFS)	#per component
{
my $component = $tmp_cfo_ref->[0];
my @files;


foreach my $file (@{$tmp_cfo_ref->[1]})
{
my $type = ENV_split_spec_t( $file);

if ($types{$type})
{
if ($FOR_AUDIT)
{
push @files, $file
if ($ALLOWED_TYPES{$type});
} else
{
push @files, $file;
}
}
}
if (@files > 0)
{
push @CFO_REFS, [ $sel_order, $component, [ @files ], $tmp_cfo_ref->[2] ];
$file_count += @files;
}
}
$sel_order++
if ($file_count > 0);
}
}




sub select_make()
{
foreach my $cfo_ref( @TMP_CFO_REFS)
{
my $component = $cfo_ref->[0];
my $files_ref = $cfo_ref->[1];

if (defined $files_ref)
{
push @CFO_REFS, [ 0, $component, $files_ref, [] ] if (@{$files_ref});
} else
{
push @CFO_REFS, [ 0, $component, [], [] ];
}
}
}




sub validate_component($)
{
my ($component) = @_;

ENV_sig( EE => "No such Component '$component'")
if (! GBSGLO_component_exists( $GBS_ROOT_PATH, $GBS_SUBSYS, $component));

if ($FOR_AUDIT)
{
ENV_sig( EE => "Component '$component' does not Audit '$CUR_GBS_AUDIT' for Target '$GBS_TARGET'")
if (! GBSGLO_component_does_audit( $GBS_SUBSYS, $component, $CUR_GBS_AUDIT, $GBS_TARGET));
} else
{
ENV_sig( EE => "Component '$component' does not generate for Target '$GBS_TARGET'")
if (! GBSGLO_component_does_build( $GBS_SUBSYS, $component, $GBS_TARGET));
}
}




sub validate_file($$)
{
my ($src_dir,
$file,
) = @_;
my $file_ok = 1;

if ($file =~ /\s/)
{
my $component = ENV_parent_dir( $src_dir, -2);
ENV_sig( W => "File: '$component:$file': No spaces allowed. File Skipped");
$file_ok = 0;
}

return $file_ok;
}




sub validate_type($$)
{
my ($file,
$type,
) = @_;
my $is_ok = 1;


if (!exists $ALLOWED_TYPES{$type})
{
if (ENV_is_wildcard( $type) &&
ENV_wildcard( $type, [@ALLOWED_TYPES]))
{

$ALLOWED_TYPES{$type} = 1;
} else
{
$is_ok = 0;
if ($FOR_AUDIT)
{
ENV_sig( W => "File '$file':", "Type '$type' not allowed in this $FUNCTION_TEXT context ($CUR_GBS_AUDIT) for Target '$GBS_TARGET'");
} else
{
ENV_sig( W => "File '$file':", "Type '$type' not allowed in this $FUNCTION_TEXT context for Target '$GBS_TARGET'");
}
}
}

return $is_ok;
}




sub get_wild_sources($$)
{
my ($src_dir,
$wild_file,
) = @_;
my @files;


my $src_dir_l = length( $src_dir) + 1;
foreach my $filespec (GBSGLO_glob( "$src_dir/$wild_file"))
{
my $file = substr( $filespec, $src_dir_l);
push @files, $file
if (validate_file( $src_dir, $file));
}

return @files;
}




sub CFO_pre_options()
{
return @PRE_OPTIONS;
}







sub CFO_get_item($)
{
my ($index) = @_;


return @{$CFO_REFS[$index]};
}




sub CFO_get_components()
{
return (wantarray) ? sort( keys %COMPONENTS ) : join( ',', keys %COMPONENTS);
}




sub is_option($)
{
my ($arg) = @_;
my $is_option;

$is_option = ENV_is_option( $arg) || $arg =~ /^["']/ || $arg =~ /["']$/;

return $is_option;
}

1;

