#===============================================================
#
#   gbsenv.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
package gbsenv;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
GBSENV_init
GBSENV_get_exec_id
GBSENV_get_exec_level
GBSENV_scripts_path
GBSENV_set_gbs_root_path
GBSENV_gbs_root_path
GBSENV_setenv
GBSENV_changed_setenv_commands
GBSENV_write_result_script
GBSENV_mode_is_interactive
GBSENV_mode_is_background
GBSENV_mode_is_foreground
GBSENV_is_integrator
GBSENV_is_administrator
GBSENV_gui_available
GBSENV_is_predefined_env
GBSENV_get_custom_envs
GBSENV_get_envs_to_reset
GBSENV_get_plugin_data
);
}




use glo::env;
use glo::spit;
use glo::shell;




sub GBSENV_init();
sub GBSENV_get_exec_id();
sub GBSENV_get_exec_level();
sub GBSENV_scripts_path();
sub GBSENV_set_gbs_root_path($);
sub GBSENV_gbs_root_path();
sub GBSENV_setenv($$);
sub GBSENV_changed_setenv_commands($@);
sub GBSENV_write_result_script($$);
sub GBSENV_mode_is_interactive();
sub GBSENV_mode_is_foreground();
sub GBSENV_mode_is_background();
sub GBSENV_is_integrator();
sub GBSENV_is_administrator();
sub GBSENV_gui_available();
sub GBSENV_is_predefined_env($);
sub GBSENV_get_custom_envs();
sub GBSENV_get_envs_to_reset();
sub GBSENV_get_plugin_data($);




my $GBS_BASE_PATH; 	    # Do not inilialize!

my $GBS_EXEC_ID; 	    # Do not inilialize!
my $GBS_EXEC_LEVEL; 	    # Do not inilialize!

my $GBS_ROOT_PATH; 	    # Do not inilialize!

my $IS_WIN32 = ENV_is_win32();
my $GBS_EXEC_MODE = ENV_getenv( 'GBS_EXEC_MODE');

my $IS_INTEGRATOR;	    # Do not initialize!
my $IS_ADMINISTRATOR;	    # Do not initialize!
my $GUI_AVAILABLE;	    # Do not initialize!




my @BASIC_ENVS = qw(
GBS_ROOT_PATH
GBS_SUBSYS
GBS_COMPONENT
GBS_TARGET
GBS_TARGETS
GBS_ALL_TARGETS
GBS_AUDIT
GBS_AUDITS
GBS_ALL_AUDITS
GBS_TOOL
GBS_TOOLS
GBS_ALL_TOOLS
);





my %PREDEFINED_ENVS = map { $_ => 1} qw(
GBS_PERL_PATH
GBS_XTITLE
GBS_BASE_PATH
);







my @CUSTOM_ENVS = qw(
GBS_TEMPLATES_PATH
GBS_MAKE
GBS_SUBMIT
GBS_SUBWIN
GBS_BATCH
GBS_BG_NOTIFIER
);






my @USER_ENVS = qw(
GBS_DEBUGGER
GBS_MAP
GBS_MODE
GBS_OPT
);





sub BEGIN
{








$GBS_EXEC_ID = ENV_getenv( 'GBS_EXEC_ID');
$GBS_EXEC_LEVEL = ENV_getenv( 'GBS_EXEC_LEVEL');
if ($GBS_EXEC_ID eq '')
{



$GBS_EXEC_ID = join( '_', ENV_get_hostname(), time, $$);
$ENV{GBS_EXEC_ID} = $GBS_EXEC_ID;
$GBS_EXEC_LEVEL = -1;
if (ENV_is_interactive && !ENV_getenv( 'GBS_PROMPT_SEPARATOR_PRINTED'))
{
ENV_say( 0, '=' x (ENV_get_term_size() - 5));
ENV_setenv( GBS_PROMPT_SEPARATOR_PRINTED => 1);
}
}
$ENV{GBS_EXEC_LEVEL} = ++$GBS_EXEC_LEVEL;






$GBS_BASE_PATH = ENV_getenv_perl_path( 'GBS_BASE_PATH');
if ($GBS_BASE_PATH eq '')
{
$GBS_BASE_PATH = ENV_perl_path_nospace( ENV_get_user_path( 1)) . '/.gbs';
ENV_setenv( GBS_BASE_PATH => $GBS_BASE_PATH);
}





if (!-e $GBS_BASE_PATH)
{
ENV_say( 1, "Creating $GBS_BASE_PATH...");
ENV_mkdir( $GBS_BASE_PATH);	    # Fails if not present
ENV_hide_file( $GBS_BASE_PATH, 1);
}




ENV_set_application( 'GBS', 'Generic Build Support', '<GBS_SCRIPTS_PATH>', '<GBS_HOME>', $GBS_BASE_PATH, '<GBS_PERL_CMD>');





map { GBSENV_setenv( $_ => '')
if (!defined $ENV{$_}) } @BASIC_ENVS;
$GBS_ROOT_PATH = ENV_getenv_perl_path( 'GBS_ROOT_PATH');
}





sub GBSENV_init()
{



{
my $must_beep;
my $gbs_beeps = ENV_getenv( 'GBS_BEEPS');
if ($gbs_beeps eq '' || $gbs_beeps eq 'YES' || $gbs_beeps eq 'ON')
{
$must_beep = 1;
} elsif ($gbs_beeps eq 'NO' || $gbs_beeps eq 'OFF')
{
$must_beep = 0;
} else
{
ENV_sig( E => "Invalid value for 'GBS_BEEPS' ($gbs_beeps).",
"Must be one of (YES, NO, ON, OFF)");
$must_beep = 1;
}
ENV_enable_beeps( $must_beep);
}
}





sub GBSENV_get_exec_id()
{
return $GBS_EXEC_ID;
}





sub GBSENV_get_exec_level()
{
return $GBS_EXEC_LEVEL;
}






sub GBSENV_scripts_path()
{
return ENV_get_application_root_path();
}






sub GBSENV_set_gbs_root_path($)
{
my ($new_root_path,
) = @_;

$GBS_ROOT_PATH = $new_root_path;
}






sub GBSENV_gbs_root_path()
{
return $GBS_ROOT_PATH;
}





sub GBSENV_setenv($$)
{
my ($name,
$value_or_ref,
) = @_;
my $value;

if ($name =~ /^GBS_.+_(PATH|ROOT)$/ || $name eq 'GBS_HOME')
{
$value = ENV_setenv_os_path( $name, $value_or_ref);
} else
{
$value = ENV_setenv( $name => $value_or_ref);
}


return $value;
}





sub GBSENV_changed_setenv_commands($@)
{
my ($must_log,
@specific_env_vars,
) = @_;
my @commands;

@specific_env_vars = ENV_get_changed_envs()
if (!@specific_env_vars);
foreach my $name (@specific_env_vars)
{
my $value = $ENV{$name};	# if PATH: already in OS format because set by ENV_setenv_os_path

push @commands, SHELL_setenv( $name, $value);
ENV_say( 0, "  $name => $value")
if ($must_log);
}

return @commands;
}







sub GBSENV_write_result_script($$)
{
my ($lines_ref,
$must_print,	# Debugging
) = @_;

my $main_name = ENV_get_main_name();	    # is uppercase
$main_name =~ s/^GBS//;			    # remove possible leading 'gbs'
my $envvar_name = "GBS_${main_name}_FILE";	    # e.g.: GBS_SWR_FILE

my $filespec = ENV_getenv( $envvar_name);	    # e.g.: .../GBSSWR_RESULT_RANDY9_5756.bat
if ($filespec eq '')
{
ENV_sig( F => "EnvVar $envvar_name is not defined");
} elsif ($filespec eq 'NONE')
{

if ($must_print)
{
my @lines = map { "NONE: $_\n" } @{$lines_ref};
ENV_print( 0, @lines);
}
} else
{
my @lines = map { "$_\n" } @{$lines_ref};
ENV_print( 0, @lines)
if ($must_print);
SPIT_script_file( $filespec, \@lines);
}
}









sub GBSENV_mode_is_interactive()
{
return ($ENV{GBS_EXEC_MODE} eq 'INTERACTIVE') ? 1 : 0;
}





sub GBSENV_mode_is_foreground()
{
return ($ENV{GBS_EXEC_MODE} eq 'FOREGROUND') ? 1 : 0;
}





sub GBSENV_mode_is_background()
{
return ($ENV{GBS_EXEC_MODE} eq 'BACKGROUND') ? 1 : 0;
}





sub GBSENV_is_integrator()
{
if (!defined $IS_INTEGRATOR)
{
my @gbs_integrator = ENV_getenv( 'GBS_INTEGRATOR');
if (@gbs_integrator)
{
my $gbs_system_name = ENV_getenv( 'GBS_SYSTEM_NAME');
$IS_INTEGRATOR = grep( ENV_wildcard( $_, $gbs_system_name), @gbs_integrator) ? 1 : 0;
} else
{
$IS_INTEGRATOR = 0;
}
}

return $IS_INTEGRATOR;
}





sub GBSENV_is_administrator()
{
if (!defined $IS_ADMINISTRATOR)
{
my @gbs_administrator = ENV_getenv( 'GBS_ADMINISTRATOR');
if (@gbs_administrator)
{
my $gbs_system_name = ENV_getenv( 'GBS_SYSTEM_NAME');
$IS_ADMINISTRATOR = grep( ENV_wildcard( $_, $gbs_system_name), @gbs_administrator) ? 1 : 0;
} else
{
$IS_ADMINISTRATOR = 0;
}
}

return $IS_ADMINISTRATOR;
}





sub GBSENV_gui_available()
{
if (!defined $GUI_AVAILABLE)
{
$GUI_AVAILABLE = (ENV_getenv( 'GBS_GUI') eq 'Tkx') ? 1 : 0;
}
return $GUI_AVAILABLE;
}




sub GBSENV_is_predefined_env($)
{
my ($env) = @_;

return (exists( $PREDEFINED_ENVS{$env}) ? 1 : 0);
}




sub GBSENV_get_custom_envs()
{
return @CUSTOM_ENVS;
}




sub GBSENV_get_envs_to_reset()
{
my @env_names;




push @env_names, grep( /^GBSEXT_/, keys( %ENV));




push @env_names, grep( ENV_getenv( $_) ne '', @CUSTOM_ENVS);





foreach my $env_name (keys %ENV)
{
if (substr( $env_name, 0, 10) eq 'GBS_FLAGS_' ||
grep $env_name eq $_, @USER_ENVS)
{
my $old_value = $ENV{$env_name};
if ($old_value ne '' && $old_value ne '""')
{
ENV_say( 1, "$env_name: '$old_value'=>''");
push @env_names, $env_name;
}
}
}


return @env_names;
}




sub GBSENV_get_plugin_data($)
{
my ($plugin_name) = @_;
my ($plugin_rel, $plugin_path) = ();

my $plugin_env_name = uc $plugin_name;
my @errors;
$plugin_rel = ENV_getenv( "GBSEXT_${plugin_env_name}_REL");
push @errors, "- 'GBSEXT_${plugin_env_name}_REL' not defined in switch.gbs"
if ($plugin_rel eq '');
$plugin_path = ENV_getenv_perl_path( "GBSEXT_${plugin_env_name}_PATH");
push @errors, "- 'GBSEXT_${plugin_env_name}_PATH' not defined in switch.gbs"
if ($plugin_path eq '');
if (@errors)
{
ENV_sig( E => "Plugin: $plugin_name:", @errors);
} else
{
ENV_sig( E => "Plugin path Directory '$plugin_path' (GBSEXT_${plugin_env_name}_PATH) does not exist")
if (! -e $plugin_path);
}

if (wantarray)
{
return ($plugin_rel, $plugin_path);
} else
{
return $plugin_rel;
}

}
1;


