#==============================================================
#
#   gbsfilecom.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
package gbsfilecom;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
GBSFILECOM_validate_path

GBSFILECOM_expand_command_data_refs
GBSFILECOM_exec
);
}




use glo::env;
use glo::slurp;
use gbsfile;




sub GBSFILECOM_validate_path($);

sub GBSFILECOM_expand_command_data_refs($$$);
sub GBSFILECOM_exec($$);

sub handle_ok_values($$);









my $IS_WIN32 = ENV_is_win32();
my $VERBOSE = ENV_getenv( 'GBS_VERBOSE');








sub GBSFILECOM_validate_path($)
{
my ($path) = @_;
my $full_path;

if (ENV_is_abs_path( $path))
{
my $abs_path_not_allowed = 1;
if (!$IS_WIN32)	# IS_LINUX
{
$abs_path_not_allowed = 0
if (substr( $path, 0, 5) eq '/usr/' ||
$path eq '/lib');
}
GBSFILE_sig( E => "Absolute path not allowed, use EnvVar from switch.gbs")
if ($abs_path_not_allowed);
}

$full_path = ENV_perl_path_nospace( ENV_expand_envs( $path));
GBSFILE_sig( E => "No such directory:", "'$full_path'")
if (!-d $full_path && substr( $full_path, 0, 5) ne '/usr/');
GBSFILE_sig( E => "Must be derrived Abs Path:", "'$full_path'")
if (!ENV_is_abs_path_perl( $full_path));


return $full_path;
}














sub GBSFILECOM_expand_command_data_refs($$$)
{
my ($command_data_refs_ref,

$pos_args_ref,	    # Target:  [ $file_name, $in_filespec, $out_filespec, $out_file_dir, $out_filetype ]



$gen_args_ref,	    # Flags from flags_*.gbs files + @command_line_flags
) = @_;
my @command_data_refs;

my @pos_args = (defined $pos_args_ref) ? map { ENV_os_path( $_) } @{$pos_args_ref} : ();
my $gen_args = (defined $gen_args_ref) ? ENV_join_quoted_space( @{$gen_args_ref}) : '';


foreach my $command_data_ref (@{$command_data_refs_ref})
{
my ($command_type, $pos_arg_tokens, $command_line, $ok_values, $exec_condition, $script_file) = @{$command_data_ref};


if ($pos_arg_tokens ne '0')
{
my $pos_par_token = substr( $pos_arg_tokens, 1);	    # % or \$




$command_line =~ s/$pos_par_token\*/$gen_args/g;




for (my $i = 1; $i <= @pos_args; $i++)
{
$command_line =~ s/$pos_par_token$i/$pos_args[$i-1]/g;
$exec_condition =~ s/$pos_par_token$i/$pos_args[$i-1]/g;
}
} else
{
if ($command_type ne 'B')	    # direct builtin (e.g. echo)
{
$command_line = "$command_line @pos_args"	    # join quoted string???
if (@pos_args);
$command_line = "$command_line $gen_args"	    # join quoted string???
if ($gen_args ne '');
}
}
push @command_data_refs, [ $command_type, $pos_arg_tokens, $command_line, $ok_values, $exec_condition, $script_file ];

}

return @command_data_refs;
}




sub GBSFILECOM_exec($$)
{
my ($must_print_commands,	    # scripts should print their own command(s). '-' == undef
$command_data_refs_ref,	    # [ [ $command_type, $pos_arg_tokens, $command_line, $ok_values, $exec_condition, $script_file ], ... ]

) = @_;
my $rc = 0;

my $nr_commands = @{$command_data_refs_ref};
my $count = 0;
my $tmp_spec;
if (!$VERBOSE)
{
$tmp_spec = ENV_get_tmp_spec( 'exec.tmp');
}
foreach my $ref (@{$command_data_refs_ref})
{


$count++;
ENV_whisper( 0, '.')
if ($count != 1);
my ($command_type, $pos_arg_tokens, $command_line, $ok_values, $exec_condition, $script_file) = @{$ref};





my $must_exec = 1;
if ($exec_condition ne '-')
{
my $type = substr( $exec_condition, 0, 1);	# F or V
my $cond = substr( $exec_condition, 1, 1);	# 0 or 1
if ($type eq 'F')
{
my $filespec = substr( $exec_condition, 2);
$filespec = ENV_expand_envs( $filespec);
my $exists = (-e $filespec) ? 1 : 0;

$must_exec = ($cond == $exists);
$exec_condition = "File exists ($filespec != $cond)";
} elsif ($type eq 'V')
{
my ($name, $value) = split( / /, substr( $exec_condition, 2), 2);
my $actual_value = ENV_getenv( $name);
$actual_value = ENV_expand_envs( $actual_value);
my $equal = ($actual_value eq $value) ? 1 : 0;

$must_exec = ($cond == $equal);
$exec_condition = "EnvVar ($name==$value) != $cond";
} else
{
ENV_sig( F => "Invalid exec-condition type ($type)");
}
}
if ($must_exec)
{



if ($command_type eq 'B')	# direct builtin (e.g. echo)
{



my ($command, $args) = split( / /, $command_line, 2);
if ($command eq 'ECHO')
{
ENV_say( 0, $args);
} elsif ($command eq 'SET')
{
my ($name, $value) = split( / /, $args, 2);

$value = ENV_expand_envs( $value);
ENV_setenv( $name => $value);
} else
{
ENV_sig( F => 'Invalid Special Command',
"'$command_line'");
}
} elsif ($command_type eq 'D' || $command_type eq 'S')  # direct or script
{



my $full_command = ENV_expand_envs( $command_line);
my $say_command_line = ($nr_commands == 1) ? $full_command : "$count:$full_command";
my $rc_ok;
if ($VERBOSE)
{
if ($must_print_commands eq '1' || ($must_print_commands eq '-' && $command_type eq 'D'))
{
ENV_say( 0, $say_command_line);
}
$rc = ENV_system( $full_command, undef, undef);





$rc_ok = handle_ok_values( $rc, $ok_values);
} else
{
$rc = ENV_system( $full_command, undef, $tmp_spec);





$rc_ok = handle_ok_values( $rc, $ok_values);
if (!$rc_ok)
{
if ($must_print_commands eq '1' || ($must_print_commands eq '-' && $command_type eq 'D'))
{
ENV_say( 0, $say_command_line);
}
}
ENV_print( 0, SLURP_file_raw( $tmp_spec));
unlink $tmp_spec;
}

if ($rc_ok)
{
$rc = 0;
} else
{
last;
}
} else
{
ENV_sig( F => "Invalid command_type '$command_type'");
}
} else
{
my $text = "Skipped because of pre-condition: $exec_condition";
if ($nr_commands == 1)
{
ENV_say( 0, $text);
} else
{
ENV_say( 0, "$count:$text");
}
}
}

return $rc;
}




sub handle_ok_values($$)
{
my ($rc,
$ok_values,	# space-separated ok_values (string)
) = @_;
my $rc_ok;

if ($ok_values eq '0')
{
$rc_ok = ($rc == 0)
} else
{
my @ok_values = split( ' ', $ok_values);
$rc_ok = grep( $_ == $rc, @ok_values);
}

return $rc_ok;
}

1;


