#==============================================================
#
#   gbsinit.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
package gbsinit;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
GBSINIT_setup
GBSINIT_validate
);
}




use glo::env;
use glo::shell;
use glo::build;
use glo::check;
use gbsenv;
use gbscmd;




sub GBSINIT_setup($);
sub GBSINIT_validate();




my $GBS_BASE_PATH = ENV_getenv_perl_path( 'GBS_BASE_PATH');
ENV_sig( F => "GBS_BASE_PATH not defined")
if ($GBS_BASE_PATH eq '');

my $IS_WIN32 = ENV_is_win32();
my $IS_DEVELOPMENT = ENV_is_development();





sub GBSINIT_setup($)
{
my ($wanted_rel,	    # '' or wanted rel
) = @_;
my @lines;

if ($wanted_rel eq '')
{
ENV_say( 1, "Initializing GBS...");
} else
{
ENV_say( 1, "Initializing GBS [$wanted_rel]...");
}




foreach my $env (qw( GBS_SCRIPTS_PATH GBS_SCRIPTS_REL GBS_SCRIPTS_PATH))
{
ENV_sig( F => "'$env' not defined")
if (ENV_getenv( $env) eq '');
}




my $gbs_scripts_path;
if ($wanted_rel ne '')
{



my $gbs_scripts_root = ENV_getenv_perl_path( 'GBS_SCRIPTS_ROOT');
my $new_gbs_scripts_path = "$gbs_scripts_root/$wanted_rel";
ENV_sig( EE => "No such GBS_SCRIPTS_REL ($wanted_rel) in $gbs_scripts_root")
if (!-d $new_gbs_scripts_path);
$gbs_scripts_path = $new_gbs_scripts_path;
ENV_setenv( GBS_SCRIPTS_REL => $wanted_rel);
ENV_set_application( undef, undef, $gbs_scripts_path, undef, undef, undef);   # only update scripts
if ($new_gbs_scripts_path eq $gbs_scripts_path)
{
ENV_say( 1, "GBS_SCRIPTS_REL '$wanted_rel' was already current")
unless ($wanted_rel eq 'beta');
} else
{
ENV_say( 1, "GBS_SCRIPTS_REL set to '$wanted_rel'");
}
} else
{
$gbs_scripts_path = GBSENV_scripts_path();
}
ENV_setenv_os_path( GBS_SCRIPTS_PATH => ENV_delink_spec( $gbs_scripts_path));
GBSCMD_scripts_path_changed();

my $gbs_version = BUILD_get_version();
my $gbs_build = BUILD_version_2_build( $gbs_version, BUILD_get_version_date());
ENV_setenv( GBS_VERSION => $gbs_version);
ENV_setenv( GBS_BUILD => $gbs_build);






my $gbs_home = ENV_getenv( 'GBS_HOME');
if ($gbs_home eq '')
{
$gbs_home = ENV_get_application_home_path();
ENV_setenv_os_path( GBS_HOME => $gbs_home);

}




ENV_setenv( GBS_PLATFORM => $^O);    # Mswin32 or linux
ENV_setenv( GBS_SHELL_FILETYPE => ENV_shell_filetype());

ENV_setenv_os_path( GBS_ROOT_PATH => '');
ENV_setenv( GBS_SUBSYS => '');
ENV_setenv( GBS_COMPONENT => '');
ENV_setenv( GBS_TARGET => '');
ENV_setenv( GBS_AUDIT => '');
ENV_setenv( GBS_TOOL => '');

ENV_setenv( GBS_PID => '$$')
if (ENV_is_lunix());








{
my $site_gbsall_file = ENV_command_filespec( "$gbs_scripts_path/../gbsall");
if (-f $site_gbsall_file)
{
push @lines, SHELL_echo( 'Executing <GBS_SCRIPTS_ROOT>/gbsall..');
push @lines, SHELL_source( $site_gbsall_file);
}

my $base_gbsall_file = ENV_command_filespec( "$GBS_BASE_PATH/gbsall");
if (-f $base_gbsall_file)
{
push @lines, SHELL_echo( 'Executing <GBS_BASE_PATH>/gbsall..');
push @lines, SHELL_source( $base_gbsall_file);
}
}








my $gbs_exec_mode = ENV_getenv( 'GBS_EXEC_MODE');
if ($gbs_exec_mode eq '')
{
$gbs_exec_mode = ENV_setenv( GBS_EXEC_MODE => 'INTERACTIVE');	# initial
}






if ($gbs_exec_mode eq 'INTERACTIVE')
{
eval { require Tkx; };
if ($@)
{

ENV_say( 1, "Perl Package 'Tkx' not found");
ENV_setenv( GBS_GUI => 'None');
} else
{
ENV_setenv( GBS_GUI => 'Tkx');
}
} else
{
ENV_setenv( GBS_GUI => 'None');
}




if ($IS_WIN32 && $IS_DEVELOPMENT)
{
my $path = ENV_getenv( 'PATH');
my $scm_path = ENV_os_path( "$gbs_scripts_path/STUBS/SCM/bin");
ENV_setenv( PATH => "$path;$scm_path")
if ( $path !~ quotemeta( ";$scm_path" ));
}




push @lines, GBSENV_changed_setenv_commands( 0);



if ($gbs_exec_mode eq 'INTERACTIVE')
{



if ($IS_WIN32)
{

push @lines, "doskey /insert /macrofile=%GBS_SCRIPTS_PATH%\\gbsdef.doskey";
push @lines, "doskey /insert /macrofile=%GBS_SCRIPTS_PATH%\\gbsdef2.doskey"
if ($IS_DEVELOPMENT);
} else
{

push @lines, SHELL_source( "$gbs_scripts_path/gbsdef.sh");
push @lines, SHELL_source( "$gbs_scripts_path/gbsdef2.sh")
if ($IS_DEVELOPMENT);
}
}


return @lines;
}





sub GBSINIT_validate()
{



{



my $gbs_log_root = ENV_getenv_perl_path( 'GBS_LOG_ROOT');
my $error_text = CHECK_path( 'Path', $gbs_log_root, 1);
if ($error_text eq '')
{
$error_text = "Path $gbs_log_root is not writable!"
if (! -w $gbs_log_root);
}
if ($error_text ne '')
{
ENV_sig( W => "GBS_LOG_ROOT ($gbs_log_root)", "- $error_text");
}




my $gbs_site = ENV_getenv_perl_path( 'GBS_SITE');
ENV_sig( W => "'GBS_SITE' not set")
if ($gbs_site eq '');
}





ENV_setenv (GBS_LOG_PATH => '');
}

1;


