#===============================================================
#
#   gbsjob.pl
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
use strict;
use warnings;
use integer;
print "*GBSJOB @ARGV\n" if ($ENV{GBSDEBUG_FILE});




use lib $ENV{GBS_SCRIPTS_PATH};
use glo::env;
use glo::args;
use glo::time;
use glo::shell;
use glo::slurp;
use glo::spit;
use glo::banner;
use glo::build;
use gbsenv;
use gbsres;
use bgnotifier;
use notify;
use logsum;




sub do_job();
sub finish($);
sub trap_sig($);
sub write_summary($$);
sub cpu_usage();
sub read_job_file();
sub get_command_args();








my $JOB_FILE;




my (
$SUBMODE,			    # Fixed: 'SUBWIN' or 'SUBMIT'
$JOBNAME,			    # Fixed: gbssysgen, gbssysmake, gbssysaudit, gbssystool
$TARGET_OR_TOOL,		    # Vary
$AUDIT,			    # Vary: may be '-'
$DATE_TIME,			    # Fixed
$OS_LOGFILE_SPEC,		    # Vary: '-' or logfile
$MUST_DISPLAY_NOTIFY_MSG,	    # Fixed
$OS_ROOT_PATH,		    # Fixed
$EXEC,			    # Vary. E.g.: gbssysgen begin bart superglo non_gbs copy_export --files=*:*.*
$DISPLAY,			    # Fixed
$COMMAND_ARGS,		    # Fixed E.g.: --targets=test,mingw*
$COMMENT,			    # Fixed
);
my $JOB;




my $TIME_START;
my $TIME_END;
my $TIME_DIFF;

my @TIMER_START;    # ($clock, $user, $system, $cuser, $csystem) in seconds
my @TIMER_END;	    # ($clock, $user, $system, $cuser, $csystem) in seconds
my @TIMER_DIFF;     # ($clock, $user, $system, $all)

my $UNAME;
{
my ($os, $host, @rest) = ENV_uname();
$UNAME = "$host ($os [@rest])";
}

my $TITLE = GBSRES_title();
my ($LICS, $LICN, $LICE) = BUILD_get_lic();






$| = 1;
my $RC = 0;
GBSENV_init();
ENV_set_verbose(1);

get_command_args();

read_job_file();
$JOB = "$JOBNAME ($TARGET_OR_TOOL / $AUDIT)";

ENV_system( SHELL_xterm_follow_up(), 0, undef)
if ($SUBMODE eq 'SUBWIN');




$RC = do_job();




ENV_exit( $RC);




sub END
{
ENV_print_end_msg( 1);

}




sub do_job()
{
my $rc = 0;

@TIMER_START = ( time, times );
$TIME_START = TIME_time2num( $TIMER_START[0]);
my $gbs_show_version = BUILD_get_show_version();

BANNER_print( 'JOB', [
[ '',         $TITLE ],
undef,
[ 'VERSION:', "$gbs_show_version" ],
[ 'LICENSE:', "$LICN ($LICS/$LICE)" ],
[ 'START:',   $TIME_START ],
[ 'JOB:',     $JOB ],
[ 'PID:',     $$ ],
[ 'HOST:',    $UNAME ],
[ 'ROOT:',    $OS_ROOT_PATH ],
[ 'EXEC:',    $EXEC ],
[ 'LOGFILE:', $OS_LOGFILE_SPEC ],
[ 'DISPLAY:', $DISPLAY ],
[ 'ARGS:',    $COMMAND_ARGS ],
[ 'COMMENT:', $COMMENT ],
] );

if ($OS_LOGFILE_SPEC eq '-')
{
$ENV{GBS_EXEC_MODE} = 'FOREGROUND';
} else
{
$ENV{GBS_EXEC_MODE} = 'BACKGROUND';
BGNOTIFIER_start( $JOBNAME, $DATE_TIME, $TARGET_OR_TOOL, $AUDIT, $OS_LOGFILE_SPEC);
}
$ENV{GBS_LOGFILE} = $OS_LOGFILE_SPEC;
$ENV{DISPLAY} = $DISPLAY;
$ENV{GBS_PID} = $$;
$ENV{GBS_EXEC_ID} = '';
$ENV{GBS_JOB_ARGS} = $COMMAND_ARGS;

use sigtrap 'handler', \&trap_sig, 'untrapped';

my $exec = $EXEC;
if (ENV_is_win32())
{
$exec =~ s/ /\$_/g;
} else
{
$exec =~ s/ /%20/g;
}

ENV_chdir( $OS_ROOT_PATH);
my @command_args = ( '!!', '-' );		# Root == current, Subsys == None
if ($JOBNAME =~ /audit/)
{
push @command_args, ("--target=$TARGET_OR_TOOL", "--audit=$AUDIT", "--tool=-");
} elsif ($JOBNAME =~ /tool/)
{
push @command_args, ("--target=-", "--audit=-", "--tool=$TARGET_OR_TOOL");
} else  # gen or make
{
push @command_args, ("--target=$TARGET_OR_TOOL", "--audit=-", "--tool=-");
}
push @command_args, "--exec=$exec";
my $command_ref = ENV_command_ref( 'gbsstart.pl', [ @command_args ]);
my $command = ENV_command_line( $command_ref);

$rc = ENV_system( $command, undef, undef);


finish( $rc);
unlink( $JOB_FILE);

return $rc;
}




sub finish($)
{
my ($rc) = @_;

ENV_say( 0,  "-");

my $state;
if ($rc == -1)
{
ENV_say( 0, "*******************",
"*** JOB KILLED  ***",
"*******************");
$state = 'KILLED';
} elsif ($rc == 0)
{
$state = 'NORMAL';
} else
{
$state = 'FAILED';
}

@TIMER_END = ( time, times );
$TIME_END = TIME_time2num( $TIMER_END[0]);
my $cpu_usage = cpu_usage();		# sets $TIME_DIFF and @TIMER_DIFF
my $display = ENV_getenv( 'DISPLAY');

if ($OS_LOGFILE_SPEC ne '-')
{
write_summary( $state, $rc);

BGNOTIFIER_end( $state, $rc);
}

if ($MUST_DISPLAY_NOTIFY_MSG)
{
NOTIFY_submit( $JOB, $rc, $OS_LOGFILE_SPEC, $state);
}

ENV_say( 0,  "=");
BANNER_print( 'JOB', [
[ '',         $TITLE ],
undef,
[ 'LICENSE:', "$LICN ($LICS/$LICE)" ],
[ 'START:',   $TIME_START ],
[ 'END:',     $TIME_END ],
[ 'ELAPSED:', $TIME_DIFF ],
[ 'JOB:',     $JOB ],
[ 'ROOT:',    $OS_ROOT_PATH ],
[ 'EXEC:',    $EXEC ],
[ 'LOGFILE:', $OS_LOGFILE_SPEC ],
[ 'USAGE:',   $cpu_usage ],
[ 'STATUS:',  "$state ($rc)" ],
[ 'DISPLAY:', $display ],
[ 'ARGS:',    $COMMAND_ARGS ],
[ 'COMMENT:', $COMMENT ],
] );

if ($SUBMODE eq 'SUBWIN')
{
ENV_say( 1, "Hit <enter> key to exit this window: ");
<STDIN>;
}
}




sub trap_sig($)
{
my ($sig) = @_;

ENV_say( 1, "Signal: $sig");

finish( -1);
ENV_exit( 8);
}




sub write_summary($$)
{
my ($state,
$rc,
) = @_;




my @summary_items = (
$JOBNAME,
$TARGET_OR_TOOL,
$UNAME,
$OS_ROOT_PATH,
$OS_LOGFILE_SPEC,
$TIME_START,
$TIME_END,
$TIME_DIFF,
"@TIMER_DIFF",
$state,
$rc,
$AUDIT,
$DATE_TIME,
$COMMAND_ARGS,
$COMMENT,
);
my $sumline = LOGSUM_write( "$OS_LOGFILE_SPEC.sum", [ @summary_items ]);




my $gbssum_path = "$OS_ROOT_PATH/silo/.gbs/gbssum";
SPIT_append_file_nl( "$gbssum_path/gbssyssum.log", $sumline);	    #append

}




sub cpu_usage()
{
my $text;
no integer;

my @diff;
for ( my $i = 0; $i < 5; $i++)
{
$diff[$i] = $TIMER_END[$i] - $TIMER_START[$i];
}





my ($c, $pu, $ps, $cu, $cs) = @diff;
my $cpu_s = $ps + $cs;
my $cpu_u = $pu + $cu;
my $cpu_a = $cpu_s + $cpu_u;

$TIME_DIFF = TIME_deltatime2num( $c);
$text = sprintf "Clock: %s, System: %.2f, User: %.2f, Total: %.2f",
"$c ($TIME_DIFF)", $cpu_s, $cpu_u, $cpu_a;
@TIMER_DIFF = ($c, $cpu_s, $cpu_u, $cpu_a);

use integer;
return $text;
}




sub read_job_file()
{
(
$SUBMODE,
$JOBNAME,
$TARGET_OR_TOOL,
$AUDIT,
$DATE_TIME,
$OS_LOGFILE_SPEC,
$MUST_DISPLAY_NOTIFY_MSG,
$OS_ROOT_PATH,
$EXEC,
$DISPLAY,
$COMMAND_ARGS,
$COMMENT,
) = SLURP_file( $JOB_FILE);

ENV_sig( F => "Invalid job-file '$JOB_FILE'")
if (!defined $EXEC);
ENV_sig( F => "Invalid mode '$SUBMODE'")
if ($SUBMODE ne 'SUBMIT' && $SUBMODE ne 'SUBWIN');
}




sub get_command_args()
{


($JOB_FILE,
) = ARGS_get( 1, 'perl -w gbsjob.pl <job-file>', undef);
}


