#===============================================================
#
#   gbssetup.pl
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright © 2010-2020 - Randy Marques - All rights reserved
#===============================================================
use strict;
use warnings;
use integer;
print "*GBSSETUP @ARGV\n" if ($ENV{GBSDEBUG_FILE});




use 5.016_003;
use lib $ENV{GBS_SCRIPTS_PATH};
use glo::env;
use glo::list;
use glo::genopt;
use glo::ask;
use glo::format;
use glo::shell;
use gbsenv;
use profile;
use settings;
use setup;
use gbscmd;




sub check_essentials();
sub menu_settings($$$);
sub do_settings($);
sub menu_startup_settings($$$);
sub do_change($);
sub handle_scripts_path($$$$);
sub handle_rel($$$$);
sub handle_site($$$$);
sub handle_log_root($$$$);
sub handle_beeps($$$$);
sub handle_proc($$$$);
sub handle_perms($$$$);
sub show_changes();
sub get_stored_values();
sub store_changes();
sub write_setup_file();
sub check_gbs_root($$$);
sub find_gbs_rels($);
sub check_command_arg($$$);
sub get_default(@);




my $IS_WIN32 = (ENV_is_win32());
my $IS_LUNIX = ! $IS_WIN32;
my $CWD = ENV_cwd();

my $GBS_SCRIPTS_PATH = GBSENV_scripts_path();
my $GBS_BASE_PATH = ENV_getenv_perl_path( 'GBS_BASE_PATH');	# %APPDATA%/.gbs or ~/.gbs
ENV_sig( F => "GBS_BASE_PATH not defined")
if ($GBS_BASE_PATH eq '');




my %MAN = (
m => 'Mandatory',
o => 'Optional',
);

my %ENV_DEFS;




my @ENVS_ORDER;

{
my @env_def_refs = (



[ GBS_SCRIPTS_ROOT	=> [ 'm', 2, 'Scripts Root',	\&handle_scripts_path ] ],
[ GBS_SCRIPTS_REL	=> [ 'm', 2, 'Version',		\&handle_rel ] ],
[ GBS_SITE        	=> [ 'm', 1, 'Site',		\&handle_site ] ],
[ GBS_LOG_ROOT   	=> [ 'm', 1, 'LogRoot',		\&handle_log_root ] ],
[ GBS_EDITOR     	=> [ 'o', 0, 'Editor',		\&handle_proc ] ],
[ GBS_BROWSER     	=> [ 'o', 0, 'Browser',		\&handle_proc ] ],
[ GBS_VIEWER     	=> [ 'o', 0, 'Vieuwer',		\&handle_proc ] ],
[ GBS_BEEPS     	=> [ 'o', 0, 'Beeps',		\&handle_beeps ] ],
[ GBS_ADMINISTRATOR     => [ 'o', 0, 'Administrator',	\&handle_perms ] ],
[ GBS_INTEGRATOR        => [ 'o', 0, 'Integrator',	\&handle_perms ] ],
);
%ENV_DEFS = map { $_->[0] => $_->[1] } @env_def_refs;
@ENVS_ORDER = map { $_->[0] } @env_def_refs;
}

my @BASIC_SETTINGS = qw( GBS_SITE GBS_LOG_ROOT );
my @GENERAL_SETTINGS = qw( GBS_BEEPS GBS_EDITOR GBS_BROWSER GBS_VIEWER );
my @SPECIAL_SETTINGS = qw( GBS_ADMINISTRATOR GBS_INTEGRATOR );
my @SWITCH_SETTINGS = qw( GBS_SCRIPTS_REL );
my @ADVANCED_SETTINGS = qw( GBS_SCRIPTS_ROOT GBS_SCRIPTS_REL );

my @MAIN_MENU_ITEMS = (
[ "Basic Settings (Site, LogDir)",
\&menu_settings, \@BASIC_SETTINGS ],
[ "General Settings (Beeps, Editor, ...)",
\&menu_settings, \@GENERAL_SETTINGS ],
[ "Special Settings (Admin, Intgrtr)",
\&menu_settings, \@SPECIAL_SETTINGS ],
[ "Switch GBS Version",
\&menu_settings, \@SWITCH_SETTINGS ],
[ "Advanced Settings",
\&menu_settings, \@ADVANCED_SETTINGS ],
[ "All Settings",
\&menu_settings, \@ENVS_ORDER ],
[ "Startup Settings Only (Fix Shortcuts, ...)",
\&menu_startup_settings ],
);




my %ENV_DATA = map { $_ => [ undef, undef, 0 ] } @ENVS_ORDER;


my @CHANGED_REFS;

my $HIGHEST_RESTART_LEVEL = 0;


my $STARTUP_SETTINGS_EXECUTED = 0;






$| = 1;
my $RC = 0;
GBSENV_init();




my @genopts = (
[ '<1>',  'menu_entry_1', "isor1..7", 0, "Menu entry for immediate execution" ],
);
GENOPT_set_optdefs( 'gbssetup', \@genopts,
"Setup/Maintain Basic GBS settings (EnvVars).",
undef);
GENOPT_parse();
my $MENU_ENTRY_1 = GENOPT_get( 'menu_entry_1');
my @MENU_ENTRIES = grep( $_ != 0, ($MENU_ENTRY_1));






{



get_stored_values();




SETUP_show_licence()
if ($ENV_DATA{GBS_LOG_ROOT}->[0] eq '' || $ENV_DATA{GBS_SITE}->[0] eq '');	# $stored_value




check_essentials();	# ROOT, REL, SITE, LOGROOT




ASK_menu( 'Select function to perform', \@MAIN_MENU_ITEMS, [ @MENU_ENTRIES ]);




check_essentials();	# ROOT, REL, SITE, LOGROOT
my $nr_changes = @CHANGED_REFS;




if ($nr_changes > 0)
{
ENV_say( 1, "$nr_changes item(s) changed");
show_changes();
if (ASK_YN( '*** Implement Changes?', 'N') eq 'Y')
{
store_changes();
} else
{
$nr_changes = 0;
}
} else
{
ENV_say( 1, "Nothing changed");
}




if ($nr_changes > 0 && !$STARTUP_SETTINGS_EXECUTED)
{
menu_startup_settings( -1, undef, []);
}




if ($nr_changes > 0)
{
write_setup_file();
}
}




ENV_exit( $RC);




sub END
{
ENV_print_end_msg( 1);
}





sub check_essentials()
{
my @missing_essentials;
do
{
@missing_essentials = ();
$HIGHEST_RESTART_LEVEL = 0;
@CHANGED_REFS = ();
foreach my $name (@ENVS_ORDER)
{
my ($man, $restart_level) = @{$ENV_DEFS{$name}};
my $data_ref = $ENV_DATA{$name};
my ($stored_value, $new_value) = @{$data_ref};

if ($stored_value ne $new_value)
{
$data_ref->[2] = 1;	# $modified
push @CHANGED_REFS, [ $name => $new_value ];
$HIGHEST_RESTART_LEVEL = $restart_level
if ($restart_level > $HIGHEST_RESTART_LEVEL);
} else
{
$data_ref->[2] = 0;	# $modified
}
push @missing_essentials, $name
if ($new_value eq '' && $man eq 'm');  # mandatory
}

if (@missing_essentials)
{
ENV_beep();
ENV_say( 1, "Some essential data is missing. You need to supply that first:",
'');
do_settings( \@missing_essentials);
}
} while (@missing_essentials);
}




sub menu_settings($$$)
{
my ($entry_nr,
$args_ref,
$entries_ref,
) = @_;

do_settings( $args_ref);
}




sub do_settings($)
{
my ($settings_ref) = @_;

my @refs;
foreach my $name (@{$settings_ref})
{
my $stored_value = $ENV_DATA{$name}->[0];

{
my $man = $MAN{ $ENV_DEFS{$name}->[0] };
push @refs, [ $man, $name, $stored_value ];
}
}
ENV_say( 1, "Values: ");
ENV_say( 0, FORMAT_table( 0, 2, '  ', undef, @refs));

ENV_say( 1, "Change:");
foreach my $name (@{$settings_ref})
{
my $continue = do_change( $name);
last
if (!$continue);
}
}




sub menu_startup_settings($$$)
{
my ($entry_nr,
$args_ref,
$entries_ref,
) = @_;


SETTINGS_set_check( $ENV_DATA{GBS_SCRIPTS_ROOT}->[1], $ENV_DATA{GBS_SCRIPTS_REL}->[1]); # new_value

$STARTUP_SETTINGS_EXECUTED = 1;
}




sub do_change($)
{
my ($name) = @_;	    # GBS_...
my $continue = 1;	    # FALSE if $ans was 'E'


my ($man, undef, $nice_name, $function_ref) = @{$ENV_DEFS{$name}};
my ($stored_value, $new_value) = @{$ENV_DATA{$name}};
my $env_value = ENV_getenv( $name);
my $again;
do
{
$again = 0;
ENV_say( 1, "$name    ($MAN{$man})");
ENV_say( 0, "  EnvVar : $env_value")
if ($env_value ne '');
ENV_say( 0, "  Current: $stored_value");
ENV_say( 0, "  New    : $new_value");

my $default;
my $ans;
if ($man eq 'o')	# optional
{
$default = 'N';
$ans = ASK_YNEQ( 'Change?', $default, 0);
} else			# mandatory
{
$default = ($stored_value eq '' && $new_value eq '') ? 'Y' : 'N';
$ans = ASK_YNQ( 'Change?', $default, 0);
}
if ($ans eq 'Y' || $ans eq 'N')
{
if ($ans eq 'Y')
{



$new_value = $function_ref->( $name, $stored_value, $new_value, $env_value);
$ENV_DATA{$name}->[1] = $new_value;
$again = 1;
}
if ($man ne 'o' && $new_value eq '')
{
ENV_beep();
ENV_say( 1, "** Value required!");
$again = 1;
}
} elsif ($ans eq 'E')
{
$continue = 0;
} else	# $ans eq 'Q'
{
ENV_say( 1, "Terminated - Nothing changed");
ENV_exit();
}
} while ($again);

return $continue;
}




sub handle_scripts_path($$$$)
{
my ($name,
$stored_value,
$new_value,
$env_value,
) = @_;
my $new_scripts_path;

my $default = ENV_parent_path( $GBS_SCRIPTS_PATH);

$new_scripts_path = ASK_path( $name, $default, 1, -1, [ \&check_gbs_root, undef ] );

if (!$stored_value || $new_scripts_path ne $stored_value)
{
my $rel_part = ($IS_WIN32) ? '%GBS_SCRIPTS_REL%' : '$GBS_SCRIPTS_REL';
my $gbs_scripts = ENV_os_path( "$new_scripts_path/$rel_part");
$ENV_DATA{GBS_SCRIPTS_PATH}->[0] = ''
if (!defined $ENV_DATA{GBS_SCRIPTS_PATH}->[0]);	    # $stored_value
$ENV_DATA{GBS_SCRIPTS_PATH}->[1] = $gbs_scripts;	    # $new_value
ENV_say( 1, "GBS_SCRIPTS_PATH set to '$gbs_scripts'");
}

return ENV_os_path( $new_scripts_path);
}




sub handle_rel($$$$)
{
my ($name,		    # ignored
$stored_value,
$new_value,
$env_value,
) = @_;
my $new_rel;

my $gbs_scripts_root = $ENV_DATA{GBS_SCRIPTS_ROOT}->[1];     # $new_value
my @gbs_rels = find_gbs_rels( $gbs_scripts_root);
if (@gbs_rels == 0)
{
ENV_sig( F => "No GBS Release directories found in $gbs_scripts_root");
} elsif (@gbs_rels == 1)
{
$new_rel = $gbs_rels[0];
ENV_say( 1, "Single GBS Release directory '$new_rel' taken");
} else
{
my $default_i = -1;
$default_i = LIST_firstidx_str( $new_value, \@gbs_rels) if ($new_value);
$default_i = LIST_firstidx_str( $env_value, \@gbs_rels) if ($env_value);
$default_i = LIST_firstidx_str( $stored_value, \@gbs_rels) if ($stored_value);
my $new_rel_i = ASK_index_from_menu( $name, $default_i, @gbs_rels);
$new_rel = $gbs_rels[$new_rel_i];
}

return $new_rel;
}




sub handle_site($$$$)
{
my ($name,		    # ignored
$stored_value,
$new_value,
$env_value,
) = @_;


my $default = get_default( $new_value, $stored_value, $env_value);

return SETUP_ask_site( $default);
}




sub handle_log_root($$$$)
{
my ($name,		    # ignored
$stored_value,
$new_value,
$env_value,
) = @_;


my $default = get_default( $new_value, $stored_value, $env_value);

return SETUP_ask_log_root( $default);
}




sub handle_beeps($$$$)
{
my ($name,		    # ignored
$stored_value,
$new_value,
$env_value,
) = @_;
my $newer_value;

my $default = get_default( $new_value, $stored_value, $env_value, 'OFF');
$default = uc $default;
$default = ($default eq 'YES' || $default eq 'ON') ? 'N' : 'Y';
my $ans = ASK_YN( 'Enable Beeps?', $default);
$newer_value = ($ans eq 'Y') ? 'YES' : 'NO';

return $newer_value ;
}




sub handle_proc($$$$)
{
my ($name,		    # ignored
$stored_value,
$new_value,
$env_value,
) = @_;
my $newer_value;

my $prompt_name = ucfirst( lc( substr( $name, 4)));




my $default = get_default( $new_value, $stored_value, $env_value);
my $default_exec = '';
my @default_args = ();
($default_exec, @default_args) = ENV_split_quoted_space( $default) if ($default);
my $default_args = "@default_args";

my $exec = ASK_filespec( "Enter exec path for $prompt_name", $default_exec, 1, 0);
$exec = ENV_os_path( $exec);
if ($exec ne '')
{
$exec = "\"$exec\"" if ($exec =~ /\s/);

my $again;
my $param = ($IS_WIN32) ? '%1%' : '$1';
$default_args = $param if (!$default_args);
my $arg = ASK_text( "Argument(s) with '$param' to specify file", $default_args,
-2, [ \&check_command_arg, $param ]);
$newer_value = "$exec $arg";
} else
{
$newer_value = '';
}

return $newer_value;
}




sub handle_perms($$$$)
{
my ($name,		    # GBS_INTEGRATOR or GBS_ADMINISTRATOR
$stored_value,
$new_value,
$env_value,
) = @_;
my $newer_value;

my $prompt = ($name eq 'GBS_INTEGRATOR') ? 'Integrator' : 'Administrator';
my $gbs_system_name = ENV_getenv( 'GBS_SYSTEM_NAME');
my $default = get_default( $new_value, $stored_value, $env_value);
my @perms = split( ',', $default);
if (grep $gbs_system_name eq $_, @perms)
{
ENV_say( 1, "You are already $prompt");
my $ans = ASK_YN( 'Disable?', 'N');
if ($ans eq 'Y')
{
@perms = grep $gbs_system_name ne $_, @perms;
}
} else
{
ENV_say( 1, "You are not $prompt");
my $ans = ASK_YN( 'Enable?', 'N');
if ($ans eq 'Y')
{
push @perms, $gbs_system_name;
}
}
$newer_value = join( ',', @perms);

return $newer_value ;
}




sub show_changes()
{
ENV_say( 1, "Changes: ");
ENV_say( 0, FORMAT_table( 0, 2, ' => ', undef, @CHANGED_REFS));
}




sub get_stored_values()
{



my $exists = PROFILE_open( $GBS_BASE_PATH);
foreach my $name (@ENVS_ORDER)
{
my $value = PROFILE_get( $name);
$ENV_DATA{$name}->[0] = $value;     # $stored_value
$ENV_DATA{$name}->[1] = $value;     # $new_value

}

if ($ENV_DATA{GBS_SCRIPTS_ROOT}->[0] eq '' || $ENV_DATA{GBS_SCRIPTS_REL}->[0] eq '')
{



if (-f "$CWD/gbsswr.pl")
{
$GBS_SCRIPTS_PATH = $CWD;
my $gbs_scripts_root = ENV_parent_path( $CWD);
my $gbs_scripts_rel = ENV_parent_dir( $CWD, -1);
$ENV_DATA{GBS_SCRIPTS_ROOT}->[1] = $gbs_scripts_root;
$ENV_DATA{GBS_SCRIPTS_REL}->[1]  = $gbs_scripts_rel;
push @CHANGED_REFS, [ GBS_SCRIPTS_ROOT => $gbs_scripts_root ];
push @CHANGED_REFS, [ GBS_SCRIPTS_REL  => $gbs_scripts_rel ];

ENV_say( 1, "GBS_SCRIPTS_PATH initialised to $GBS_SCRIPTS_PATH");
}
}
}




sub store_changes()
{
ENV_say( 1, "Store Changes...");




foreach my $ref (@CHANGED_REFS)
{
my ($name, $new_value) = @{$ref};
PROFILE_set( $name => $new_value);

}

PROFILE_close(1 );	    # close and write
}




sub write_setup_file()
{
ENV_say( 1, "Set Env. Changes...");



my @lines;
if ($HIGHEST_RESTART_LEVEL == 2)
{



if (ENV_getenv( 'GBS_EXEC_MODE') ne '')	# GBS active?
{
push @lines, SHELL_echo( "*** GBS Exiting... ***");
push @lines, GBSCMD_command( 'gbsexit');
push @lines, SHELL_echo( "*** GBS Restarting... ***");
push @lines, GBSCMD_base_command( 'gbsinit');
}
} else
{



foreach my $name (@ENVS_ORDER)
{
my ($stored_value, $new_value, $modified) = @{$ENV_DATA{$name}};
if ($modified)
{
push @lines, SHELL_setenv( $name => $new_value);
ENV_say( 2, "$name => $new_value");
}
}
if ($HIGHEST_RESTART_LEVEL == 1)
{



if (ENV_getenv( 'GBS_ROOT_PATH') ne '') # rsw active?
{
push @lines, SHELL_echo( "*** Restarting the current Root...");
push @lines, GBSCMD_command( 'gbsswr', '.');
}
}
}

GBSENV_write_result_script( \@lines, 0);
}




sub check_gbs_root($$$)
{
my ($values_ref,
$default_ref,
$args_ref,
) = @_;
my $error_txt = '';

my $path = $values_ref->[0];

my @gbs_rels = find_gbs_rels( $path);
$error_txt = "'$path' is not a valid GBS_SCRIPTS_PATH"
if (!@gbs_rels);

return $error_txt;
}




sub find_gbs_rels($)
{
my ($path) = @_;
my @gbs_rels;

my @rels = glob( "$path/*/gbsswr.pl");
@gbs_rels = map { ENV_parent_dir( $_, -2) } @rels;

return @gbs_rels;
}




sub check_command_arg($$$)
{
my ($values_ref,
$default_ref,
$param,
) = @_;
my $error_txt = '';

my $arg = $values_ref->[0];
my $q_param = quotemeta $param;
$error_txt = "'$param' missing"
if ($arg !~ /$q_param/);

return $error_txt;
}





sub get_default(@)
{
my @values = @_;
my $default = '';

foreach my $value (@values)
{
if ($value)
{
$default = $value;
last;
}
}

return $default;
}


