#===============================================================
#
#   gbssilo.pl
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
use strict;
use warnings;
use integer;
print "*GBSSILO @ARGV\n" if ($ENV{GBSDEBUG_FILE});




use lib $ENV{GBS_SCRIPTS_PATH};
use glo::env;
use glo::genopt;
use glo::html;
use glo::spit;
use glo::slurp;
use glo::time;
use gbsenv;
use gbsres;
use gbsglo;
use gbssilo;
use gbshtml;




sub create_summaries();
sub sys_sum_file($$);
sub audit_sum_file();
sub create_target_audits_overview();




my $GBS_ROOT_PATH = GBSENV_gbs_root_path();
my $GBS_SILO_PATH = "$GBS_ROOT_PATH/silo";

my $GBS_SYSTEM_NAME = ENV_getenv( 'GBS_SYSTEM_NAME');
my $GBS_ROOT_PARENT = ENV_getenv( 'GBS_ROOT_PARENT');
my $GBS_LOG_PATH = ENV_getenv_perl_path( 'GBS_LOG_PATH');

my $NOW_DATETIME = TIME_time2num();	    # YYYYY-MM-DD-HH:MM:SS
my $TODAY = substr( $NOW_DATETIME, 0, 10);  # YYYYY-MM-DD-HH






my $RC = 0;
GBSENV_init();




my @genopts = (
[ 'ci',     'create_index',	     'bso',   1, "Create the HTML Index" ],
[ 'cs',     'create_summaries',  'bso',   1, "Create the HTML Summaries" ],
[ 'vi',     'view_index',	     'bso',   1, "View index in HTML browser" ],
[ 'noc',	'no_create',	     'bso',   0, "No HTML Creation (forces --ci and --cs to '-')" ],
);
my @genconflicts = (


[ [ noc  => 1 ], '=' => 'ci', '=' => 'cs' ],
);
GENOPT_set_optdefs( 'gbssilo', \@genopts,
'Generate silo index and show in HTML browser',
undef);
GENOPT_set_conflicts( \@genconflicts);
GENOPT_parse();
my $CREATE_SUMMARIES = GENOPT_get( 'create_summaries');
my $CREATE_INDEX = GENOPT_get( 'create_index');
my $VIEW_INDEX = GENOPT_get( 'view_index');
my $NO_CREATE = GENOPT_get( 'no_create');

if ($NO_CREATE)
{
$CREATE_SUMMARIES = 0;
$CREATE_INDEX = 0;
}

{



GBSHTML_init( 1);	    # Use Silo




if ($CREATE_SUMMARIES)
{
create_summaries();
create_target_audits_overview()
}




GBSSILO_create_index()
if ($CREATE_INDEX);




GBSSILO_view_index()
if ($VIEW_INDEX);
}




ENV_exit( $RC);




sub END
{
ENV_print_end_msg( 1);
}




sub create_summaries()
{
my $sum_filespec = "$GBS_SILO_PATH/.gbs/gbssum/gbssyssum.log";
if (-e $sum_filespec)
{



my @build_lines;
my @audit_lines;
my @tool_lines;
my $line_nr = 0;
foreach my $sum_line (SLURP_file( $sum_filespec))
{
$line_nr++;
next if ($sum_line eq '');

if (substr( $sum_line, 0, 11) eq 'gbssysaudit')
{
push @audit_lines, "$line_nr\t$sum_line";
} elsif (substr( $sum_line, 0, 10) eq 'gbssystool')
{
push @tool_lines, "$line_nr\t$sum_line";
} else	# gbssysgen and gbssysmake
{
push @build_lines, "$line_nr\t$sum_line";
}
}
if (@build_lines)
{
sys_sum_file( gbssysbuild => \@build_lines)
}
if (@audit_lines)
{
sys_sum_file( gbssysaudit => \@audit_lines)
}
if (@tool_lines)
{
sys_sum_file( gbssystool => \@tool_lines)
}
}




audit_sum_file();
}




sub sys_sum_file($$)
{
my ($name,			# 'gbssysaudit', 'gbssysbuild', 'gbssystool'
$lines_ref,
) = @_;

my $is_audit = ($name eq 'gbssysaudit') ? 1 : 0;
my $is_tool = ($name eq 'gbssystool') ? 1 : 0;




my $filespec = "$GBS_SILO_PATH/.gbs/gbssum/$name.html";
my @lines;
my $head_title = "Silo - $GBS_SYSTEM_NAME [$GBS_ROOT_PARENT] - Overview: $name";

push @lines, GBSHTML_doc_start( $filespec, $head_title, 1, '', undef); # $want_scripts, $body_class, $target_frame_name
push @lines, GBSHTML_doc_top( [ $head_title, HTML_link( $GBS_LOG_PATH, HTML_bold( 'GBS Log Directory'))],
( [ "Date: $NOW_DATETIME", '', '' ],
));




my @row_refs;




if ($is_audit)
{
push @lines, HTML_text( 'Please be aware that only the HTML-summary files were saved. Not the files referred to.'),
HTML_br(),
HTML_bold_txt( 'The files referred to will always be the last audited file versions.'),
HTML_br();
}
push @lines, HTML_br(),
HTML_text( 'Note that the following table is sortable by the items in the top row.');




foreach my $line (reverse( @{$lines_ref}))
{
my (
$line_nr,
$jobname,
$target_or_tool,
$uname,
$root_path,			    # os format
$logfile,			    # os format
$time_start,
$time_end,
$time_diff,
$timer_diff,
$state,			    # NORMAL, FAILED, KILLED
$rc,
$action,
$date_time,			    # YYYYMMDD-HHMMSS
$command_args,
$comments,
undef
) = split( /\t/, $line);


if (defined $comments)
{
$logfile = ENV_perl_canon_paths( $logfile);
$comments = ''
if ($comments eq '<<EOF>>');

my $logfile_html;
if (-e $logfile)
{
$logfile_html = HTML_link( {target => '_blank'}, $logfile, 'VIEW');
} else
{
my ($path, $file) = ENV_split_spec_df( $logfile);
my $moved_logfile = "$GBS_LOG_PATH/$file";
$logfile_html = (-e $moved_logfile) ? HTML_link( {target => '_blank'}, $moved_logfile, 'VIEW*') :
HTML_span( {title => $moved_logfile}, 'Missing');
}
my $state_html = HTML_div( {class => $state}, $state);
if ($date_time eq '')
{
CORE::say( $line);
}
$date_time = TIME_unpack( $date_time);
$date_time =~ s/ /&nbsp;/;
if (substr( $date_time, 0, 10) eq $TODAY)
{
$date_time .= '*';
$date_time = HTML_span( { style => "background-color: #F0F0F0"}, $date_time);	# very light grey
}
if ($is_audit)
{
my ($audit) = $logfile =~ /gbssysaudit_${target_or_tool}_(.*)_\d/;
$audit = '*UNK*'
if (!defined $audit);
my $htmlfile = "$GBS_SILO_PATH/audits/" . ENV_split_spec_n( $logfile) . "_sum.html";
my $htmlfile_html = (-e $htmlfile) ? HTML_link( {target => '_blank'}, $htmlfile, 'VIEW') :
HTML_span( {title => $htmlfile}, 'Missing');
push @row_refs, [ $jobname, $audit, $target_or_tool, $date_time, $state_html, $rc, $logfile_html, $htmlfile_html, $command_args, $comments ];
} else
{
push @row_refs, [ $jobname, $target_or_tool, $date_time, $state_html, $rc, $logfile_html, $command_args, $comments ];
}
} else
{
$line =~ s/^\d+\t//;
ENV_sig( W => "Invalid log line ($line_nr)", $line);
}
}

push @lines, HTML_br();
if ($is_audit)
{
push @lines, HTML_table( [ qw( - - - - -C -R -C -C - - ) ],
[ qw( Job Audit Target DateTime State RC LogFile HTML<BR>Sum Comand-Args Comments) ],
\@row_refs, border => 1, class => 'sortable border1');
} elsif ($is_tool)
{
push @lines, HTML_table( [ qw( - - - -C -R -C - - ) ],
[ qw( Job Tool DateTime State RC LogFile Comand-Args Comments) ],
\@row_refs, border => 1, class => 'sortable border1');
} else
{
push @lines, HTML_table( [ qw( - - - -C -R -C - - ) ],
[ qw( Job Target DateTime State RC LogFile Comand-Args Comments) ],
\@row_refs, border => 1, class => 'sortable border1');
}




push @lines, GBSHTML_doc_bottom();
push @lines, GBSHTML_doc_end();

SPIT_file_nl( $filespec, \@lines);
ENV_say( 1, "Created Silo $name");
}




sub audit_sum_file()
{
my @html_files = ENV_glob( "$GBS_LOG_PATH/gbsaudit_*_sum.html");
if (@html_files)
{



my $name = 'gbsaudit';
my $filespec = "$GBS_SILO_PATH/.gbs/gbssum/$name.html";
my @lines;
my $head_title = "Silo - $GBS_SYSTEM_NAME [$GBS_ROOT_PARENT] - Overview: $name";

push @lines, GBSHTML_doc_start( $filespec, $head_title, 1, '', undef); # $want_scripts, $body_class, $target_frame_name
push @lines, GBSHTML_doc_top( [ $head_title, HTML_link( $GBS_LOG_PATH, HTML_bold( 'GBS Log Directory'))],
( [ "Date: $NOW_DATETIME", '', '' ],
));



my @row_refs;




push @lines, HTML_text( 'Please be aware that only the HTML-summary files were saved. Not the files referred to.'),
HTML_br(),
HTML_bold_txt( 'The files referred to will always be the last audited file versions.'),
HTML_br();

push @lines, HTML_br(),
HTML_text( 'Note that the following table is sortable by the items in the top row.');




foreach my $html_filespec (@html_files)
{

my ($jobname, $target_or_tool, $audit, $numtime) = GBSRES_split_logfile_spec( $html_filespec);
if (defined $jobname)
{
my $date_time = TIME_unpack( $numtime);
$date_time =~ s/ /&nbsp;/;
if (substr( $date_time, 0, 10) eq $TODAY)
{
$date_time .= '*';
$date_time = HTML_span( { style => "background-color: #F0F0F0"}, $date_time);	# very light grey
}
push @row_refs, [ $jobname, $audit, $target_or_tool, $date_time,
HTML_link( {target => '_blank'}, $html_filespec, 'VIEW')];
}
}
@row_refs = sort { $b->[3] cmp $a->[3] } @row_refs;	# $numtime

push @lines, HTML_br();
push @lines, HTML_table( [ qw( - - - - -C ) ],
[ qw( Job Audit Target DateTime HTML<BR>Sum) ],
\@row_refs, border => 1, class => 'sortable border1');



push @lines, GBSHTML_doc_bottom();
push @lines, GBSHTML_doc_end();

SPIT_file_nl( $filespec, \@lines);
ENV_say( 1, "Created Silo $name");
}
}




sub create_target_audits_overview()
{
my $path = "$GBS_SILO_PATH/target_audits_overview";
ENV_mkdir( $path);
my $filespec = "$path/index.html";
my @lines;




my $head_title = "Silo - $GBS_SYSTEM_NAME [$GBS_SYSTEM_NAME] - Targets and Audits Overview";

push @lines, GBSHTML_doc_start( $filespec, $head_title, 1, '', undef); # $want_scripts, $body_class, $target_frame_name
push @lines, GBSHTML_doc_top( [ $head_title, HTML_link( '../index.html', HTML_bold( 'Silo Home')) ],
([ "Date: $NOW_DATETIME", '', ''],
));








push @lines, HTML_br(),
HTML_text( 'Note that the following table is sortable by the items in the top row.');




my @row_refs;
my @gbs_all_targets = ENV_getenv( 'GBS_ALL_TARGETS');
my %gbs_all_targets = map { $_ => 1 } @gbs_all_targets;
my $nr_targets = @gbs_all_targets;
my @gbs_all_audits =  ENV_getenv( 'GBS_ALL_AUDITS');
my %gbs_all_audits = map { $_ => 1 } @gbs_all_audits;
my $nr_audits = @gbs_all_audits;

foreach my $subsys (GBSGLO_subsystems())
{
my $sstype = GBSGLO_subsystem_type( $subsys);

if ($sstype eq 'GBS')
{



foreach my $component (GBSGLO_components( $subsys))
{

my $component_path = "$GBS_ROOT_PATH/dev/$subsys/comp/$component";
foreach my $target (@gbs_all_targets)
{
my $ss_target = (GBSGLO_subsystem_does_build( $subsys, $target)) ? $target : "$target*";
my @found_audits;
foreach my $audit (@gbs_all_audits)
{
if (-e "$component_path/aud/$audit/$target")
{
push @found_audits, $audit;
}
}
if (@found_audits)
{
foreach my $audit (@found_audits)
{
my $ss_audit = (GBSGLO_subsystem_does_audit( $subsys, $audit, $target)) ? $audit : "$audit*";
push @row_refs, [ $subsys, $sstype, $ss_target, $ss_audit, $component, $ss_target, $audit ];
}
} else
{
push @row_refs, [ $subsys, $sstype, $ss_target, '', $component, $ss_target, '' ]
if ($ss_target ne "$target*");
}
}
}
} else
{



my $nr_ss_targets = 0;
foreach my $ss_target( @gbs_all_targets)
{
if (GBSGLO_subsystem_does_build( $subsys, $ss_target))
{
my $nr_ss_audits = 0;
foreach my $ss_audit (@gbs_all_audits)
{
if (GBSGLO_subsystem_does_audit( $subsys, $ss_audit, $ss_target))
{
push @row_refs, [ $subsys, $sstype, $ss_target, $ss_audit, '', '', ''];
$nr_ss_audits++;
$nr_ss_targets++;
}
}
if ($nr_ss_audits == 0)
{
push @row_refs, [ $subsys, $sstype, $ss_target, '', '', '', '' ];
$nr_ss_targets++;
}
}
}
push @row_refs, [ $subsys, $sstype, '', '', '', '', '' ]
if ($nr_ss_targets == 0);

}
}

push @lines, HTML_table( [ qw( - - - - - - - ) ],
[ qw( SubSys SSTYPE Target Audit Component Target Audit) ],
\@row_refs, border => 1, class => 'sortable border1');




push @lines, GBSHTML_doc_bottom();
push @lines, GBSHTML_doc_end();

SPIT_file_nl( $filespec, \@lines);
ENV_say( 1, "Created Silo Targets and Audits Overview");
}


