#==============================================================
#
#   gbsswr.pl
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
use strict;
use warnings;
use integer;
print "*GBSSWR @ARGV\n" if ($ENV{GBSDEBUG_FILE});




use lib $ENV{GBS_SCRIPTS_PATH};
use glo::env;
use glo::shell;
use glo::ask;
use glo::genopt;
use glo::slurp;
use glo::format;
use gbsenv;
use run;
use roots;
use rootsglo;
use validate;
use system;
use fspec;
use swb;
use swa;
use swt;
use swc;
use sws;
use swr;
use swglo;
use gbsdb;
use profile;




sub ask_root();
sub do_add_root($);
sub ask_parent_path($);
sub check_gbs_root($$$);
sub do_remove_root($);
sub do_show_roots();
sub select_root($$);
sub add_to_roots_list($);
sub check_and_write_roots_list();




my $CWD = ENV_cwd();

my $GBS_ROOT_PATH = GBSENV_gbs_root_path();
my $GBS_SYSTEM_NAME = ENV_getenv( 'GBS_SYSTEM_NAME');
my $GBS_SUBSYS = ENV_getenv( 'GBS_SUBSYS');
my $GBS_COMPONENT = ENV_getenv( 'GBS_COMPONENT');
my $GBS_TARGET = ENV_getenv( 'GBS_TARGET');
my $GBS_AUDIT = ENV_getenv( 'GBS_AUDIT');
my $GBS_TOOL = ENV_getenv( 'GBS_TOOL');

my $CANCELLED = 0;






$| = 1;
my $RC = 0;
GBSENV_init();




my @genopts = (
[ '<1>',	'root_path', 	   'sso',      '', "System / Root: '' == ASK, '-' == none, '.' == current, '!' == CWD, '!<abs_file/dir_spec>', '!!' == CWD = GBS_ROOT_PATH" ],
[ '<2>',	'subsystem',	   'sso',      '', "SubSystem: '-' == none, '.' == current" ],
[ '<3>',	'component',	   'sso',      '', "Component: '-' == none, '.' == current" ],
[ 'target',	'target',	   'sso',     '.', "Target: '-' == none, '.' == current" ],
[ 'audit',	'audit',	   'sso',     '.', "Audit: '-' == none, '.' == current" ],
[ 'tool',	'tool',		   'sso',     '.', "Tool: '-' == none, '.' == current" ],
[ 'new',	'create_new_root', 'bso',	0, "Create a new System / Root" ],
[ 'add',	'add_root',	   'bso',	0, "Add a Root to the Roots List" ],
[ 'remove', 'remove_root',	   'bso',	0, "Remove a Root from the Roots List" ],
[ 'show',   'show_roots',	   'bso',	0, "Show the Roots List" ],
[ 'exec',	'execute_command', 'sso',      '', "(GBS-)Command(s) to execute. 'none' = OK" ],
[ 'nocd',	'no_cd',	   'bso',	0, "Do not 'cd' to the selected directories "],
);
my @genconflicts = (
[ [ new => 1],	'=', 'add', '=', 'remove', '=', 'show',   '=', 'exec', '=', '<2>', '=', '<3>', '=', 'target' ],
[ [ add => 1],	'=', 'new', '=', 'remove', '=', 'show',   '=', 'exec', '=', '<2>', '=', '<3>', '=', 'target' ],
[ [ remove => 1],	'=', 'new', '=', 'add',    '=', 'show',   '=', 'exec', '=', '<2>', '=', '<3>', '=', 'target' ],
[ [ show => 1],	'=', 'new', '=', 'add',    '=', 'remove', '=', 'exec', '=', '<1>', '=', '<2>', '=', '<3>', '=', 'target' ],
[ [ '<1>' => '-' ], '=', '<2>' ],
[ [ '<1>' => '-' ], '=', '<3>' ],
[ [ '<1>' => '-' ], '=', 'target' ],
[ [ '<2>' => '-' ], '=', '<3>' ],
);
GENOPT_set_optdefs( [ 'gbsswr', 'swr' ], \@genopts,
[ 'Set a current Root/System or',
'Create a new one, Add / Remove one from the Roots List or Show the Roots List'],
"You can only '--add' a Root if it was originally created by 'swr --new'");
GENOPT_set_conflicts( \@genconflicts);
GENOPT_parse();
my $ROOT_PATH = GENOPT_get( 'root_path');     	    # Unix or Windows format
my $SUBSYS = GENOPT_get( 'subsystem');
my $COMPONENT = GENOPT_get( 'component');
my $TARGET = GENOPT_get( 'target');
my $AUDIT = GENOPT_get( 'audit');
my $TOOL = GENOPT_get( 'tool');
my $NEW = GENOPT_get( 'create_new_root');
my $ADD = GENOPT_get( 'add_root');
my $REMOVE = GENOPT_get( 'remove_root');
my $SHOW_ROOTS = GENOPT_get( 'show_roots');
my $EXEC = GENOPT_get( 'execute_command');
my $NO_CD = GENOPT_get( 'no_cd');

$ROOT_PATH = ENV_perl_canon_paths( $ROOT_PATH)		    # Perl format
if ($ROOT_PATH ne '');





VALIDATE_gbs();	    # Checks if GBS is active

my @ALL_ROOT_PATHS = ROOTS_get_all( 1);	    # Perl paths

{



if (!$NEW && !$ADD && @ALL_ROOT_PATHS == 0)
{
ENV_say( 1, "No Root/System(s) defined. Create or Add?");
if (ASK_YN( "Create now?", 'Y') eq 'Y')
{
$NEW = 1;
} elsif (ASK_YN( "Add now?", 'Y') eq 'Y')
{
$ADD = 1;
}
}




if ($NEW)
{



SWR_set( '', 0);
my $new_root_path = RUN_gbsnew( 'new_root', $ROOT_PATH);
if ($new_root_path ne '')
{
$ROOT_PATH = $new_root_path;
$SUBSYS = '-';
$COMPONENT = '-';
$TARGET = '-';
$AUDIT = '-';
$TOOL = '-';
add_to_roots_list( $ROOT_PATH);
} else
{
$ROOT_PATH = '.';
$CANCELLED = 1;
}
} elsif ($ADD)
{



my $added_root_path = do_add_root( $ROOT_PATH);
$ROOT_PATH = ($added_root_path eq '') ? '.' : $added_root_path;
} elsif ($REMOVE)
{



my $removed_root_path = do_remove_root( ENV_perl_path_nospace( $ROOT_PATH));
my $current_root_path = GBSENV_gbs_root_path();


if ($removed_root_path eq $current_root_path)
{
ENV_say( 1, "Current Root removed");
$ROOT_PATH = '-';	# No current
} else
{
$ROOT_PATH = '.';	# Keep current
$CANCELLED = 1;
}

} elsif ($SHOW_ROOTS)
{



do_show_roots();
}

if (!$CANCELLED)
{



if ($ROOT_PATH eq '')
{
$ROOT_PATH = ask_root();
}

my @swglo_items = SWGLO_validate( $TARGET, $COMPONENT, $SUBSYS, $ROOT_PATH, $AUDIT, $TOOL);
my ($new_target, $new_component, $new_subsys, $new_root_path, $new_audit, $new_tool) = @swglo_items;





SWC_set( $GBS_SUBSYS, '', 0)	    # Clear Current Component
if ($GBS_COMPONENT ne '');
SWS_set( '', 0)			    # Clear Current SubSys
if ($GBS_SUBSYS ne '');
SWA_set( '', 0)			    # Clear Current Audit
if ($GBS_AUDIT ne '');
SWB_set( '', 0)			    # Clear Current Target
if ($GBS_TARGET ne '');
SWT_set( '', 0)			    # Clear Current Tool
if ($GBS_TOOL ne '');
SWR_set( '', 0)			    # Clear Current Root
if ($GBS_ROOT_PATH ne '');




ENV_setenv( GBS_ROOT_PATH   => ENV_os_path( $new_root_path));
ENV_setenv( GBS_TARGET	    => $new_target);
ENV_setenv( GBS_AUDIT	    => $new_audit);
ENV_setenv( GBS_TOOL	    => $new_tool);
ENV_setenv( GBS_SUBSYS	    => $new_subsys);
ENV_setenv( GBS_COMPONENT   => $new_component);




{
my @options = (
GENOPT_makeopt( 0, create_new_root => $NEW),
GENOPT_makeopt( 0, execute_command	=> $EXEC),
GENOPT_makeopt( 0, no_cd		=> $NO_CD),
);

ENV_setenv( GBS_SWR_SET_ARGS => "@options");
}




{



my @lines;




push @lines, SWR_exit( $GBS_ROOT_PATH)
if ($GBS_ROOT_PATH ne '');




push @lines, SWGLO_set_title();




push @lines, GBSENV_changed_setenv_commands( 0);	    # $must_log = 0




if ($new_root_path ne '')
{
push @lines, SWR_entry( $new_root_path);
push @lines, SHELL_conditional_ne( GBS_RC => 0, SHELL_echo( "Execution of switch.gbs failed. GBS_SITE not ok?"));
}




push @lines, SHELL_conditional_ne( GBS_RC => 0, SHELL_echo_failed());

push @lines, SHELL_exit_rc( 1);
GBSENV_write_result_script( \@lines, 0);	# $print_lines = 0
}
}
}

ENV_exit( $RC);




sub END
{
if ($CANCELLED)
{
ENV_print_end_msg( 1);
} else
{
ENV_print_end_msg( 0);
}
}




sub ask_root()
{
my $root_path;	    # Perl format




if (@ALL_ROOT_PATHS == 0)
{
ENV_sig( EE => "No Root(s) defined for this user",
"use 'swr --add' to use an existing Root/System",
"or  'swr --new' to define a new Root/System");

} elsif (@ALL_ROOT_PATHS == 1)
{
$root_path = ENV_perl_path_nospace( $ALL_ROOT_PATHS[0]);
ENV_say( 1, "Single Root '$root_path' taken by default");
} else
{



my $default_root_path = ENV_getenv_perl_path( 'GBS_ROOT_PATH');
$default_root_path = GBSDB_last_root_get()
if ($default_root_path eq '');
$root_path = select_root( "Select Root/System Directory", \$default_root_path);
}

return $root_path;
}




sub do_add_root($)
{
my ($root_path) = @_;	# From CommandLine - Perl format

ENV_say( 1, "Add Root/System ($root_path)");

if ($root_path eq '')
{



ENV_say( 1, "Please note that you can only ADD a Root that was originally created with 'swr --new'");
if (SWGLO_check_system( $CWD))
{



ENV_say( 1, "Currently in '$CWD',",
"Which seems to be the Top of a GBS Root");
$root_path = $CWD
if (ASK_YNQ( "Do you want to add it?", 'N', 1) eq 'Y');   # exit on quit
}

if ($root_path eq '')
{



$root_path = ask_parent_path( $CWD);
}

my $root_parent_path = ENV_parent_path( $GBS_ROOT_PATH);
if ($root_path eq '' && $root_parent_path ne '' && $root_parent_path ne $CWD)
{



$root_path = ask_parent_path( $root_parent_path);
}

if ($root_path eq '')
{



if (ROOTS_get_nr() > 0)
{
ENV_say( 1, "Currently in Roots List:");
ROOTSGLO_print_root_cols();
}
$root_path = ASK_path( "Enter existing GBS Root Directory",
'', 1, 0, [ \&check_gbs_root, undef ]);
$root_path = ENV_perl_canon_paths( $root_path);
}
} else
{



$root_path = ENV_perl_path_nospace( $root_path);
ENV_say( 1, "Adding $root_path");
ENV_sig( EE => "No such directory: $root_path")
if (!-d $root_path);
ENV_sig( EE => "Not a GBS Root: $root_path")
if (!SWGLO_check_system( $root_path));
}

if ($root_path eq '')
{
ENV_sig( W => "No Root/System added!");
$CANCELLED = 1;
} else
{
if (SWGLO_check_system_other_platform( $root_path))
{
ENV_say( 1, "Adding Other Platform...");
$root_path = RUN_gbsnew( 'new_other_root', $root_path);
if ($root_path ne '')
{



$TARGET = '-';
$AUDIT = '-';
$TOOL = '-';

add_to_roots_list( $root_path);
} else
{
$CANCELLED = 1;
}
} else
{
add_to_roots_list( $root_path);
}
}

return $root_path;
}




sub ask_parent_path($)
{
my ($parent_path) = @_;
my $root_path = '';




my @root_refs;

my $nr_this_platform = 0;
my $nr_other_platform = 0;
my $nr_in_roots = 0;
foreach my $dir (SLURP_dir_dirs( $parent_path, 0))
{
my $is_this_platform = 0;
my $is_other_platform = 0;
my $this_path = "$parent_path/$dir";

if (SWGLO_check_system( $this_path))
{
$is_this_platform = 1;
$nr_this_platform++;
}
if (SWGLO_check_system_other_platform( $this_path))
{
$is_other_platform = 1;
$nr_other_platform++;
}

if ($is_this_platform || $is_other_platform)
{
my $is_in_roots = ROOTS_exists( $this_path);
$nr_in_roots++
if ($is_in_roots);

push @root_refs, [ $dir, $is_this_platform, $is_other_platform, $is_in_roots ]
}
}

if (@root_refs)
{



if ($parent_path eq $CWD)
{
ENV_say( 1, "Currently in '$parent_path',",
"Which seems to be a Parent of GBS Roots");
} else
{
ENV_say( 1, "Currently in GBS Root '$parent_path'");
}
my $nr_not_in_roots = @root_refs - $nr_in_roots;
if ($nr_not_in_roots > 0)
{
ENV_say( 1, "It contains $nr_not_in_roots non-added GBS Root directories:");
my @dirs;
foreach my $ref (@root_refs)
{
my ($dir, $is_this_platform, $is_other_platform, $is_in_roots) = @{$ref};
if (!$is_in_roots)
{
if ($is_other_platform)
{
$dir = "*$dir";			    # mark '*' for other platform
} else
{
$dir = " $dir";			    # mark ' ' for this platform
}
push @dirs, $dir;
}
}
ENV_say( 0, FORMAT_cols( 0, 4, '  ', 3, \@dirs));
if (ASK_YNQ( 'Add one of these?', 'N', 1) eq 'Y')
{
my $selected_dir;
if (@dirs == 1)
{
$selected_dir = $dirs[0];
ENV_say( 1, "Single GBS Root taken");
} else
{
$selected_dir = ASK_value_from_menu( "Select Root/System Directory", 0, ( '', @dirs));
}
if ($selected_dir ne '')
{
$root_path = "$parent_path/" . substr( $selected_dir, 1);	# remove leading ' ' or '*'
} else
{
ENV_say( 1, "No GBS Root selected");
}

}
} else
{
ENV_say( 1, "It contains NO non-added GBS Root directories");
}
}

return $root_path;
}




sub check_gbs_root($$$)
{
my ($values_ref,
$default_ref,
$args_ref,
) = @_;
my $error_txt = '';

my $root_path = $values_ref->[0];

if ($root_path ne '')
{
if (-d $root_path)
{
$error_txt = "Not a GBS Root: $root_path"
if (!SWGLO_check_system( $root_path));
} else
{
$error_txt = "No such directory '$root_path'";
}
}

return $error_txt;
}




sub do_remove_root($)
{
my ($root_path) = @_;	# Perl format

ENV_say( 1, "Remove Root/System $root_path");

if ($root_path eq '')
{
$root_path = select_root( "Select Root to remove from Roots-List", 0);
}

if ($root_path eq '-')
{
$CANCELLED = 1;
} else
{
$root_path = ENV_perl_path_nospace( $root_path);
if (ROOTS_delete( $root_path))
{
ENV_say( 1, "Removed Root '$root_path'",
"Note that the Root Directories/Files are NOT deleted")
if (-e $root_path);
} else
{
ENV_sig( EE => "Root '$root_path' not in Roots-List");
}
if (-e FSPEC_system( 'system.gbs', $root_path))
{
my $system_name = eval { SYSTEM_get( $root_path, 'system_name') };
if (defined $system_name)
{
foreach my $envvar (qw( GBS_ADMINISTRATOR GBS_INTEGRATOR ))
{
my @values = ENV_getenv( $envvar);
my @new_values = grep( $_ ne $system_name, @values);
GBSENV_setenv( $envvar => \@new_values);
}
PROFILE_open( undef);
PROFILE_remove( GBS_ADMINISTRATOR => $system_name);
PROFILE_remove( GBS_INTEGRATOR => $system_name);
PROFILE_close(1);
}
} else
{
ENV_sig( I => 'No system.gbs file found');
}
}

check_and_write_roots_list();

return $root_path;
}




sub do_show_roots()
{
ENV_say( 1, "Show Roots-List");

ROOTSGLO_print_root_cols();

check_and_write_roots_list();
$CANCELLED = 1;
}




sub select_root($$)
{
my ($prompt,		# for ASK_value_from_menu
$default,		# for ASK_value_from_menu (value or ref)
) = @_;
my $selected_root_path;

my $help_line = ROOTSGLO_get_helpline();
my @root_path_refs = ROOTSGLO_format_root_paths( 1);
unshift @root_path_refs, '';		# Add <none>

$selected_root_path = ASK_value_from_menu( [ $prompt, $help_line ], $default, @root_path_refs);
if ($selected_root_path eq '')
{
$selected_root_path = '-';
}

return $selected_root_path;
}




sub add_to_roots_list($)
{
my ($new_root_path) = @_;   # Perl format

if (ROOTS_add( $new_root_path))
{
ENV_say( 1, "Root '$new_root_path' added to Roots List...");
check_and_write_roots_list();
} else
{
ENV_sig( W => "Root '$new_root_path' already in Roots List");
}
}




sub check_and_write_roots_list()
{
foreach my $root_path (ROOTS_get_all( 1))
{
if (!-e $root_path)
{
ENV_say( 1, "Root '$root_path'",
"in Roots-list does not exist (anymore)");
ROOTS_delete( $root_path)
if (ASK_YN( 'Remove?', 'Y') eq 'Y');
}
}
ROOTS_write();
}


