#==============================================================
#
#   gbssws.pl
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
use strict;
use warnings;
use integer;
print "*GBSSWS @ARGV\n" if ($ENV{GBSDEBUG_FILE});




use lib $ENV{GBS_SCRIPTS_PATH};
use glo::env;
use glo::shell;
use glo::ask;
use glo::genopt;
use gbsenv;
use gbsglo;
use gbscmd;
use run;
use validate;
use swb;
use swc;
use sws;
use swglo;




sub ask_subsys();




my $CWD = ENV_cwd();
my $GBS_ROOT_PATH = GBSENV_gbs_root_path();

my $GBS_SUBSYS = ENV_getenv( 'GBS_SUBSYS');
my $GBS_COMPONENT = ENV_getenv( 'GBS_COMPONENT');
my $GBS_TARGET = ENV_getenv( 'GBS_TARGET');
my $GBS_AUDIT = ENV_getenv( 'GBS_AUDIT');

my $CANCELLED = 0;






$| = 1;
my $RC = 0;
GBSENV_init();




my @genopts = (
[ '<1>',	'subsystem',	     'sso',    '', "SubSystem. '' == ASK, '-' == none, '.' == current" ],
[ '<2>',	'component',	     'sso',    '', "Component. '-' == none, '.' == current" ],
[ 'target',	'target',	     'sso',    '', "Target. '-' == none, '.' == current" ],
[ 'new',	'create_new_subsys', 'bso',     0, "Create a new SubSystem" ],
[ 'nocd',	'no_cd',	     'bso',     0, "Do not 'cd' to the selected directories "],
);
my @genconflicts = (
[ [ new => 1],	'=', '<1>', '=', '<2>', '=', 'target' ],
[ [ '<1>' => '-' ], '=', '<2>' ],
);
GENOPT_set_optdefs( [ 'gbssws', 'sws' ], \@genopts,
'Set and/or Create new Current SubSystem',
undef);
GENOPT_set_conflicts( \@genconflicts);
GENOPT_parse();
my $SUBSYS = GENOPT_get( 'subsystem');
my $COMPONENT = GENOPT_get( 'component');
my $TARGET = GENOPT_get( 'target');
my $NEW = GENOPT_get( 'create_new_subsys');
my $NO_CD = GENOPT_get( 'no_cd');




VALIDATE_root();

my $GBS_DEV_PATH = "$GBS_ROOT_PATH/dev";
my @ALL_SUBSYS = GBSGLO_subsystems();




if (!$NEW && @ALL_SUBSYS == 0)
{
if (ASK_YN( "No Subsystem(s) defined. Create now?", 'Y') eq 'Y')
{
$NEW = 1;
}
}




if ($NEW)
{
$SUBSYS = RUN_gbsnew( 'new_subsys', $SUBSYS);
if ($SUBSYS eq '')
{
$SUBSYS = '.';
$CANCELLED = 1;
} else
{
$COMPONENT = '-';
}
}

if (!$CANCELLED)
{



if ($SUBSYS eq '')
{
$SUBSYS = ask_subsys();
}
my ($new_target, $new_component, $new_subsys) = SWGLO_validate( $TARGET, $COMPONENT, $SUBSYS);





if ($new_subsys ne $GBS_SUBSYS)
{
SWC_set( $new_subsys, '', 0);	# Clear Current Component

my $must_write = ($new_subsys ne '' || $SUBSYS eq '-');
SWS_set( $new_subsys, $must_write);
}

if ($new_component ne $GBS_COMPONENT)
{
my $must_write = ($new_component ne '' || $COMPONENT eq '-');
SWC_set( $new_subsys, $new_component, $must_write);
}

if ($new_target ne $GBS_TARGET)
{
SWB_set( $new_target, 1);
}

ENV_sig( W => "No current SubSystem")
if ($new_subsys eq '' && $SUBSYS ne '-');
ENV_sig( W => "No current Component")
if ($new_component eq '' && $COMPONENT ne '-' && GBSGLO_subsystem_is_full_gbs( $new_subsys));
ENV_sig( W => "No current Target")
if ($new_target eq '' && $TARGET ne '-');








{
my $new_cwd = ($NO_CD) ? $CWD : SWGLO_new_cwd( $GBS_ROOT_PATH, $new_subsys, $new_component);

my @lines;
if (ENV_get_changed_envs() || $new_cwd ne $CWD)
{



push @lines, GBSENV_changed_setenv_commands( 0);




push @lines, SWGLO_set_title( $GBS_ROOT_PATH, $new_subsys, $new_component);




push @lines, SHELL_cd( $new_cwd);
}

push @lines, GBSCMD_command( 'gbsshow', '--brief');

GBSENV_write_result_script( \@lines, 0);
}
}

ENV_exit( $RC);




sub END
{
ENV_print_end_msg( 0);
}




sub ask_subsys()
{
my $subsys;




if (scalar @ALL_SUBSYS == 0)
{
ENV_sig( EE => "No SubSystem(s) defined - use 'sws --new' to define a SubSystem");
} elsif (scalar @ALL_SUBSYS == 1)
{
$subsys = $ALL_SUBSYS[0];
ENV_say( 1, "Single SubSystem '$subsys' taken by default");
} else
{



my $longest_string = 0;
my @subsys_refs;

foreach my $subsys (@ALL_SUBSYS)
{
my ($ss_type, $both_length);
$ss_type = GBSGLO_subsystem_type( $subsys);
$both_length = length( $subsys) + length( $ss_type);
push @subsys_refs, [ $subsys, $ss_type, $both_length ];
$longest_string = $both_length
if ($both_length > $longest_string);
}

my @menu_refs = ( [ '', '-' ] );
foreach my $ref (@subsys_refs)
{
my ($subsys, $ss_type, $both_length) = @{$ref};
my $is_target = (GBSGLO_subsystem_does_build( $subsys, $GBS_TARGET)) ? '+' : '-';
my $is_audit = (GBSGLO_subsystem_does_audit( $subsys, $GBS_AUDIT, $GBS_TARGET)) ? '+' : '-';
my $right_string = "[$ss_type$is_target$is_audit]";
my $string = "$subsys  " . (' ' x ($longest_string - $both_length)) . $right_string;
push @menu_refs, [ $string , $subsys ];
}
my $default_subsys = ENV_getenv( 'GBS_SUBSYS');
my $prompt = "Select SubSystem [$GBS_TARGET,$GBS_AUDIT]";
my $help =   '[<SubSysType><t><a>] -> t = Active for current Target, a = Active for current Audit';
$subsys = ASK_value_from_menu( [ $prompt, $help ], \$default_subsys, @menu_refs);
$subsys = '-'
if ($subsys eq '');
}

return $subsys;
}


