#===============================================================
#
#   gbsuninstall.pl
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright © 2010-2020 - Randy Marques - All rights reserved
#===============================================================
use strict;
use warnings;
use integer;
print "*GBSUNINSTALL @ARGV\n" if ($ENV{GBSDEBUG_FILE});




use 5.016_003;
use lib $ENV{GBS_SCRIPTS_PATH};
use glo::env;
use glo::slurp;
use glo::genopt;
use glo::ask;
use gbsenv;
use settings;




sub do_ask();
sub do_all();
sub do_full();
sub do_versions();
sub uninstall_version($);
sub get_versions();




my $IS_WIN32 = (ENV_is_win32());
my $IS_LUNIX = ! $IS_WIN32;
my $IS_DEVELOPMENT = ENV_is_development();

my $GBS_SCRIPTS_ROOT = ENV_perl_paths( ENV_getenv( 'GBS_SCRIPTS_ROOT'));
my $GBS_SCRIPTS_PATH = GBSENV_scripts_path();
my $GBS_BASE_PATH = ENV_getenv_perl_path( 'GBS_BASE_PATH');	# %APPDATA%/.gbs or ~/.gbs
ENV_sig( EE => 'GBS_BASE_PATH not defined. GBS is not installed')
if ($GBS_BASE_PATH eq '');




my %ACTIONS = (

ask		=> [ 0, \&do_ask ],
all		=> [ 0, \&do_all ],
full	=> [ 0, \&do_full ],
versions	=> [ 1, \&do_versions ],
);




my $INTERACTIVE = 0;






$| = 1;
my $RC = 0;
GBSENV_init();




my @genopts = (
[ '<1>',  'action',	    'ssos?,all,full,versions',	'',  'Action. Default is interactive ask'   ],
[ '<*>',  'versions',   'sao',			'',  'Versions  List'			    ],
);
GENOPT_set_optdefs( 'gbsuninstall', \@genopts,
'Uninstall GBS version(s).',
undef);
GENOPT_parse();
my $ACTION = GENOPT_get( 'action');
my @LIST = GENOPT_get( 'versions');




if ($ACTION eq '')
{
$ACTION = 'ask';
$INTERACTIVE = 1;
}

if ($ACTION eq '?')
{
ENV_say( 1, 'Actions: ask all full versions');
} else
{
my $func_ref = $ACTIONS{ $ACTION };
if (defined $func_ref)
{
my ($list_required, $function) = @{$func_ref};
if ($list_required)
{
ENV_sig( EE => 'Version(s) required')
if (@LIST == 0);
} else
{
ENV_sig( EE => "Version(s) (@LIST) not allowed")
if (@LIST != 0);
}



&$function();
} else
{
ENV_sig( F => "Invalid action '$ACTION'");
}
}




ENV_exit( $RC);




sub END
{
ENV_print_end_msg( 1);
}




sub do_ask()
{
my @main_menu_items = (
[ 'Uninstall Full',		    \&do_full ],
[ 'Uninstall All Versions',	    \&do_all ],
[ 'Uninstall Specific Versions',    \&do_versions ],
);
ASK_menu( 'Select function to perform', \@main_menu_items, undef);
}




sub do_full()
{
ENV_say( 1, 'Uninstall Full')
if (!$INTERACTIVE);




do_all();




SETTINGS_uninstall_gbs( $INTERACTIVE);
}




sub do_all()
{
ENV_say( 1, 'Uninstall All Versions')
if (!$INTERACTIVE);

map { uninstall_version( $_) } get_versions();
}




sub do_versions()
{
my @all_versions = get_versions();

if (@all_versions)
{
if ($INTERACTIVE)
{
my $version;
my @all_versions = get_versions();
do
{
$version = ASK_value_from_menu( '', -1, ('<Exit>', @all_versions));
if ($version ne '<Exit>')
{
uninstall_version( $version);
@all_versions = get_versions();

}
} while ($version ne '<Exit>' && @all_versions);
} else
{
ENV_say( 1, 'Uninstall Specific Versions');




foreach my $version (@LIST)
{
ENV_sig( EE => "Version '$version' does not exist. Valid versions are:", @all_versions)
if (!-d "$GBS_SCRIPTS_ROOT/$version");
}




map { uninstall_version( $_) } @LIST;
}
} else
{
ENV_say( 1, "No versions found in '$GBS_SCRIPTS_ROOT'");
}
}




sub uninstall_version($)
{
my ($version) = @_;

ENV_say( 2, "Uninstalling $version...");




my $version_path = "$GBS_SCRIPTS_ROOT/$version";
ENV_say( 2, "Delete: $version_path");
if ($IS_DEVELOPMENT)
{
ENV_say( 2, '- Simulate Delete');
} else
{
ENV_rmtree( $version_path, 1, 1, 'E');	    # do not delete top-dir, force_delete
}




SETTINGS_uninstall_version( $version, $INTERACTIVE);
}




sub get_versions()
{
return grep( /^(\d+\.\d\d[\d_-]*|beta|cur)$/, SLURP_dir_dirs( $GBS_SCRIPTS_ROOT, 0));
}


