#===============================================================
#
#   gbswhich.pl
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
use strict;
use warnings;
use integer;
print "*GBSWHICH @ARGV\n" if ($ENV{GBSDEBUG_FILE});




use lib $ENV{GBS_SCRIPTS_PATH};
use glo::env;
use glo::ask;
use glo::genopt;
use glo::format;
use glo::slurp;
use gbsenv;
use gbsglo;
use system;
use scope;
use flincs;
use target;
use audit;
use tool;
use sys;
use validate;
use gbsoptenv;




sub do_show();
sub do_show_type($);
sub do_show_component($);
sub search_file($$);
sub search_file_by_path($$$);




my $GBS_TARGET = ENV_getenv( 'GBS_TARGET');
my $GBS_TARGETS = ENV_getenv( 'GBS_TARGETS');
my @GBS_TARGETS = ENV_getenv( 'GBS_TARGETS');
my @GBS_ALL_TARGETS = ENV_getenv( 'GBS_ALL_TARGETS');
my @GBS_AUDITS = ENV_getenv( 'GBS_AUDITS');
my @GBS_ALL_AUDITS = ENV_getenv( 'GBS_ALL_AUDITS');
my @GBS_TOOLS = ENV_getenv( 'GBS_TOOLS');
my @GBS_ALL_TOOLS = ENV_getenv( 'GBS_ALL_TOOLS');

my $GBS_TARGET_PLUGIN = ENV_getenv( 'GBS_TARGET_PLUGIN');
my @SUB_DIRS = qw( inc loc bld dat opt sav ALL );






$| = 1;
my $RC = 0;
GBSENV_init();




my @genopts = (
[ '<1>',    'file_or_type_or_comp', 'sso', "", "filetype or Component or filename or SubDir/filename or */filename" ],
[ '<2>',    'search_type',          'sso', "", "filetype to search" ],
);
my @genenvs = qw( FLAGS_* );
GENOPT_set_optdefs( 'gbswhich', \@genopts,
'Lookup header-files, objects, flags',
undef);
GENOPT_set_envdefs( \&GBSOPTENV_set, \&GBSOPTENV_print, 'GBS_', \@genenvs);
GENOPT_parse();
my $FILE_OR_TYPE_OR_COMP = GENOPT_get( 'file_or_type_or_comp');
my $SEARCH_TYPE = GENOPT_get( 'search_type');




my $GBS_ROOT_PATH = VALIDATE_root();
my $GBS_SUBSYS = ENV_getenv( 'GBS_SUBSYS');
my $SS_TYPE = ENV_getenv( 'GBS_SSTYPE');
my $IS_FULL_GBS = ($SS_TYPE eq 'GBS') ? 1 : 0;
my $GBS_COMPONENT = ENV_getenv( 'GBS_COMPONENT');

if ($GBS_COMPONENT ne '')
{
ENV_chdir( "$GBS_ROOT_PATH/dev/$GBS_SUBSYS/comp/$GBS_COMPONENT/src");
}




my @ALL_SRC_TYPES;
my @ALL_INC_TYPES;
my @ALL_BLD_TYPES;
my %BLD_TYPES;	#key = src_type, value = [ @bld_types_refs ]
my %SRC_TYPES;	#key = bld_type, value = [ @src_types ]
if ($GBS_TARGET)
{
@ALL_SRC_TYPES = TARGET_get_src_types( $GBS_TARGET);
@ALL_INC_TYPES = TARGET_get_inc_types( $GBS_TARGET);
@ALL_BLD_TYPES = TARGET_get_bld_types( $GBS_TARGET);
foreach my $src_type (@ALL_SRC_TYPES)
{
my @bld_types = TARGET_get_src_items( $GBS_TARGET, $src_type, 'OUT_TYPES' );
$BLD_TYPES{$src_type} = [ @bld_types ];


push @{$SRC_TYPES{ shift @bld_types }}, "$src_type (Primary)";
map { push @{$SRC_TYPES{$_}}, $src_type } @bld_types;
}
}




if ($SEARCH_TYPE ne '')
{
VALIDATE_target();
ENV_sig( EE => "'$SEARCH_TYPE' must be an src-type (@ALL_SRC_TYPES)")
if (!grep( $SEARCH_TYPE eq $_, @ALL_SRC_TYPES));
}




if ($FILE_OR_TYPE_OR_COMP)
{
my ($dir, $name, $type) = ENV_split_spec_dnt( $FILE_OR_TYPE_OR_COMP);
my $file = "$name$type";

if ($SEARCH_TYPE ne '')
{



ENV_sig( EE => "This function is only possible for full GBS SubSystems")
if (!$IS_FULL_GBS);
ENV_sig( EE => "Cannot specify '$dir/' when specifying search_type ($SEARCH_TYPE)")
if ($dir ne '');
ENV_sig( EE => "Must specify 'name.type' in first parameter when specifying search_type ($SEARCH_TYPE)")
if ($name eq '' || $type eq '');
VALIDATE_target();
search_file_by_path( $FILE_OR_TYPE_OR_COMP, $name, $type);
} else
{
if ($dir eq '' && $name eq '' && $type ne '.')
{



ENV_sig( EE => "Second parameter ($SEARCH_TYPE) must be empty when first is .type")
if ($SEARCH_TYPE ne '');
VALIDATE_target();
do_show_type( $type);
} elsif ($FILE_OR_TYPE_OR_COMP eq '.' || ($dir eq '' && $name ne '' && $type eq ''))
{
ENV_sig( EE => "This function is only possible for full GBS SubSystems")
if (!$IS_FULL_GBS);
my $component;
if ($FILE_OR_TYPE_OR_COMP eq '.')
{
ENV_sig( EE => "No Current Component")
if ($GBS_COMPONENT eq '');
$component = $GBS_COMPONENT;
} else
{
ENV_sig( EE => "No such Component '$FILE_OR_TYPE_OR_COMP'")
if ( !GBSGLO_component_exists( $GBS_ROOT_PATH, $GBS_SUBSYS, $FILE_OR_TYPE_OR_COMP));
$component = $FILE_OR_TYPE_OR_COMP;
}
do_show_component( $component);
} elsif ($type eq '' || $name eq '')
{
ENV_sig( EE => "Must specify a 'name.type' in first parameter ($FILE_OR_TYPE_OR_COMP)");
} elsif ($dir eq '')
{



ENV_sig( EE => "This function is only possible for full GBS SubSystems")
if (!$IS_FULL_GBS);
VALIDATE_target();
if (grep $_ eq $type, @ALL_SRC_TYPES)
{
search_file( 'src', $file);
} elsif (grep $_ eq $type, @ALL_BLD_TYPES)
{
search_file( 'bld' ,$file);
} else
{
ENV_sig( EE => "Do not know how to search '$FILE_OR_TYPE_OR_COMP'");
}
} else
{



ENV_sig( EE => "This function is only possible for full GBS SubSystems")
if (!$IS_FULL_GBS);
search_file( $dir, $file);
}
}
} else
{



do_show();
}




ENV_exit( $RC);




sub END
{
ENV_print_end_msg(1);
}




sub do_show()
{
my @row_refs;

ENV_sig( W => "This Target ($GBS_TARGET) has no sources defined (gbsedit target.gbs)")
if (!@ALL_SRC_TYPES);

ENV_say( 1, "Overview");

push @row_refs, [ 'src_types', "@ALL_SRC_TYPES" ];
push @row_refs, [ 'inc_types', "@ALL_INC_TYPES" ];
push @row_refs, [ 'bld_types', "@ALL_BLD_TYPES" ];
my @skiptypes = ENV_getenv( 'GBS_SKIPTYPES');
push @row_refs, [ 'skiptypes', "@skiptypes" ];




push @row_refs, [ 'Targets', "@GBS_TARGETS" ];
push @row_refs, [ 'All Targets', "@GBS_ALL_TARGETS" ];




if (@GBS_AUDITS)
{
foreach my $audit (@GBS_AUDITS)
{
my $plugin = SYSTEM_get_os_ref_value( $GBS_ROOT_PATH, '.', 'audits', $audit);
my $plugin_rel = GBSENV_get_plugin_data( $plugin);
my @in_types = AUDIT_get_src_types( $audit);
push @row_refs, [ 'Audits', "$audit [$plugin ($plugin_rel)] (@in_types)" ];
}
} else
{
push @row_refs, [ 'Audits', '' ];
}
push @row_refs, [ 'All Audits', "@GBS_ALL_AUDITS" ];




if (@GBS_TOOLS)
{
foreach my $tool (@GBS_TOOLS)
{
my $plugin = SYSTEM_get_os_ref_value( $GBS_ROOT_PATH, '.', 'tools', $tool);
my $plugin_rel = GBSENV_get_plugin_data( $plugin);
my ($value_name, @values) = TOOL_get_action( $tool);
push @row_refs, [ 'Tools', "$tool [$plugin ($plugin_rel)] ($value_name => @values)" ];
}
} else
{
push @row_refs, [ 'Tools', '' ];
}
push @row_refs, [ 'All Tools', "@GBS_ALL_TOOLS" ];




{
push @row_refs, [ 'Cur. Target', $GBS_TARGET ];
if ($GBS_TARGET)
{
my @opt_row_refs = TARGET_option_items( $GBS_TARGET);
my @opt_lines = FORMAT_table( -2, 0, ' ', undef, @opt_row_refs);
map { push @row_refs, [ '', $_ ] } @opt_lines;
}
}




{
my @app_row_refs = SYS_app_items();
if (@app_row_refs)
{
my @app_lines = FORMAT_table( -2, 0, ' ', undef, @app_row_refs);
push @row_refs, [ 'Appl. Envs', shift @app_lines ];
map { push @row_refs, [ '', $_ ] } @app_lines;
} else
{
push @row_refs, [ 'Appl. Envs', 'None' ];
}
}




{
my @gbsext_envs = grep( /^GBSEXT_.+_PATH$/, keys( %ENV));
if (@gbsext_envs)
{
my @gbsext_env_refs = map { [ ' ', $_, '=>' , ENV_getenv( $_) ] } @gbsext_envs;
map { $_->[0] = 'x' if ($_->[3] ne 'PATH' && ! -d ENV_perl_path_nospace($_->[3])) } @gbsext_env_refs;
my @app_lines = FORMAT_table( -2, 0, ' ', undef, @gbsext_env_refs);
push @row_refs, [ 'GBSEXT_Paths', shift @app_lines ];
map { push @row_refs, [ '', $_ ] } @app_lines;
} else
{
push @row_refs, [ 'GBSEXT_Paths', 'None' ];
}
}




{
my @path = ENV_split_path( ENV_getenv( 'PATH'));
my $head = 'PATH';
foreach my $path (@path)
{
if ($path =~ /%/)
{
my $exp_path = ENV_expand_envs( $path);
my $lead = (-d $exp_path) ? ' ' : 'x';
push @row_refs, [ $head, "$lead $exp_path ($path)" ];
} else
{
my $lead = (-d $path) ? ' ' : 'x';
push @row_refs, [ $head, "$lead $path" ];
}
$head = '';
}
}




{
my ($os, $user, @rest) = ENV_uname();
push @row_refs, [ "SysInfo", "$os [@rest]" ];
push @row_refs, [ "Node", $user ];
}




my @lines = FORMAT_table( 0, 2, ': ', undef, @row_refs);
ASK_more( 1, '', \@lines);
}




sub do_show_type($)
{
my ($type) = @_;

ENV_say( 1, "Type '$type':");

my $is_src_type = grep( $type eq $_, @ALL_SRC_TYPES);
my $is_bld_type = grep( $type eq $_, @ALL_BLD_TYPES);
if (!($is_src_type || $is_bld_type || $type eq '.glkb'))
{
if (GBSGLO_exclude_specs( "any$type"))
{
ENV_say( 0, "  Excluded src type '$type'");
} else
{
ENV_sig( W => "Unknown src/bld type '$type'");
$RC = 1;
}
}

TARGET_set_envs( $GBS_TARGET);
if ($is_src_type)
{
my $src_type = $type;	    # make more readable
my @row_refs;

my ($gen_type_ref, $glkb_ref, $include_inc, $include_bld, $inc_types_ref, $out_types_ref,
$opt_out_types_ref, $out_files_ref, $order) =
TARGET_get_src_items( $GBS_TARGET, $src_type, qw( TYPE GLKB INCLUDE_INC INCLUDE_BLD INC_TYPES OUT_TYPES OPT_OUT_TYPES
OUT_FILES ORDER));
my $uc_type = uc substr( $src_type, 1);




{
my $plugin_rel = GBSENV_get_plugin_data( $GBS_TARGET_PLUGIN);
push @row_refs, [ 'Target', '=', "$GBS_TARGET [$GBS_TARGET_PLUGIN ($plugin_rel)]" ];
my ($gen_type, $include_re, $comment_re) = @{$gen_type_ref};
push @row_refs, [ 'Generic Type', '=', "$gen_type, Order = $order, Include-Inc = $include_inc, Include-Bld = $include_bld" ];
if (substr( $gen_type, 0, 2) eq 'gl')
{
my ($glkb_spec, $lib_spec) = @{$glkb_ref};
push @row_refs, [ 'GLKB', '=', "$glkb_spec $lib_spec" ];
}
}




{
my @command_data_refs = TARGET_get_src_command_data_refs( $GBS_TARGET, $src_type);
my $count = 1;
foreach my $command_data_ref (@command_data_refs)
{
my ($command_type, undef, $command_line, $ok_values, $exec_condition, undef) = @{$command_data_ref};

my $command_type_text = (($command_type eq 'S') ? 'Script' : 'Direct');
push @row_refs, [ "Command-$count", '=', $command_type_text ];
push @row_refs, [ '', '', $command_line ];
$count++;
}
}




my @inc_types = @{$inc_types_ref};
push @row_refs, [ 'Include Types' , '=', "@inc_types" ];




my @out_types = @{$out_types_ref};
push @row_refs, [ 'Output Types' , '=', "@out_types" ];




my @opt_out_types = @{$opt_out_types_ref};
push @row_refs, [ 'Opt. Output Types' , '=', "@opt_out_types" ];




my @out_files = @{$out_files_ref};
push @row_refs, [ 'Output Files' , '=', "@out_files" ];




push @row_refs, [ "GBS_BLD_$uc_type", '=', $out_types[0] ];

if ($IS_FULL_GBS)
{
my @option_refs = TARGET_get_options( $GBS_TARGET, 0);





{
my @defaults;
foreach my $ref (@option_refs)
{
my ($opt_name, $default_value_name) = @{$ref};
push @defaults, "$opt_name = $default_value_name";
}
push @row_refs, [ 'Options Defaults', '=', join( ', ', @defaults) ];
}




push @row_refs, [ 'Options', '=', '' ];
foreach my $ref (@option_refs)
{
my ($opt_name) = @{$ref};
my @values;
foreach my $sub_option (TARGET_get_option_names( $opt_name))
{
my $values_ref = TARGET_get_src_subitems( $GBS_TARGET, $src_type, $opt_name, $sub_option);
push @values, "$sub_option=(@{$values_ref})";
}
$opt_name = substr( "$opt_name     ", 0, 8);
push @row_refs, [ '', '', "$opt_name: @values" ];
}




push @row_refs, [ 'Includes Tree', '=', '' ];
my @incsfile_refs = FLINCS_get_incs_tree( $src_type, $GBS_COMPONENT);
my $separator_printed = 0;
foreach my $ref (@incsfile_refs)
{
my ($filespec, $type, $lines_ref) = @{$ref};
$filespec = GBSGLO_short_filespecs( $filespec);
if ($type eq 'B' && !$separator_printed)
{
push @row_refs, [ '', '', '--' ];
$separator_printed = 1;
}
push @row_refs, [ '', '', $filespec ];
map { push @row_refs, [ '', '', "  $_" ] }
GBSGLO_short_filespecs( @{$lines_ref});
}




push @row_refs, [ 'Flags Tree', '=', '' ];
my @flagsfile_refs = FLINCS_get_flags_tree( $src_type, $GBS_COMPONENT);
$separator_printed = 0;
foreach my $ref (@flagsfile_refs)
{
my ($filespec, $type, $lines_ref) = @{$ref};
$filespec = GBSGLO_short_filespecs( $filespec);
if ($type eq 'B' && !$separator_printed)
{
push @row_refs, [ '', '', '--' ];
$separator_printed = 1;
}
push @row_refs, [ '', '', $filespec ];
if (@{$lines_ref} < 6)
{
map { push @row_refs, [ '', '', "  $_" ] } @{$lines_ref};
} else
{
my @lines = @{$lines_ref};
my $between = @lines - 6;
map { push @row_refs, [ '', '', "  $_" ] } @lines[0..2];
push @row_refs, [ '', '', "  ...($between)" ];
map { push @row_refs, [ '', '', "  $_" ] } @lines[-3..-1];
}
}




my @flags = FLINCS_get_flags( $src_type, $GBS_COMPONENT);
my $flags = ENV_join_quoted_space( @flags);
push @row_refs, [ 'Gen Flags', '=', $flags ];




my $key = "GBS_FLAGS_$uc_type";
my $gbs_flags = ENV_getenv( $key );
push @row_refs, [ $key, '=', $gbs_flags ];




{




my @set_refs = TARGET_get_inits( $GBS_TARGET, 'SET');
if (@set_refs)
{
push @row_refs, [ 'Set', '=', '' ];
foreach my $ref (@set_refs)
{
my ($name, $value) = @{$ref};
push @row_refs, [ '', '', "$name = $value" ];
}
}
my @setpath_refs = TARGET_get_inits( $GBS_TARGET, 'SETPATH');
if (@setpath_refs)
{
push @row_refs, [ 'SetPath', '=', '' ];
foreach my $ref (@setpath_refs)
{
my ($name, @values) = @{$ref};
push @row_refs, [ '', '', "$name = @values" ];
}
}
}




foreach my $audit( AUDIT_get_audits_by_src( $src_type))
{

my $plugin = SYSTEM_get_os_ref_value( $GBS_ROOT_PATH, '.', 'audits', $audit);
my $plugin_rel = GBSENV_get_plugin_data( $plugin);
push @row_refs, [ 'Audit', '=', "$audit [$plugin ($plugin_rel)]" ];
my @audit_item_refs = ( [ AUDIT   => AUDIT_get_src_command_data_refs( $audit, $src_type) ],
[ SUMMARY => AUDIT_get_section_commands_data_refs( $audit, 'SUMMARY') ],
[ MAINT	  => AUDIT_get_section_commands_data_refs( $audit, 'MAINT') ],
);
foreach my $ref (@audit_item_refs)
{
my ($type, @command_data_refs) = @{$ref};
my $count = 1;
foreach my $command_data_ref (@command_data_refs)
{
my ($command_type, undef, $command_line, $ok_values, $exec_condition, undef) = @{$command_data_ref};

my $command_type_text = ($command_type eq 'S') ? 'Script' : 'Direct';
my $long_type = sprintf( "%-7s", $type);
push @row_refs, [ "  $long_type Command-$count", '=', $command_type_text ];
push @row_refs, [ '', '', $command_line ];
$count++;
}
}
}
} else
{
push @row_refs, [ '**No more info**', ':', "SubSys '$GBS_SUBSYS' is non Full GBS" ];
}

my @lines = FORMAT_table( 0, 2, ' ', undef, @row_refs);
ENV_say( 0, "Src type:");
ASK_more( 1, '', \@lines);
}

if ($is_bld_type)
{
my $bld_type = $type;		# make more readable
ENV_say( 0, "  ----")
if ($is_src_type);
my @src_types = @{$SRC_TYPES{ $bld_type}};
ENV_say( 0, "Bld type:");
ENV_say( 0, "  Source(s)\t= @src_types");
}

if ($type eq '.glkb')		# fixed - nonexistng type
{
my @row_refs;




my @search_paths = FLINCS_get_incs( '.glkb', $GBS_COMPONENT);
push @row_refs, [ 'Search Path', '=', '' ];
map { push @row_refs, [ '', '', $_ ] } @search_paths;

ENV_say( 0, "GLKB type:");
ENV_say( 0, FORMAT_table( 0, 2, ' ', undef, @row_refs));
}
}




sub do_show_component($)
{
my ($component) = @_;

my $gbs_subsys_path = "$GBS_ROOT_PATH/dev/$GBS_SUBSYS";
my %all_targets = map { $_ => 1 } @GBS_ALL_TARGETS;
my %all_audits = map { $_ => 1 } @GBS_AUDITS;
{
ENV_say( 0, "SubSys '$GBS_SUBSYS':");
my @row_refs;

my @targets = grep { exists $all_targets{$_} } SLURP_dir_dirs( "$gbs_subsys_path/build", 0);
push @row_refs, [ 'Targets', "@targets" ];

my @audits = grep { exists $all_audits{$_} } SLURP_dir_dirs( "$gbs_subsys_path/audit", 0);
push @row_refs, [ 'Audits', "@audits" ];

ENV_say( 0, FORMAT_table( 0, 2, ': ', undef, @row_refs));
}

{
ENV_say( 0, "Component '$component':");
my $gbs_component_path = "$gbs_subsys_path/comp/$component";
my @row_refs;

my @targets = grep { exists $all_targets{$_} } SLURP_dir_dirs( "$gbs_component_path/bld", 0);
push @row_refs, [ 'Targets', "@targets" ];

my @audits = grep { exists $all_audits{$_} } SLURP_dir_dirs( "$gbs_component_path/aud", 0);
push @row_refs, [ 'Audits', "@audits" ];

my @scope_dirs = SCOPE_get_components( $component);
push @row_refs, [ 'Scope', "@scope_dirs" ];

ENV_say( 0, FORMAT_table( 0, 2, ': ', undef, @row_refs));
}
}




sub search_file($$)
{
my ($dir,
$file) = @_;

my $search = $dir;
$search = "bld/$GBS_TARGET"
if ($search eq 'bld');
ENV_say( 1, "Search '$file' in $search':");

$search = '*'
if ($dir eq 'ALL');

my $root_path_l = length $GBS_ROOT_PATH;
my $count = 0;
my @found_files = glob(  "$GBS_ROOT_PATH/dev/*/comp/*/$search/$file");
push @found_files, glob( "$GBS_ROOT_PATH/dev/*/comp/*/bld/*/$file")
if ($dir eq 'ALL');
if (@found_files)
{
map { ENV_say( 0, "  ..." . substr( $_, $root_path_l)) } @found_files;
$count = @found_files;
}
ENV_say( 1, "File '$dir/$file': $count match(es)");
}




sub search_file_by_path($$$)
{
my ($file,
$name,
$type,
) = @_;

ENV_say( 1, "Search '$file' by '$SEARCH_TYPE':");




TARGET_set_envs( $GBS_TARGET);
my @bld_paths = FLINCS_get_incs( $SEARCH_TYPE, $GBS_COMPONENT);
push @bld_paths, FLINCS_get_bld_incs( $SEARCH_TYPE);
push @bld_paths, "$GBS_ROOT_PATH/sysbuild/makemake_stubs/ALL";
push @bld_paths, "$GBS_ROOT_PATH/sysbuild/makemake_stubs/$GBS_TARGET";

@bld_paths = ENV_perl_path_nospace( @bld_paths);




my $count = 0;
my $file_found = 0;
if (@bld_paths)
{
my @row_refs;
foreach my $dir (@bld_paths)
{
if (-d $dir)
{
my $spec = "$dir/$file";
if (-f $spec)
{
push @row_refs, [ '=>', $spec];
$file_found = 1;
$count++;
} else
{
push @row_refs, [ '', $dir];
}
} else
{
push @row_refs, [ 'xx', $dir];
}
}
ENV_say( 0, FORMAT_table( 0, 2, ' ', undef, @row_refs));
ENV_say( 0, "  ** File '$file' not found in paths")
if (!$file_found);
}
ENV_say( 1, "File '$file': $count match(es)");
}


