#===============================================================
#
#   check.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
package glo::check;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
CHECK_num_value
CHECK_string_value
CHECK_path
CHECK_other_path
CHECK_dir
CHECK_dirs
CHECK_file
CHECK_filespec
);
}




use glo::env;




sub CHECK_num_value($$@);
sub CHECK_string_value($$@);
sub CHECK_path($$$);
sub CHECK_other_path($$);
sub CHECK_dir($$$$);
sub CHECK_dirs($$$$);
sub CHECK_file($$$$);
sub CHECK_filespec($$$);

sub check_path($$$);
sub check_not_chars($$$);




my $IS_WIN32 = (ENV_is_win32());

my $NOT_PATH_CHARS = ':&()<>^;|,*? ';
my $NOT_DIR_CHARS  = ':&()<>^;|,*? ./\\';
my $NOT_DIRS_CHARS = ':&()<>^;|,*? .';
my $NOT_FILE_CHARS = ':&()<>^;|,*? /\\';




sub CHECK_num_value($$@)
{
my ($item_name,	    # undef == 'value'
$value,		    #
@allowed_values_or_refs,    # ( $value or [$min, $max] ...)
) = @_;
my $error_text = '';    # '' = OK


my $value_ok = 0;
foreach my $allowed_value_or_ref (@allowed_values_or_refs)
{
if (ref $allowed_value_or_ref)
{
my ($min, $max) = @{$allowed_value_or_ref};
if (defined $min && defined $max)
{
if ($value >= $min && $value <= $max)
{
$value_ok = 1;
last;
}
} elsif (defined $min)
{
if ($value >= $min)
{
$value_ok = 1;
last;
}
} elsif (defined $max)
{
if ($value <= $max)
{
$value_ok = 1;
last;
}
} else
{

}
} else
{
if ($value == $allowed_value_or_ref)
{
$value_ok = 1;
last;
}
}
}

if (!$value_ok)
{
my @allowed_values;
foreach my $allowed_value_or_ref (@allowed_values_or_refs)
{
if (ref $allowed_value_or_ref)
{
my ($min, $max) = @{$allowed_value_or_ref};
if (defined $min && defined $max)
{
push @allowed_values, "$min - $max";
} elsif (defined $min)
{
push @allowed_values, ">= $min";
} elsif (defined $max)
{
push @allowed_values, "<= $max";
} else
{

}
} else
{
push @allowed_values, $allowed_value_or_ref;
}
}
my $allowed_values = join( ',', @allowed_values);
$item_name = (defined $item_name) ? ucfirst $item_name : 'Value';
$error_text = "$item_name must be $allowed_values";
}

return $error_text;
}




sub CHECK_string_value($$@)
{
my ($item_name,	    # undef == 'value'
$value,		    #
@allowed_values_or_refs,    # ( $value or [$min, $max] ...)
) = @_;
my $error_text = '';    # '' = OK

my $value_ok = 0;
foreach my $allowed_value_or_ref (@allowed_values_or_refs)
{
if (ref $allowed_value_or_ref)
{
my ($min, $max) = @{$allowed_value_or_ref};
if (defined $min && defined $max)
{
if ($value ge $min && $value le $max)
{
$value_ok = 1;
last;
}
} elsif (defined $min)
{
if ($value ge $min)
{
$value_ok = 1;
last;
}
} elsif (defined $max)
{
if ($value le $max)
{
$value_ok = 1;
last;
}
} else
{

}
} else
{
if ($value eq $allowed_value_or_ref)
{
$value_ok = 1;
last;
}
}
}

if (!$value_ok)
{
my @allowed_values;
foreach my $allowed_value_or_ref (@allowed_values_or_refs)
{
if (ref $allowed_value_or_ref)
{
my ($min, $max) = @{$allowed_value_or_ref};
if (defined $min && defined $max)
{
push @allowed_values, "$min - $max";
} elsif (defined $min)
{
push @allowed_values, ">= $min";
} elsif (defined $max)
{
push @allowed_values, "<= $max";
} else
{

}
} else
{
push @allowed_values, $allowed_value_or_ref;
}
}
my $allowed_values = join( ',', @allowed_values);
$item_name = (defined $item_name) ? ucfirst $item_name : 'Value';
$error_text = "$item_name must be $allowed_values";
}

return $error_text;
}




sub CHECK_path($$$)
{
my ($item_name,	    # undef == 'path'
$perl_path,	    # abs_path
$must_exist,	    # -1 == may not exist, 0 == no check, 1 == must exist
) = @_;
my $error_text = '';    # '' = OK

$item_name = 'Path'
if (!defined $item_name);




$error_text = check_path( $item_name, $perl_path, 0);   # $other_os = 0




if ($error_text eq '')
{
if (-e $perl_path)
{
if (-d _)
{
if ($must_exist == -1)	    # may not exist
{
$error_text = "$item_name '$perl_path' already exists";
}
} else
{
if ($must_exist == 1 || $must_exist == 0) # must or may exist
{
if (-f _)
{
$error_text = "'$perl_path' is a file";
} else
{
$error_text = "'$perl_path' is not a path or file";
}
}
}
} else
{
if ($must_exist == 1)	    # must exists
{
$error_text = "$item_name '$perl_path' does not exist";
}
}
}

return $error_text;
}




sub CHECK_other_path($$)
{
my ($item_name,	    # undef == 'path'
$perl_path,	    # abs_path
) = @_;
my $error_text = '';    # '' = OK

$item_name = 'Path'
if (!defined $item_name);




$error_text = check_path( $item_name, $perl_path, 1);   # $other_os = 0

return $error_text;
}




sub check_path($$$)
{
my ($item_name,	    # undef == 'path'
$perl_path,	    # abs_path
$other_os,	    # 0 or 1 (bool)
) = @_;
my $error_text = '';    # undef = OK

my $for_win32 = $IS_WIN32 ^ $other_os;

if ($for_win32)
{



if ($perl_path !~ m!^(//.+|[A-Za-z]:(/|$))!)
{
$error_text = "Abs path on Windows must start with '//' or '<letter>:/'";
} else
{
my $test_path = substr( $perl_path, 2);	#remove 'X:' or '//' for further tests
$error_text = check_not_chars( $item_name, $test_path, $NOT_PATH_CHARS);
}
} else
{



if ($perl_path !~ m!^(/|~/)!)
{
$error_text = "Abs path on Lunix must start with '/' or '~/'";
} else
{
$error_text = check_not_chars( $item_name, $perl_path, $NOT_PATH_CHARS);
}
}

return $error_text;
}




sub CHECK_dir($$$$)
{
my ($item_name,	    # undef == 'Directory'
$dir,
$perl_path,	    # abs_path. May be ''
$must_exist,	    # -1 == may not exist, 0 == no check, 1 == must exist
) = @_;
my $error_text = '';    # '' = OK

$item_name = 'Directory'
if (!defined $item_name);




$error_text = check_not_chars( $item_name, $dir, $NOT_DIR_CHARS);




if ($error_text eq '' && $perl_path ne '')
{
my $new_path = "$perl_path/$dir";
if (-e $new_path)
{
if (-d _)
{
if ($must_exist == -1)	    # may not exist
{
$error_text = "$item_name '$new_path' already exists";
}
} else
{
if ($must_exist == 1 || $must_exist == 0) # must or may exist
{
if (-f _)
{
$error_text = "'$new_path' is a file";
} else
{
$error_text = "'$new_path' is not a path or file";
}
}
}
} else
{
if ($must_exist == 1)	    # must exists
{
$error_text = "$item_name '$new_path' does not exist";
}
}
}

return $error_text;
}



sub CHECK_dirs($$$$)
{
my ($item_name,	    # undef == 'Directory'
$dirs,
$perl_path,	    # abs_path. May be ''
$must_exist,	    # -1 == may not exist, 0 == no check, 1 == must exist
) = @_;
my $error_text = '';    # '' = OK

$item_name = 'Directories'
if (!defined $item_name);




$error_text = check_not_chars( $item_name, $dirs, $NOT_DIRS_CHARS);
if ($error_text eq '')
{
my $fc = substr( $dirs, 0, 1);
if ($fc eq '/' || $fc eq '\\')
{
$error_text = "$item_name '$dirs' may not start with '$fc'";
}
}




if ($error_text eq '' && $perl_path ne '')
{
my $new_path = "$perl_path/$dirs";
if (-e $new_path)
{
if (-d _)
{
if ($must_exist == -1)	    # may not exist
{
$error_text = "$item_name '$new_path' already exists";
}
} else
{
if ($must_exist == 1 || $must_exist == 0) # must or may exist
{
if (-f _)
{
$error_text = "'$new_path' is a file";
} else
{
$error_text = "'$new_path' is not a path or file";
}
}
}
} else
{
if ($must_exist == 1)	    # must exists
{
$error_text = "$item_name '$new_path' does not exist";
}
}
}

return $error_text;
}




sub CHECK_file($$$$)
{
my ($item_name,	    # undef == 'file'
$file,		    #
$perl_path,	    # abs_path. May be ''
$must_exist,	    # -1 == may not exist, 0 == no check, 1 == must exist
) = @_;
my $error_text = '';    # '' = OK

$item_name = 'File'
if (!defined $item_name);




$error_text = check_not_chars( $item_name, $file, $NOT_FILE_CHARS);




if ($error_text eq '' && $perl_path ne '')
{
my $filespec = "$perl_path/$file";
if (-e $filespec)
{
if (-f _)
{
if ($must_exist == -1)	    # may not exist
{
$error_text = "$item_name '$filespec' already exists";
}
} else
{
if ($must_exist == 1 || $must_exist == 0) # must or may exist
{
if (-d _)
{
$error_text = "'$filespec' is a directory";
} else
{
$error_text = "'$filespec' is not a path or file";
}
}
}
} else
{
if ($must_exist == 1)	    # must exists
{
$error_text = "$item_name '$filespec' does not exist";
}
}
}

return $error_text;
}




sub CHECK_filespec($$$)
{
my ($item_name,	    # undef == 'filespec'
$filespec,	    #
$must_exist,	    # -1 == may not exist, 0 == no check, 1 == must exist
) = @_;
my $error_text = '';    # '' = OK

$item_name = 'Filespec'
if (!defined $item_name);

if (-d $filespec)
{
$error_text = "$item_name '$filespec' is a Directory";
} else
{
my ($path, $file) = ENV_split_spec_df( $filespec);

if ($path eq '')
{
$error_text = "No path specifed in $filespec";
} elsif ($file eq '')
{
$error_text = "No file specified in $filespec";
}

if ($error_text eq '')
{
$error_text = CHECK_path( $item_name, $path, $must_exist);
}
if ($error_text eq '')
{
$error_text = CHECK_file( $item_name, $file, $path, $must_exist);
}
}

return $error_text;
}





sub check_not_chars($$$)
{
my ($item_name,	    # undef == 'value'
$value,
$invalid_chars,
) = @_;
my $error_txt = '';

my $not_mask = quotemeta $invalid_chars;

if ($value =~ /[$not_mask]/)
{
$item_name = (defined $item_name) ? ucfirst $item_name : 'Value';
$error_txt = "$item_name may not contain '$invalid_chars' ($value)";
}

return $error_txt;
}

1;


