#===============================================================
#
#   os_win.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
#   Documentation:
#	http://search.cpan.org/~jdb/Win32-0.47/Win32.pm					(Win32 functions)
#	http://search.cpan.org/~rgarcia/perl-5.6.2/lib/Win32.pod			(Win32 functions)
#	http://msdn.microsoft.com/en-us/library/windows/desktop/bb762494(v=vs.85).aspx	(CSIDL)
#===============================================================
package glo::os_win;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
OS_beep
OS_get_os_name
OS_get_hostname
OS_get_uname
OS_get_userid
OS_get_cwd
OS_short_path_name
OS_long_path_name
OS_copy_file
OS_hide_file
OS_get_desktop_path
OS_get_startmenu_path
OS_get_programs_path
OS_get_user_path
OS_is_administrator
);
}




use Win32;
use Win32::File;
use Win32::Sound;




sub OS_beep();
sub OS_get_os_name();
sub OS_get_hostname();
sub OS_get_uname();
sub OS_get_userid();
sub OS_get_cwd();
sub OS_short_path_name($);
sub OS_long_path_name($);
sub OS_copy_file($$);
sub OS_hide_file($$);
sub OS_get_desktop_path($);
sub OS_get_startmenu_path($);
sub OS_get_programs_path($);
sub OS_get_user_path($);
sub OS_is_administrator();

sub win32_set_hidden_attribute($$);
sub get_folder_path($);








(my $MAIN_NAME = uc $0 ) =~ s/\.PL$//;	    # UC and remove .pl
$MAIN_NAME =~ s!.*[/\\]!!;		    # remove leading device/directories
my $PREFIX = "$MAIN_NAME/OS_WIN";
my $PREFIX_ = ' ' x ((length $PREFIX) + 1);




my $OS_NAME;
my $HOSTNAME;
my ($UNAME_OS, $UNAME_HOST, @UNAME_REST);
my $USERID;

my   $USER_DESKTOP_PATH;
my $SYSTEM_DESKTOP_PATH;
my   $USER_STARTMENU_PATH;
my $SYSTEM_STARTMENU_PATH;
my   $USER_PROGRAMS_PATH;
my $SYSTEM_PROGRAMS_PATH;
my   $USER_USER_PATH;
my $SYSTEM_USER_PATH;




sub OS_beep()
{
my ($here) = __FILE__ =~ m^(.*)(/|\\)^;
my $filespec = "$here/beep.wav";


if (-e $filespec)
{

Win32::Sound::Play( $filespec);
} else
{

print "\a";
}
}




















sub OS_get_os_name()
{
if (! defined $OS_NAME)
{
$OS_NAME = Win32::GetOSName();
$OS_NAME = 'WinXP'
if ($OS_NAME =~ /^WinXP/);
}

return $OS_NAME;
}




sub OS_get_hostname()
{
if (! defined $HOSTNAME)
{
chomp( $HOSTNAME = `hostname 2> NUL`);
$HOSTNAME = $^O
if (!$HOSTNAME);
}

return $HOSTNAME;
}





sub OS_get_uname()
{
if (!defined $UNAME_OS)
{
my $os = OS_get_os_name();
my ($descr, $major, $minor, $build, $id, $spmajor, $spminor, $suitemask, $productype) = Win32::GetOSVersion();
$UNAME_OS = "$os ($descr)";
$UNAME_HOST = OS_get_hostname();
@UNAME_REST = ( $ENV{PROCESSOR_IDENTIFIER} );
}


return ($UNAME_OS, $UNAME_HOST, @UNAME_REST);
}





sub OS_get_userid()
{
if (!defined $USERID)
{
$USERID = Win32::LoginName();
}

return $USERID;
}




sub OS_get_cwd()
{
my $pwd;

$pwd = Win32::GetCwd();
$pwd =~ s!\\!/!g;		# Perl path

return $pwd;
}







sub OS_short_path_name($)
{
my ($path,
) = @_;
my $os_short_path;

$path =~ tr!/!\\!;

my $long_path = Win32::GetLongPathName( $path);	# first expand to get the case right
if (defined $long_path)
{
$os_short_path = Win32::GetShortPathName( $long_path)	# then shorten it
} else
{




my @path_parts = split( m'\\', $path);
my @pops;
do
{
push @pops, pop @path_parts;
my $test_path = join( '\\', @path_parts);

my $test_long_path = Win32::GetLongPathName( $test_path);
if (defined $test_long_path)
{
my $test_os_short_path = Win32::GetShortPathName( $test_long_path);
if (defined $test_os_short_path)
{
$os_short_path = join( '\\', $test_os_short_path, @pops);
}
}
} while (!defined $os_short_path && @path_parts);

}
if (!defined $os_short_path)
{
$os_short_path = $path;				# original if not succeeded

}


return $os_short_path;
}






sub OS_long_path_name($)
{
my ($path,
) = @_;
my $long_path;

$long_path = Win32::GetLongPathName( $path);


return $long_path;
}




sub OS_copy_file($$)
{
my ($in_file,
$out_file,
) = @_;
my ($rc, $command, $fail_reason);




my $copied = Win32::CopyFile( $in_file, $out_file, 1);	    # 1 == OVERWRITE
if (!$copied && -f $out_file)
{
chmod oct(777), $out_file;
$copied = Win32::CopyFile( $in_file, $out_file, 1);	    # 1 == OVERWRITE
}
if ($copied)
{
$rc = 0;
} else
{
$rc = 1;
$command = "Win32::CopyFile( $in_file, $out_file, 1);";
$fail_reason = $!;
}

return ($rc, $command, $fail_reason);
}




sub OS_hide_file($$)
{
my ($filespec,
$must_hide	    # 0 = unhide, 1 = hide, undef = test-only
) = @_;
my $old_state;

$old_state = win32_set_hidden_attribute( $filespec, $must_hide);

return $old_state;
}





















sub win32_set_hidden_attribute($$)
{
my ($file,
$on_or_off) = @_;	# 0 = unhide, 1 = hide, undef = test_only
my $was_hidden;

my $old_attrib;
Win32::File::GetAttributes( $file, $old_attrib) || die( "Unable to GetAttributes $file\n- $!");
if (defined $on_or_off)
{
my $new_attrib;
if ($on_or_off)
{
$new_attrib = $old_attrib | Win32::File::HIDDEN;
} else
{
$new_attrib = $old_attrib & ~Win32::File::HIDDEN;
}
if ($new_attrib != $old_attrib)
{
Win32::File::SetAttributes( $file, $new_attrib) || die( "Unable to SetAttributes $file\n- $!");
}
}
$was_hidden = ($old_attrib & Win32::File::HIDDEN) ? 1 : 0;


return $was_hidden;
}




sub OS_get_desktop_path($)
{
my ($system_item) = (@_);

if ($system_item)
{
if (!defined $SYSTEM_DESKTOP_PATH)
{
$SYSTEM_DESKTOP_PATH = get_folder_path( Win32::CSIDL_COMMON_DESKTOPDIRECTORY);
}
return $SYSTEM_DESKTOP_PATH;
} else
{
if (!defined $USER_DESKTOP_PATH)
{
$USER_DESKTOP_PATH = get_folder_path( Win32::CSIDL_DESKTOP);

}
return $USER_DESKTOP_PATH;
}
}




sub OS_get_startmenu_path($)
{
my ($system_item) = (@_);

if ($system_item)
{
if (!defined $SYSTEM_STARTMENU_PATH)
{
$SYSTEM_STARTMENU_PATH = get_folder_path( Win32::CSIDL_COMMON_PROGRAMS);
}
return $SYSTEM_STARTMENU_PATH;
} else
{
if (! defined $USER_STARTMENU_PATH)
{
$USER_STARTMENU_PATH = get_folder_path( Win32::CSIDL_PROGRAMS);

}
return $USER_STARTMENU_PATH;
}
}






sub OS_get_programs_path($)
{
my ($system_item) = (@_);

if ($system_item)
{
if (!defined $SYSTEM_PROGRAMS_PATH)
{

$SYSTEM_PROGRAMS_PATH = get_folder_path( Win32::CSIDL_PROGRAM_FILES)
if (!defined $SYSTEM_PROGRAMS_PATH);

}
return $SYSTEM_PROGRAMS_PATH;
} else
{
if (!defined $USER_PROGRAMS_PATH)
{
$USER_PROGRAMS_PATH = OS_get_user_path( 0) . '/MyPrograms';
}
return $USER_PROGRAMS_PATH;
}
}






sub OS_get_user_path($)
{
my ($system_item) = (@_);

if ($system_item)
{
if (!defined $SYSTEM_USER_PATH)
{
$SYSTEM_USER_PATH = get_folder_path( Win32::CSIDL_APPDATA);
}
return $SYSTEM_USER_PATH;
} else
{
if (!defined $USER_USER_PATH)
{
$USER_USER_PATH = get_folder_path( Win32::CSIDL_PERSONAL);
}
return $USER_USER_PATH;
}
}




sub get_folder_path($)
{
my ($csidl) = @_;
my $path;

$path = Win32::GetFolderPath( $csidl);
if (defined $path)
{
$path =~ tr!\\!/!;
}

return $path;
}




sub OS_is_administrator()
{
return Win32::IsAdminUser();
}

1;

