#===============================================================
#
#   spit.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
package glo::spit;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
SPIT_file
SPIT_append_file
SPIT_file_nl
SPIT_append_file_nl
SPIT_tmp_file
SPIT_tmp_file_nl
SPIT_script_file
SPIT_tmp_script_file
);
}




use glo::env;
use glo::shell;




sub SPIT_file($@);
sub SPIT_append_file($@);
sub SPIT_file_nl($@);
sub SPIT_append_file_nl($@);
sub SPIT_tmp_file($@);
sub SPIT_tmp_file_nl($@);
sub SPIT_script_file($@);
sub SPIT_tmp_script_file($@);

sub print_line($$);
sub say_line($$);
sub unable_to_open($$);




my $RECURSE_LEVEL;





sub SPIT_file($@)
{
my ($file,
@lines,		# lines and/or refs
) = @_;

if (open( my $out_fh, '>', $file))
{
$RECURSE_LEVEL = 0;
map { print_line( $out_fh, $_) } @lines;
close( $out_fh) || ENV_sig( F => "Unable to close file '$file'", "- $!");
} else
{
unable_to_open( $file, '>');
}
}





sub SPIT_append_file($@)
{
my ($file,
@lines,		# lines and/or refs
) = @_;

if (open( my $out_fh, '>>', $file))
{
$RECURSE_LEVEL = 0;
map { print_line( $out_fh, $_) } @lines;
close( $out_fh) || ENV_sig( F => "Unable to close file '$file'", "- $!");
} else
{
unable_to_open( $file, '>>');
}
}





sub SPIT_file_nl($@)
{
my ($file,
@lines,		# lines and/or refs
) = @_;

if (open( my $out_fh, '>', $file))
{
$RECURSE_LEVEL = 0;
map { say_line( $out_fh, $_) } @lines;
close( $out_fh) || ENV_sig( F => "Unable to close file '$file'", "- $!");
} else
{
unable_to_open( $file, '>');
}
}





sub SPIT_append_file_nl($@)
{
my ($file,
@lines,		# lines and/or refs
) = @_;

if (open( my $out_fh, '>>', $file))
{
$RECURSE_LEVEL = 0;
map { say_line( $out_fh, $_) } @lines;
close( $out_fh) || ENV_sig( F => "Unable to close file '$file'", "- $!");
} else
{
unable_to_open( $file, '>>');
}
}





sub SPIT_tmp_file($@)
{
my ($tmp_file,
@lines,		# lines and/or refs
) = @_;

my $file = ENV_get_tmp_spec( $tmp_file);

if (open( my $out_fh, '>', $file))
{
$RECURSE_LEVEL = 0;
map { print_line( $out_fh, $_) } @lines;
close( $out_fh) || ENV_sig( F => "Unable to close file '$file'", "- $!");
} else
{
unable_to_open( $file, '>');
}

return $file;
}





sub SPIT_tmp_file_nl($@)
{
my ($tmp_file,
@lines,		# lines and/or refs
) = @_;

my $file = ENV_get_tmp_spec( $tmp_file);

if (open( my $out_fh, '>', $file))
{
$RECURSE_LEVEL = 0;
map { say_line( $out_fh, $_) } @lines;
close( $out_fh) || ENV_sig( F => "Unable to close file '$file'", "- $!");
} else
{
unable_to_open( $file, '>');
}

return $file;
}





sub SPIT_script_file($@)
{
my ($file,
@lines,		# lines and/or refs
) = @_;

if (open( my $out_fh, '>', $file))
{
$RECURSE_LEVEL = 0;
map { CORE::say( $out_fh $_) } SHELL_batch_header();
map { print_line( $out_fh, $_) } @lines;
close( $out_fh) || ENV_sig( F => "Unable to close file '$file'", "- $!");
chmod 0755, $file;	# set file to 'x'
} else
{
unable_to_open( $file, '>');
}
}






sub SPIT_tmp_script_file($@)
{
my ($tmp_file,	# without .type
@lines,		# lines and/or refs
) = @_;

my $file = ENV_get_tmp_spec( $tmp_file) . ENV_shell_filetype();

if (open( my $out_fh, '>', $file))
{
$RECURSE_LEVEL = 0;
map { CORE::say( $out_fh $_) } SHELL_batch_header();
map { print_line( $out_fh, $_) } @lines;
map { CORE::say( $out_fh $_) } SHELL_exit_rc( 0);
close( $out_fh) || ENV_sig( F => "Unable to close file '$file'", "- $!");
chmod 0755, $file;	# set file to 'x'
} else
{
unable_to_open( $file, '>');
}

return $file;
}






sub print_line($$)
{
my ($out_fh,
$line_or_ref) = @_;

$RECURSE_LEVEL++;
ENV_sig( F => "Max Recurse level exceeded ($RECURSE_LEVEL)")
if ($RECURSE_LEVEL > 20);

if (ref $line_or_ref)	# REF
{
if (ref $line_or_ref eq 'SCALAR')
{
say_line( $out_fh, $$line_or_ref);
} elsif (ref $line_or_ref eq "ARRAY")
{
map { print_line( $out_fh, $_) } @{$line_or_ref};
} elsif (ref $line_or_ref eq "HASH")
{
ENV_sig( F => 'SPIT of HASH not implemented (yet)');
} else
{
ENV_sig( F => 'SPIT of ' . ref $line_or_ref . ' not supported');
}
} else			# SCALAR
{
print $out_fh $line_or_ref;
}

$RECURSE_LEVEL--;
}






sub say_line($$)
{
my ($out_fh,
$line_or_ref) = @_;

$RECURSE_LEVEL++;
ENV_sig( F => "Max Recurse level exceeded ($RECURSE_LEVEL)")
if ($RECURSE_LEVEL > 20);

if (ref $line_or_ref)	# REF
{
if (ref $line_or_ref eq 'SCALAR')
{
say_line( $out_fh, $$line_or_ref);
} elsif (ref $line_or_ref eq "ARRAY")
{
map { say_line( $out_fh, $_) } @{$line_or_ref};
} elsif (ref $line_or_ref eq "HASH")
{
ENV_sig( F => 'SPIT of HASH not implemented (yet)');
} else
{
ENV_sig( F => 'SPIT of ' . ref $line_or_ref . ' not supported');
}
} else			# SCALAR
{
CORE::say( $out_fh $line_or_ref);
}

$RECURSE_LEVEL--;
}




sub unable_to_open($$)
{
my ($file,
$mode,	    # > or >>
) = @_;

my $action = ($mode eq '>>') ? 'append' : 'create';
ENV_sig( F => "Unable to $action file '$file'",
"- $!");
}

1;
