#===============================================================
#
#   pclint.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
#
#   Call:
#	arg_1	= file_name	(without type)
#	arg_2	= in_filespec	(relative file spec - file_name + type
#	arg_3	= out_file_base (relative file spec - with in file type)
#	arg_4	= out_file_dir	(relative file spec - directory only)
#	arg_5	= out_filetype
#	arg_6-N = command_flags (from command line)
#   Environment:
#	General:
#	    GBS_INCS		Include paths ( -I<dir> ... )
#	    GBS_SYSINCS 	System include paths ( -SI<dir> ... )
#	    GBS_AUDIT_FLAGS	Passed from session
#	    GBS_FLAGS		From flags_<type> files
#	    GBS_SYSFLAGS	From bld_flags_<type> file
#	    GBS_AUDIT_OPTIONS	($MUST_SHOW_STDOUT)
#	Application:
#	    GBS_TARGET
#	    GBS_AUDIT
#	    GBSEXT_PCLINT_REL
#	    GBSEXT_PCLINT_PATH
#	    GBS_COMPONENT
#============================================================================
package pclint;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
PCLINT_init
PCLINT_set
PCLINT_pre
PCLINT_run
PCLINT_post
PCLINT_load
);
}




use glo::env;
use glo::slurp;
use gbsenv;
use scafile;
use lib "$ENV{GBS_SCRIPTS_PATH}/plugins/audit/pclint/bin";
use pclintglo;




sub PCLINT_init($$);
sub PCLINT_set($$);
sub PCLINT_pre($$$);
sub PCLINT_run();
sub PCLINT_post();
sub PCLINT_load();

sub handle_unkn_msg($$$);
sub get_msg_level($);




my $GBS_ROOT_PATH = GBSENV_gbs_root_path();
my $GBS_TARGET = ENV_getenv( 'GBS_TARGET');
my $GBS_AUDIT = ENV_getenv( 'GBS_AUDIT');




my $PLUGIN_NAME;	    # pclint
my $GBSEXT_PLUGIN_PATH;




my @GBS_DEFAULTS;	    # @PRE_OPTIONS, @FORMAT_OPTIONS
my @COMPILER_SETTINGS;	    #
my @GBS_SETTINGS;	    # @POST_OPTIONS




my $GOUT_FILE_SPEC;

my $SOURCE_FILE_SPEC;
my $OS_SOURCE_FILE_SPEC;
my $PP_FILE_SPEC;	    # $OUT_FILE_BASE.p";




my @PRE_OPTIONS = (
'-fff',	    # Do not fold file-names
'-wlib(1)',	    # Report only syntax errors in library files
'-passes(2)',   # Make sure to make two passes (for better error messages)
'-v',	    # Verbose to stdout (file.out)
'-libclass(angle)',
);

my @FORMAT_OPTIONS = (
'+ffn',			# Use full path name
'-width(0,0)',		# Do not break lines
'-"format=*GBS*%f*%l*%c*%t*%n*%m"',	# -format
'-"hFb^3"',		        # Output: Full files-spec, below, columns marker = ^, 3 lines
'+flm',			# Do not allow changes in format after this flag
);
my @POST_OPTIONS = (
'-zero',	    # do not count nr messages in rc

'-u',	    # unit checkout: process a single file
'-e830',	    # Ignore 'Location cited in prior...' message
'-e831',	    # Ignore 'Reference cited in prior...' message
);

my @PP_OPTIONS = (

'-b',	    # suppress PC-Lint flag
'-zero',	    # do not count nr messages in rc
'-w0',	    # disable all but fatal errors
);






sub PCLINT_init($$)
{
(	$PLUGIN_NAME,		# pclint
$GBSEXT_PLUGIN_PATH,
) = @_;


PCLINTGLO_init();

my @plugin_msg_filespecs = PCLINTGLO_get_msg_filespecs();	    # only one file

return (\&handle_unkn_msg, [ @plugin_msg_filespecs ], \&PCLINTGLO_read_msg_file, 1);        # $help_file_required
}





sub PCLINT_set($$)
{
my ($compiler_settings_ref,
$extra_gbs_defaults_ref,	# e.g.: quiet-options (none for PCLint)
) = @_;




@GBS_DEFAULTS = (@PRE_OPTIONS, @FORMAT_OPTIONS);





@COMPILER_SETTINGS = @{$compiler_settings_ref};




push @POST_OPTIONS, @{$extra_gbs_defaults_ref};
@GBS_SETTINGS = (@POST_OPTIONS);

}





sub PCLINT_pre($$$)
{
(my $src_path,
my $source_file,
$GOUT_FILE_SPEC,
) = @_;
my @printable_out_files;

$SOURCE_FILE_SPEC = "$src_path/$source_file";
$OS_SOURCE_FILE_SPEC = ENV_os_path( $SOURCE_FILE_SPEC);
my $out_file_path = "../aud/$GBS_AUDIT/$GBS_TARGET";
my $out_file_base = "$out_file_path/$source_file";
$PP_FILE_SPEC = "$out_file_base.p";

ENV_setenv( GBS_PCLINT_FLAGS => ENV_join_quoted_space( @PRE_OPTIONS, @FORMAT_OPTIONS, @COMPILER_SETTINGS, @POST_OPTIONS));
ENV_setenv( GBS_PCL_PP_FLAGS => ENV_join_quoted_space( @COMPILER_SETTINGS, @PP_OPTIONS));

SCAFILE_garg_write( '//', \@GBS_DEFAULTS, \@COMPILER_SETTINGS, \@GBS_SETTINGS);

@printable_out_files = ( $GOUT_FILE_SPEC, $PP_FILE_SPEC );

return @printable_out_files;
}






sub PCLINT_run()
{
my $rc = 0;

ENV_sig( F => "The PCLINT_run function is obsolete");

return $rc;
}





sub PCLINT_post()
{

my @warning_refs;


my @total_refs;


my @metric_refs;


my $nr_command_line_errors = 0;




{
my @lines = ( '', '', SCAFILE_gout_read());

















my %duplicates;
for (my $i = 0; $i < @lines; $i++)
{




next if (substr( $lines[$i], 0, 5) ne '*GBS*');
my $line = $lines[$i];

my ($os_filespec, $line_nr, $not_used_col_nr, $level_text, $msg_nr, $msg_text) = split( '\*', substr($line, 5), 6);
{

if (exists $duplicates{$line})  # May appear twice because also output to stderr
{

} else
{
$duplicates{$line} = 1;
if ($os_filespec eq '')
{
print "$level_text $msg_nr: $msg_text\n";
$nr_command_line_errors++;
} else
{
my $msg_level = '';		# filled by GBSBGSCA
my $src_line_nr;
my $col_nr;
my $help_file_url = '';	# filled by GBSBGSCA
my @extra_src_lines;	# macro expansion

if ($os_filespec eq $OS_SOURCE_FILE_SPEC)	# In Source
{
$os_filespec = '-';
$src_line_nr = $line_nr;
} else					# In Include
{
$src_line_nr = -1;
}

my ($lead) = $lines[$i - 1] =~ /^(\s*)\^/;		# look for '^' in previous line
$col_nr = (defined $lead) ? length( $lead) : -1;

for (my $j = 2; $lines[$i - $j] =~ /^#\.\.\./; $j++)
{
unshift @extra_src_lines, $lines[$i - $j];
}

push @warning_refs, [ $src_line_nr, $col_nr, $os_filespec, $line_nr, $msg_level, $level_text, $msg_nr, $msg_text, $help_file_url, @extra_src_lines ];

}
}
}
}





@warning_refs = grep( $_->[6] != 830 && $_->[6] != 831, @warning_refs);	    # $msg_nr
}




{
my $s_nr_lines = -1;
my $t_nr_lines = -1;
my $t_nr_const = -1;




$s_nr_lines = () = SLURP_file( $SOURCE_FILE_SPEC);





if (-e $PP_FILE_SPEC)
{

my @lines = grep( $_ !~ /^(\/\*|$)/, SLURP_file( $PP_FILE_SPEC));   # remove lines beginning with /* lint comments or empty lines
$t_nr_lines = @lines;

$t_nr_const = grep( /\bconst\b/, @lines);
}




@total_refs = (
[ 0, $s_nr_lines, $t_nr_lines ],	# Lines
[ 1, -1,	      -1 ],		# Funcs
[ 2, -1,	      -1 ],		# Class
[ 3, -1,	      $t_nr_const ],	# Const
);




@metric_refs = (
[ TPP => $s_nr_lines, $s_nr_lines, $s_nr_lines ],	# Total source lines before pre-processing
[ TLN => $t_nr_lines, $t_nr_lines, $t_nr_lines ],	# Total source lines after pre-processing
);
}

if ($nr_command_line_errors == 0)
{
return (\@warning_refs, \@total_refs, \@metric_refs);
} else
{
return (undef, undef, undef);
}
}





sub handle_unkn_msg($$$)
{
my ($msg_nr,		# defined
$msg_level,		# must be defined
$level_text,		# defined
) = @_;
my $help_file_url = '';

$msg_level = get_msg_level( $msg_nr);
$help_file_url = "msg_help.html#$msg_nr";	# Create a 'msg does not exist.html' file?

return ($msg_level, $level_text, $help_file_url);
}




sub get_msg_level($)
{
my ($msg_nr) = @_;
my $msg_level;

$msg_level = $msg_nr * 1;	    # make it integer
$msg_level -= 1000
if ($msg_level >= 1000);    # C++ adds 1000
if ($msg_level <= 199)
{
$msg_level = 1;		    # Syntax Error
} elsif ($msg_level <= 399)
{
$msg_level = 0;		    # Internal Error
} elsif ($msg_level <= 699)
{
$msg_level = 2;		    # Warning
} elsif ($msg_level <= 899)
{
$msg_level = 3;		    # Info
} else  #if ($msg_level <= 999)
{
$msg_level = 4;		    # Elective
}

return $msg_level;
}





sub PCLINT_load()
{
ENV_say( 1, "Loaded");
}

1;



