#===============================================================
#
#   pclintglo.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
#   Generic package for qac and qac++
#===============================================================
package pclintglo;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
PCLINTGLO_init
PCLINTGLO_get_msg_filespecs
PCLINTGLO_read_msg_file
);
}




use glo::env;
use glo::slurp;
use scadef;
use scahelpglo;




sub PCLINTGLO_init();
sub PCLINTGLO_get_msg_filespecs();
sub PCLINTGLO_read_msg_file($);

sub handle_cur_type($$$);
sub get_msg_level($);




my $GBS_AUDIT = ENV_getenv( 'GBS_AUDIT');
my $GBSEXT_PLUGIN_PATH = ENV_getenv_perl_path( 'GBSEXT_PCLINT_PATH');





my $MSG_FILE_SPEC;		# "$GBSEXT_PLUGIN_PATH/msg.txt";		# must exist
my @LEVEL_TEXTS;





sub PCLINTGLO_init()
{
ENV_sig( F => "pclint: GBSEXT_PCLINT_PATH ($GBSEXT_PLUGIN_PATH): No such directory")
if (! -d $GBSEXT_PLUGIN_PATH);

$MSG_FILE_SPEC = "$GBSEXT_PLUGIN_PATH/msg.txt";		# must exist
}




sub PCLINTGLO_get_msg_filespecs()
{
return ($MSG_FILE_SPEC);
}




sub PCLINTGLO_read_msg_file($)
{
my ($all_info_wanted) = @_;    # Bool
my @msg_def_refs;







my @msg_level_refs = SCADEF_get_levels( $GBS_AUDIT);



map { $LEVEL_TEXTS[$_->[0]] = $_->[2] } @msg_level_refs;	# $level $text




















my @in_lines = SLURP_file( $MSG_FILE_SPEC);
my @out_lines;
my $cur_type = 0;   # 0 = introduction, 1 = section, 2 = errormessage
foreach my $line (@in_lines)
{
if ($line =~ /^\d\d\.\d\s/)
{



push @msg_def_refs, handle_cur_type( $cur_type, $all_info_wanted, \@out_lines)
if ($all_info_wanted || $cur_type == 2);
@out_lines = ();
$cur_type = 1;
push @out_lines, $line;
} elsif ($line =~ /^\d{1,4}\*?\s+/)
{



push @msg_def_refs, handle_cur_type( $cur_type, $all_info_wanted, \@out_lines)
if ($all_info_wanted || $cur_type == 2);
@out_lines = ();
$cur_type = 2;
push @out_lines, $line;
} else
{
push @out_lines, $line;
}
}
if (@out_lines)
{
push @msg_def_refs, handle_cur_type( $cur_type, $all_info_wanted, \@out_lines)
if ($all_info_wanted || $cur_type == 2);
}

return @msg_def_refs;
}




sub handle_cur_type($$$)
{
my ($cur_type,	    # 0 = introduction, 1 = section, 2 = errormessage
$all_info_wanted,
$lines_ref,
) = @_;
my @msg_def_refs;

my @lines = @{$lines_ref};

if ($cur_type == 2)
{




my ($msg_nr, $heading_text) = $lines[0] =~ /^(\d+)\*?\s+(.*)/s;
ENV_sig( F => "Cannot parse line in $MSG_FILE_SPEC", "Line: $lines[0]")
if (!defined $msg_nr);
my $i = 1;
while ($heading_text !~ /--/)
{
my $next_line = $lines[$i++];
if (defined $next_line)
{
$next_line =~ s/^\s+/ /;
$heading_text .= $next_line;
} else
{
$heading_text .= '  --';
}
}

my $level = get_msg_level( $msg_nr);
my $level_text = $LEVEL_TEXTS[$level];	# $text
my $help_file_url = SCAHELPGLO_get_msg_url( $msg_nr);
if ($all_info_wanted)
{

my ($heading) = $heading_text =~ /(.*?)  --/s;	# note: 2 spaces before '--'!
$heading = '***'
if (!defined $heading);

push @msg_def_refs, [ $msg_nr, $level, $level_text, $heading_text, $help_file_url, $heading, [ @lines ]];
} else
{
push @msg_def_refs, [ $msg_nr, $level, $level_text, $heading_text, $help_file_url ];
}
} elsif ($cur_type == 1)
{




my $heading = shift @lines;
push @msg_def_refs, [ undef, $heading, [ @lines ] ];
} elsif ($cur_type == 0)
{




my $heading = 'Introduction';
push @msg_def_refs, [ undef, $heading, [ @lines ] ];
} else
{
ENV_sig( F => "Invalid cur_type ($cur_type)");
}

return @msg_def_refs;
}




sub get_msg_level($)
{
my ($msg_nr) = @_;
my $msg_level;

$msg_level = $msg_nr * 1;	    # make it integer
$msg_level -= 1000
if ($msg_level >= 1000);    # C++ adds 1000
if ($msg_level <= 199)
{
$msg_level = 1;		    # Syntax Error
} elsif ($msg_level <= 399)
{
$msg_level = 0;		    # Internal Error
} elsif ($msg_level <= 699)
{
$msg_level = 2;		    # Warning
} elsif ($msg_level <= 899)
{
$msg_level = 3;		    # Info
} else  #if ($msg_level <= 999)
{
$msg_level = 4;		    # Elective
}

return $msg_level;
}

1;


