#===============================================================
#
#   run.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
package run;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
RUN_gbsnew
RUN_gbssub_script
RUN_browser
RUN_editor
RUN_viewer
);
}




use glo::env;
use glo::slurp;
use glo::proc;
use glo::file;
use gbsenv;
use gbsglo;
use gbscmd;
use fspec;




sub RUN_gbsnew($$);
sub RUN_gbssub_script($);
sub RUN_browser($);
sub RUN_editor($$);
sub RUN_viewer($);




my $GBS_ROOT_PATH = GBSENV_gbs_root_path();

my @XTERM_COMMANDS = qw( less more type cat);
my %XTERM_COMMANDS = map { $_ => 1 } @XTERM_COMMANDS;	    # we may want to make this more generic in the future...





sub RUN_gbsnew($$)
{
my ($command_arg,	    # new_root_path, new_subsys, $new_component, new_target
$arg) = @_;
my $result;

$arg = undef
if (defined $arg && $arg eq '-');




my $os_tmp_dir = ENV_get_os_tmpdir_path();
my $gbs_new_result_file = "$os_tmp_dir/GBSNEW_RESULT_${^O}_${$}.tmp";
$ENV{GBS_NEW_FILE} = $gbs_new_result_file;

my @args = ($command_arg);
push @args, $arg
if (defined $arg);





my $rc = ENV_system( GBSCMD_command( 'gbsnew', @args), undef, undef);




if ($rc == 0 && -e $gbs_new_result_file)
{
($result, my @lines) = SLURP_file( $gbs_new_result_file);


foreach my $line (@lines)
{
my ($env_name, $value) = split( /=/, $line);
GBSENV_setenv( $env_name, $value);
}
GBSGLO_reset();	    # clear cache (uses the results above!)
} else
{
$result = '';
if ($rc == 1)
{
ENV_sig( W => "'gbsnew' completed abruptly ($rc)");
} else
{
ENV_sig( E => "'gbsnew' completed abruptly ($rc)");
}
}
unlink $gbs_new_result_file;

return $result;
}




sub RUN_gbssub_script($)
{
my ($action,
) = @_;


my $gbs_subsys = ENV_getenv( 'GBS_SUBSYS');
ENV_sig( F => "'GBS_SUBSYS' Not defined")
if ($gbs_subsys eq '');

my $command_spec = FSPEC_subsys_script( 'gbssub', $GBS_ROOT_PATH, $gbs_subsys);
if (! -f $command_spec)
{
$command_spec = GBSGLO_short_filespecs( $command_spec);
ENV_sig( EE => "SubSystem '$gbs_subsys' (non-GBS):",
"- No 'gbssub' command ($command_spec)");
}

my $full_command = ENV_expand_envs( $command_spec);

return ENV_system( ENV_command_ref( $full_command, [ $action ]), undef, undef);
}




sub RUN_browser($)
{
my ($filespec,
) = @_;

my $gbs_base_dir = ENV_getenv_perl_path( 'GBS_BASE_PATH');
my $fh = FILE_lock_ex_wait( "$gbs_base_dir/.browser.lck");
PROC_spawn_detached( GBSCMD_browse_file( $filespec));
sleep 1;		# give browser time to startup
FILE_unlock( $fh);
}




sub RUN_editor($$)
{
my ($filespec,
$editor_opt,
) = @_;

PROC_spawn_detached( GBSCMD_edit_file( $filespec, $editor_opt));
}





sub RUN_viewer($)
{
my ($filespec,
) = @_;

my $command_ref = GBSCMD_view_file( $filespec);
my $command = $command_ref->[2];
if (exists $XTERM_COMMANDS{lc $command})	# less, more, etc
{
my $xtitle = ENV_split_spec_f( $filespec);
PROC_spawn_xterm( $command_ref, $xtitle, undef, undef, undef, undef, undef);
} else
{
PROC_spawn_detached( $command_ref);
}
}

1;


