#===============================================================
#
#   settings.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
package settings;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
SETTINGS_set_check
SETTINGS_uninstall_version
SETTINGS_uninstall_gbs
);
}




BEGIN
{
if ($^O eq 'MSWin32')
{
require glo::winreg;
import  glo::winreg;
} else
{
require glo::bash;
import  glo::bash;
}
}
use glo::bash;	    # TBR
use glo::env;
use glo::file;
use glo::shortcut;
use gbsenv;




sub SETTINGS_set_check($$);
sub SETTINGS_uninstall_version($$);
sub SETTINGS_uninstall_gbs($);

sub win32_check_registry();
sub linux_check_login_files();
sub check_predefined_envvar($$$);
sub copy_setup_files();




my $IS_WIN32 = (ENV_is_win32());
my $IS_LINUX = ! $IS_WIN32;
my $IS_DEVELOPMENT = ENV_is_development();




my $GBS_BASE_PATH = ENV_getenv_perl_path( 'GBS_BASE_PATH');	#  %APPDATA%/.gbs or ~/.gbs
$GBS_BASE_PATH =~ s/GBS~\d$/.gbs/
if ($IS_WIN32);

my $SHELL_FILE_TYPE = ENV_shell_filetype();

my @LINUX_ALIAS_REFS = (

[ gbs	    => ". $GBS_BASE_PATH/gbsinit.sh" ],
[ gbshelp	    => "$GBS_BASE_PATH/define.sh cur perl gbshelp.pl" ],
[ gbssetup	    => "$GBS_BASE_PATH/define.sh cur source gbssetup.sh" ],
[ gbsuninstall  => "$GBS_BASE_PATH/define.sh cur source gbsuninstall.sh" ],
);

my $STARTMENU_PATH = ENV_get_startmenu_path( 0);    # user_path



my $WIN32_SHORTCUT_ARGS_FORMAT = '/K %s';
my $LINUX_SHORTCUT_ARGS_FORMAT = '-title "GBS xterm" -geometry 100x40+100+100 -bg white -fg darkblue -sb -ls -e "bash %s; bash -login"';

my $SHORTCUT_ARGS_FORMAT = ($IS_WIN32) ? $WIN32_SHORTCUT_ARGS_FORMAT : $LINUX_SHORTCUT_ARGS_FORMAT;

my $SETUP_ACTION = ($IS_WIN32) ? 'exec gbssetup.bat' : 'source gbssetup.sh';
my $UNINSTALL_ACTION = ($IS_WIN32) ? 'exec gbsuninstall.bat' : 'source gbsuninstall.sh';
my @SHORTCUT_REFS = (

[ 'GBS Startup [%s]',   'Application', 'gbs',	      [ 'DESKTOP', 'STARTMENU/GBS' ], 'Startup a GBS Command Window [%s]' ],
[ 'GBS Help [%s]',      'Link', 'doc/index.html',	      [ 'DESKTOP', 'STARTMENU/GBS' ], 'Startup the GBS Help [%s]' ],
[ 'GBS Setup [%s]',	    'Application', $SETUP_ACTION,     'STARTMENU/GBS', 'Setup GBS' ],
[ 'GBS Uninstall [%s]', 'Application', $UNINSTALL_ACTION, 'STARTMENU/GBS', 'Uninstall GBS' ],
);




my $THIS_GBS_REL;
my $THIS_GBS_SCRIPTS;




sub SETTINGS_set_check($$)
{
my ($new_scripts_root_path,
$new_rel,
) = @_;

ENV_say( 1, "Set/Check Startup Settings...");

$THIS_GBS_REL = $new_rel;					# may have new value
$THIS_GBS_SCRIPTS = "$new_scripts_root_path/$new_rel";	# may have new value





copy_setup_files();




if ($IS_WIN32)
{



win32_check_registry();
} else  # LINUX
{
linux_check_login_files();
}




if (ENV_get_desktop_path( 0) ne '')		# user_path
{




ENV_say( 1, "Checking Shortcuts...");
my $update_count = 0;
my $init_file = "$THIS_GBS_SCRIPTS/setup/shortcut";

my @shortcut_rels;
push @shortcut_rels, ($THIS_GBS_REL eq 'beta') ? 'beta' : 'cur';
push @shortcut_rels, 'cur'
if ($IS_DEVELOPMENT);
my $define_spec = ENV_command_filespec( "$GBS_BASE_PATH/define");    # Note: shortcuts do no expand EnvVars

foreach my $shortcut_rel (@shortcut_rels)	# 'beta' and/or 'cur'
{
my $icon_name = ($shortcut_rel eq 'beta') ? 'gbs_test' : 'gbs';
my $icon_spec = "$GBS_BASE_PATH/$icon_name";
my $define_command = "$define_spec $shortcut_rel";
foreach my $ref (@SHORTCUT_REFS)
{
my ($shortcut_name, $shortcut_type, $action, $locations_ref, $comment) = @{$ref};
my $command_spec;
my $command_args;
if ($shortcut_type eq 'Application')
{
$command_spec = undef;
$command_args = sprintf( $SHORTCUT_ARGS_FORMAT, "$define_command $action");
} else	# $shortcut_type eq 'Link'
{
$command_spec = "$THIS_GBS_SCRIPTS/$action";
$command_args = '';
}
$update_count += SHORTCUT_update( $locations_ref,
$init_file,
sprintf( $shortcut_name, $shortcut_rel),
$shortcut_type,	    # $type: Application or Link
sprintf( $comment, $THIS_GBS_REL),

$icon_spec,		    # $icon_spec
$command_spec,	    # $command_spec
$command_args,	    # $command_args
undef,		    # $working_directory
undef);		    # show: 0 = NORMAL, 1 = Maximised, -1 = Minimised
}
}




if ($update_count > 0)
{
ENV_sig( I => "$update_count standard GBS Shortcut(s) on the DeskTop and StartMenu were updated.",
"If you have your own GBS Shortcuts, please update them accordingly.");
}
} else
{
my $dtp = ENV_get_desktop_path( 0);
ENV_say( 1, "Desktop directory ( $dtp) not found.",
"No checking or creation of Shortcuts");
}
}




sub SETTINGS_uninstall_version($$)
{
my ($version,
$interactive,
) = @_;




if ($version eq 'cur' || $version eq 'beta')
{
ENV_say( 2, 'Removing Desktop and Startmenu shortcuts...');

foreach my $ref (@SHORTCUT_REFS)
{
my ($shortcut_name, $shortcut_type, $action, $locations_ref, $comment) = @{$ref};
SHORTCUT_delete( $locations_ref, sprintf( $shortcut_name, $version));
}
}
}




sub SETTINGS_uninstall_gbs($)
{
my ($interactive) = @_;




{
ENV_say( 2, 'Removing Startmenu directory...');
my $gbs_startmenu_path = "$STARTMENU_PATH/GBS";
if ($IS_DEVELOPMENT)
{
ENV_say( 2, "- Simulate Removing $gbs_startmenu_path");
} else
{
ENV_rmtree( $gbs_startmenu_path, 1, 1, 'E');
}
}




ENV_say( 2, 'Removing .gbs directory...');
ENV_say( 2, "Removing $GBS_BASE_PATH");
if ($IS_DEVELOPMENT)
{
ENV_say( 2, "- Simulate Removing $GBS_BASE_PATH");
} else
{
ENV_rmtree( $GBS_BASE_PATH, 1, 1, 'E');
}




if ($IS_LINUX)
{
my %env_file_specs;
foreach my $ref (@LINUX_ALIAS_REFS)
{
my ($alias) = @{$ref};
my ($value, $filespec, $is_login, $line_nr) = BASH_get_alias( $alias);
$env_file_specs{$filespec} = 1
if (defined $value);
}
foreach my $filespec (keys %env_file_specs)
{
BASH_remove_aliases( $filespec, 1, 'gbs', @LINUX_ALIAS_REFS);
}
}

BASH_cleanup();
}




sub win32_check_registry()
{



ENV_say( 1, "Checking Registry...");
foreach my $envvar_name (WINREG_get_userenv_names())
{
if (substr( uc $envvar_name, 0, 3) eq 'GBS')
{
my $value = WINREG_get_userenv( $envvar_name);
$value = '' if (!defined $value);
check_predefined_envvar( $envvar_name, $value, 'Registry')
}
}
}




sub linux_check_login_files()
{



my $bash_file = BASH_parse( 'EE');

foreach my $envvar_name (BASH_get_envvar_names( 1))  # export_variables only
{
my ($value, $is_export, $is_login, $filespec, $line_nr) = BASH_get_envvar( $envvar_name);

ENV_sig( W => "- It is unnecessary and unwise to set EnvVars in '$filespec($line_nr)'")
if (!$is_login);
if (substr( $envvar_name, 0, 3) eq 'GBS')
{
check_predefined_envvar( $envvar_name, $value, "$filespec($line_nr)");
}
}




foreach my $ref (@LINUX_ALIAS_REFS)
{
my ($alias, $command) = @{$ref};
my $alias_command = BASH_get_alias( $alias);
if (!defined $alias_command || $alias_command ne $command)
{

ENV_say( 1, "Adding/Replacing aliases...");
BASH_add_aliases( undef, 1, 'gbs', @LINUX_ALIAS_REFS);
last;
}
}

BASH_cleanup();
}






sub check_predefined_envvar($$$)
{
my ($name,		    # Must start with 'GBS'. Mixed case ok.
$value,
$source,	    # For errormessage. Win32: 'Registry. Linux: "$filespec($line_nr)"
) = @_;

my $warn_line;
my $uc_name = uc $name;
if (substr( $uc_name, 0, 4) eq 'GBS_')
{
if (!GBSENV_is_predefined_env( $uc_name))	# E.g: GBS_PERL_PATH, GBS_BASE_PATH
{
$warn_line = "** This GBS EnvVar may not be defined here! **";
}
} elsif (substr( $uc_name, 0, 7) eq 'GBSEXT_')
{
$warn_line = "** GBSEXT_ EnvVars must be defined in the switch.gbs$SHELL_FILE_TYPE file! **";
} elsif (substr( $uc_name, 0, 7) eq 'GBSALL_')
{
$warn_line = "** GBSALL_ EnvVars must be defined in the gbsall$SHELL_FILE_TYPE file! **";
}

if ($warn_line)
{
ENV_sig( W => "User EnvVar $name=$value",
$warn_line,
"- $source");
}
}





sub copy_setup_files()
{
my $nr_copied = 0;

my @wild_files = ($IS_WIN32) ? ( '*.bat', '*.ico') : ( '*.sh', '*.png');	# scripts and icons
my $from_dir = "$THIS_GBS_SCRIPTS/setup";

foreach my $wild_file (@wild_files)
{
foreach my $filespec (glob( "$from_dir/$wild_file"))
{
my $file = ENV_split_spec_f( $filespec);
my $replicated = FILE_replicate( $filespec, "$GBS_BASE_PATH/$file");
if ($replicated == 1)
{
ENV_say( 1, "Copied $file to $GBS_BASE_PATH");
if ($IS_LINUX && $file =~ /\.sh$/)
{
ENV_say( 2, "- chmod u+x");
ENV_chmod( 'u+x', "$GBS_BASE_PATH/$file");
}
$nr_copied++;
}
}
}

return $nr_copied;
}

1;

