#==============================================================
#
#   swsglo.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
package swglo;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
SWGLO_validate
SWGLO_new_cwd
SWGLO_split_spec
SWGLO_find_root_path
SWGLO_check_system
SWGLO_check_system_other_platform
SWGLO_set_title
);
}




use glo::env;
use glo::build;
use swb;
use gbsenv;
use gbsrc;
use system;
use roots;
use gbsglo;
use gbscmd;




sub SWGLO_validate($;$$$$$);
sub SWGLO_new_cwd($$$);
sub SWGLO_split_spec($);
sub SWGLO_find_root_path($);
sub SWGLO_check_system($);
sub SWGLO_check_system_other_platform($);
sub SWGLO_set_title(@);

sub validate_root($);
sub validate_base($);
sub find_root_path($);
sub validate_subsys($$);
sub validate_component($$$);
sub validate_target($$$$);
sub assume_target($$$$);
sub validate_audit($$$);
sub assume_audit($$);
sub check_gbs_subsys($);
sub check_nongbs_subsys($);




my $CWD = ENV_cwd();
my $GBS_ROOT_PATH = GBSENV_gbs_root_path();
my $GBS_SUBSYS = ENV_getenv( 'SUBSYS');

my ($THIS_CMD_EXT, $OTHR_CMD_EXT) = ENV_shell_filetype();




sub SWGLO_validate($;$$$$$)
{
my ($target,	    # '', '-', '.', <target>
$component,	    # '', '-', '.', <component>
$subsys,	    # '', '-', '.', <subsys>
$root_path,	    # '', '-', '.', '!!', '!<os_spec>', <number>, <os_spec>
$audit,		    # '', '-', '.', <audit>
$tool,		    # '', '-', '.', <tool>
) = @_;
my ($new_target, $new_component, $new_subsys, $new_root_path, $new_audit, $new_tool);

if (0)
{
print "** ROOT_PATH = $root_path\n"
if (defined $root_path);
print "** SUBSYS    = $subsys\n"
if (defined $subsys);
print "** COMP      = $component\n"
if (defined $component);
print "** TARGET    = $target\n"
if (defined $target);
print "** AUDIT     = $audit\n"
if (defined $audit);
print "** TOOL      = $tool\n"
if (defined $tool);
print "**\n";
}




if (defined $root_path)
{

if ($root_path eq '-')
{
$new_root_path = '';
$subsys = '-';
$component = '-';
$target = '-';	# only for swr!
$audit = '-';	# only for swr!
$tool = '-';	# only for swr!
} else
{
if ($root_path eq '!!')
{
$new_root_path = validate_root( $CWD);
} elsif (substr( $root_path, 0, 1) eq '!')
{
my $this_path = ($root_path eq '!') ? $CWD : substr( $root_path, 1);
($new_root_path, my $found_subsys, my $found_component) = find_root_path( $this_path);
$subsys = $found_subsys
if ($subsys eq '.' || $subsys eq '');
$component = $found_component
if ($component eq '.' || $component eq '');
} else	# $root_path eq '.', <number> or <spec>
{
$new_root_path = validate_root( $root_path);
validate_base( $new_root_path);
}
}

if ($new_root_path ne '')
{
my $new_gbs_root_version = SYSTEM_get( $new_root_path, 'root_version');
my $gbs_version = ENV_getenv( 'GBS_VERSION');
if ($new_gbs_root_version lt $gbs_version)
{
ENV_sig( W => "GBS ROOT-VERSION ($new_gbs_root_version) mismatch ($gbs_version)",
"Please run gbsmaint 7 9 (Upgrade)",
"Currencies are reset");
$subsys = '-';
$component = '-';
$target = '-';
$audit = '-';
$tool = '-';
}
} else
{
$audit = '';
$tool = '';
}
} else
{
$new_root_path = GBSENV_gbs_root_path();
}




if (defined $subsys)
{

if ($subsys eq '-')
{
$new_subsys = '';
$component = '-';
} else # $subsys eq '.' or <name>
{
$subsys = '.'
if ($subsys eq '');
$new_subsys = validate_subsys( $new_root_path, $subsys);
}
} else
{
$new_subsys = ENV_getenv( 'GBS_SUBSYS');
}




if (defined $component)
{

if ($component eq '-')
{
$new_component = '';
} else # $component eq '.' or <name>
{
$component = '.'
if ($component eq '');
$new_component = validate_component( $new_root_path, $new_subsys, $component);
}
} else
{
$new_component = ENV_getenv( 'GBS_COMPONENT');
}




if (defined $target)
{

if ($target eq '-')
{
$new_target = '';
} else # $target eq '.' or <name>
{
$target = '.'
if ($target eq '');
$new_target = validate_target( $new_root_path, $new_subsys, $new_component, $target);
}
} else
{
$new_target = ENV_getenv( 'GBS_TARGET');
}





if ($target ne '-')
{
if (defined $audit)
{

if ($audit eq '-')
{
$new_audit = '';
} else # $audit eq '.' or <name>
{
$audit = '.'
if ($audit eq '');
$new_audit = validate_audit( $new_root_path, $audit, $new_target);
}
} else
{
$new_audit = ENV_getenv( 'GBS_AUDIT');
}
} else
{
$new_audit = '';
}




$new_tool = GBSRC_get_root( $new_root_path, 'tool');	# TBS


return (wantarray) ? ($new_target, $new_component, $new_subsys, $new_root_path, $new_audit, $new_tool) : $new_target;
}




sub SWGLO_new_cwd($$$)
{
my ($root_path,
$subsys,
$component,
) = @_;
my $new_cwd;

if ($component ne '')
{
$new_cwd = "$root_path/dev/$subsys/comp/$component/src";
} elsif ($subsys ne '')
{
$new_cwd = "$root_path/dev/$subsys";
} elsif ($root_path ne '')
{
$new_cwd = $root_path;
} else  # $root_path eq ''
{





my $cwd = ENV_cwd();
if ($GBS_ROOT_PATH ne '' &&
substr( $cwd, 0, length($GBS_ROOT_PATH)) . '/' eq $GBS_ROOT_PATH . '/')
{
$new_cwd = ENV_parent_path( $GBS_ROOT_PATH);
ENV_say( 1, "Setting CWD to $new_cwd...");
} else
{
$new_cwd = $cwd;
}
}

return $new_cwd;
}




sub validate_root($)
{
my ($root_path,	# '.', <number>, <spec>

) = @_;
my $new_root_path = '';	# perl_spec



if ($root_path eq '.')
{
my $os_root_path = GBSRC_get_base( 'root');
ENV_sig( EE => "No current Root/System")
if ($os_root_path eq '');
$new_root_path = ENV_perl_canon_paths( $os_root_path);
} elsif ($root_path =~ /^\d+$/)	# numeric
{
my @all_os_roots = ROOTS_get_all( 0);
my $nr_roots = @all_os_roots;
if ($root_path >= 1 && $root_path <= $nr_roots)
{
$new_root_path = ENV_perl_canon_paths( $all_os_roots[$root_path - 1]);
} else
{
ENV_sig( EE => "Numeric Root-selection must be 1-$nr_roots");
}
} else
{
$new_root_path = ENV_perl_path_nospace( $root_path);
}

if ($new_root_path ne '')
{
if (-d $new_root_path)
{
if (!SWGLO_check_system( $new_root_path))
{
if (SWGLO_check_system_other_platform( $new_root_path))
{
ENV_sig( EE => "Root '$root_path' ($new_root_path) is a GBS Root on another platform");
} else
{
ENV_sig( EE => "Root '$root_path' ($new_root_path) is not a valid GBS Root");
}
}
} else
{
ENV_sig( EE => "Root '$root_path' ($new_root_path) does not exist (anymore)");
}
}




{
my $build_now = ENV_getenv( 'GBS_BUILD');
my $build_start = SYSTEM_get( $new_root_path, 'build_start');
my $build_end = SYSTEM_get( $new_root_path, 'build_end');

if ($build_now < $build_start ||
($build_end && $build_now > $build_end))
{
my $build_now_print = BUILD_build_2_version( $build_now);
my $build_start_print = BUILD_build_2_version( $build_start);
my $build_end_print = BUILD_build_2_version( $build_end);
if ($build_now < $build_start)
{
ENV_sig( EE => "Current GBS version: $build_now_print ($build_now) too old.",
"Need at least GBS version $build_start_print ($build_start).",
"Range: [$build_start-$build_end]");

} else
{
ENV_sig( EE => "Current GBS version: $build_now_print ($build_now) too new.",
"Need latest GBS version $build_end_print ($build_end).",
"Range: [$build_start-$build_end]");
}
$new_root_path = '';
}
}





return $new_root_path;
}




sub validate_base($)
{
my ($root_path) = @_;

if (! ROOTS_exists( $root_path))
{
ENV_sig( W => "'$root_path' is not listed in Roots List",
"Use 'swr --add' to add the Root to the Roots List",
"Continueing anyway...");
}
}




sub find_root_path($)
{
my ($cwd) = @_;
my ($root_path, $subsys, $component) = SWGLO_split_spec( $cwd);

if ($root_path ne '')
{

if ($subsys ne '')
{

if ($component ne '')
{

} else
{
$component = '-';
}
} else
{
$subsys = '-';
$component = '-';
}
ENV_say( 1, "Found: $root_path $subsys $component");
} else
{
ENV_sig( EE => "No GBS-Root found in '$cwd'");
}

return ($root_path, $subsys, $component, $cwd);
}




sub validate_subsys($$)
{
my ($root_path,
$subsys,	# '.' = current;
) = @_;
my $new_subsys = '';



if ($subsys eq '.')
{
$subsys = $GBS_SUBSYS
if ($root_path eq $GBS_ROOT_PATH);
if ($subsys eq '' || $subsys eq '.')
{
$subsys = GBSRC_get_root( $root_path, 'subsys');

}
}

if ($subsys ne '')
{
if (GBSGLO_subsystem_exists( $root_path, $subsys))
{
my $subsys_path = "$root_path/dev/$subsys";
if (GBSGLO_subsystem_is_full_gbs( $subsys, $root_path))
{
if (check_gbs_subsys( $subsys_path))
{
$new_subsys = $subsys;
} else
{
ENV_sig( EE => "SubSys '$subsys' is not a valid Full-GBS SubSystem",
"One or more GBS directories and/or files are missing");
}
} else
{
if (check_nongbs_subsys( $subsys_path))
{
$new_subsys = $subsys;
} else
{
ENV_sig( EE => "SubSys '$subsys' is not a valid Non-GBS SubSystem",
"One or more GBS directories and/or files are missing");
}
}
} else
{
ENV_sig( EE => "SubSys '$subsys' does not exist (anymore)");
}
}

return $new_subsys;
}




sub validate_component($$$)
{
my ($root_path,
$subsys,
$component,  # '.' = current;
) = @_;
my $new_component = '';



my $full_gbs = GBSGLO_subsystem_is_full_gbs( $subsys, $root_path);
if ($component eq '.')
{
if ($full_gbs)
{
my $gbs_subsys = ENV_getenv( 'GBS_SUBSYS');

if ($root_path eq $GBS_ROOT_PATH && $subsys eq $gbs_subsys)
{
$component = ENV_getenv( 'GBS_COMPONENT');
}
if ($component eq '' || $component eq '.')
{
$component = GBSRC_get_subsys( $root_path, $subsys, 'component');
if ($component ne '' && ! -d "$root_path/dev/$subsys/comp/$component/src")
{
ENV_sig( EE => "Component <$component> does not exist (anymore)");
}
}
} else
{
$component = '';
}
}

if ($component ne '')
{
if ($full_gbs)
{
my $comp_path = "$root_path/dev/$subsys/comp";
if (GBSGLO_component_exists( $root_path, $subsys, $component) &&
-d "$comp_path/$component/src" && -d "$comp_path/$component/loc")
{
$new_component = $component;
} else
{
ENV_sig( EE => "'$component' is not a Component in this SubSystem");
}
} else
{
ENV_sig( EE => "Cannot specify a component ($component) for Non GBS SubSystem");
}
}

return $new_component;
}




sub validate_target($$$$)
{
my ($root_path,
$subsys,
$component,
$target,	# '.' = current;
) = @_;
my $new_target;


my $original_target = $target;




my @all_targets;
if ($root_path eq $GBS_ROOT_PATH)
{
@all_targets = ENV_getenv( 'GBS_TARGETS');
}
if (@all_targets == 0)
{
my @all_target_refs = SYSTEM_get_os_refs( $root_path, '.', 'targets');   # [ $target, $plugin ]
@all_targets = map { $_->[0] } @all_target_refs;
}

if (@all_targets == 0)
{
ENV_sig( W => "No Targets defined for System '$root_path'");
$new_target = '';
} else
{



if ($target eq '.')
{
$target = ENV_getenv( 'GBS_TARGET')
if ($root_path eq $GBS_ROOT_PATH);
if ($target eq '.' || $target eq '')
{

$target = GBSRC_get_root( $root_path, 'target');

}
}

if ($target eq '')
{
$new_target = '';
} else
{



if (grep $target eq $_, @all_targets)
{
if (GBSGLO_subsystem_is_full_gbs( $subsys, $root_path))
{
$new_target = SWB_validate_dirs( $root_path, $subsys, $component, $target);
} else
{
$new_target = $target;
}
if ($new_target eq '' && $original_target eq '.')
{
$new_target = assume_target( $root_path, $subsys, $component, [@all_targets]);
}
} else
{
ENV_sig( EE => "No such Target '$target' for System '$root_path'",
"Valid Targets are: @all_targets");
}
}
}

return $new_target;
}




sub assume_target($$$$)
{
my ($root_path,
$subsys,
$component,
$all_targets_ref,
) = @_;
my $new_target = '';

if (scalar @{$all_targets_ref} == 1)
{
$new_target = $all_targets_ref->[0];
ENV_say( 1, "Single Target '$new_target' taken as default");
} else
{
if ($subsys eq '')
{
$new_target = $all_targets_ref->[0];
ENV_say( 1, "Target '$new_target' tentatively taken as default");
} else
{
if ($component eq '')
{
my $bld_path = "$root_path/dev/$subsys/comp/$component/bld";
foreach my $target (@{$all_targets_ref})
{
if (-d "$bld_path/$target")
{
$new_target = $target;
last;
}
}
}
if ($new_target eq '')
{
my $gen_path = "$root_path/dev/$subsys/build";
foreach my $target (@{$all_targets_ref})
{
if (-d "$gen_path/$target")
{
$new_target = $target;
last;
}
}
}
if ($new_target eq '')
{
ENV_say( 1, "No Target directories found");
} else
{
ENV_say( 1, "Target '$new_target' tentatively assumed");
}
}
}

return $new_target;
}




sub validate_audit($$$)
{
my ($root_path,
$audit,	# '.' = current;
$target,
) = @_;
my $new_audit;


my $original_audit = $audit;




my @all_audits;
if ($root_path eq $GBS_ROOT_PATH)
{
@all_audits = ENV_getenv( 'GBS_AUDITS');
}
if (@all_audits == 0)
{
my @all_audit_refs = SYSTEM_get_os_refs( $root_path, '*', 'audits');   # [ $audit, $plugin ]
@all_audits = map { $_->[0] } @all_audit_refs;
}

if (@all_audits == 0 && $audit ne '.')
{
ENV_sig( W => "No Audits defined for System '$root_path'");
} else
{



if ($audit eq '.')
{
$audit = ENV_getenv( 'GBS_AUDIT')
if ($root_path eq $GBS_ROOT_PATH);
if ($audit eq '.' || $audit eq '')
{

$audit = GBSRC_get_root( $root_path, 'audit');

}
}

if ($audit eq '')
{
$new_audit = '';
} else
{



if (grep $audit eq $_, @all_audits)
{
$new_audit = $audit;
if ($new_audit eq '' && $original_audit eq '.')
{
$new_audit = assume_audit( $root_path, [ @all_audits ]);
}
} else
{
ENV_sig( EE => "No such Audit '$audit' for System '$root_path'",
"Valid Audits are: @all_audits");
}
}
}

return $new_audit;
}




sub assume_audit($$)
{
my ($root_path,	# not used (yet)
$all_audits_ref,
) = @_;
my $new_audit = '';

if (scalar @{$all_audits_ref} == 1)
{
$new_audit = $all_audits_ref->[0];
ENV_say( 1, "Single Audit '$new_audit' taken as default");
}

return $new_audit;
}




sub SWGLO_find_root_path($)
{
my ($filespec) = @_;	# Abs dir or file spec
my $root_path = '';

my @dirs = split( '/', $filespec);
pop @dirs if (-f $filespec);
while (@dirs)
{
my $possible_root_path = join( '/', @dirs);
if (-f "$possible_root_path/system.gbs" && SWGLO_check_system( $possible_root_path))
{
$root_path = $possible_root_path;
last;
}
pop @dirs;
}

return $root_path;
}




sub SWGLO_split_spec($)
{
my ($filespec) = @_;
my ($root_path, $subsys, $component, $remain) = ( '', '', '', '' );

my $gbs_root_path = GBSENV_gbs_root_path();
if ($gbs_root_path ne '' && ENV_is_in_path( $filespec, $gbs_root_path))
{
$root_path = $gbs_root_path;
} else
{
$root_path = SWGLO_find_root_path( $filespec);
}

if ($root_path ne '')
{
if ($filespec =~ m!$root_path/dev/([^/]+)!)
{
$subsys = $1;
if ($filespec =~ m!$root_path/dev/$subsys/comp/([^/]+)!)
{
$component = $1;
$remain = substr( $filespec, length( "$root_path/dev/$subsys/comp/$component"));
} else
{
$remain = substr( $filespec, length( "$root_path/dev/$subsys"));
}
} else
{
$remain = substr( $filespec, length( $root_path));
}
$remain = substr( $remain, 1) if ($remain =~ m!^/!);
}

return ($root_path, $subsys, $component, $remain);
}




sub SWGLO_check_system($)
{
my ($root_path) = @_;


return ( -d "$root_path/dev" &&
-d "$root_path/ext" &&
-d "$root_path/res" &&
-f "$root_path/system.gbs" &&
-f "$root_path/switch.gbs$THIS_CMD_EXT");
}




sub SWGLO_check_system_other_platform($)
{
my ($root_path) = @_;


return ( -d "$root_path/dev" &&
-d "$root_path/ext" &&
-d "$root_path/res" &&
-f "$root_path/system.gbs" &&
-f "$root_path/switch.gbs$OTHR_CMD_EXT");
}





sub check_gbs_subsys($)
{
my ($subsys_path) = @_;

return ( -d "$subsys_path/comp" &&
-d "$subsys_path/build" &&
-d "$subsys_path/audit");
}





sub check_nongbs_subsys($)
{
my ($subsys_path) = @_;

return ( -d "$subsys_path/app" &&
-d "$subsys_path/build" &&
-d "$subsys_path/audit");
}





sub SWGLO_set_title(@)
{
my @text = @_;  # $root_path, $subsys, $component
my @lines;

if (!GBSENV_mode_is_background())	    # INTERACTIVE or FOREGROUND
{
if (@text == 1 && $text[0] eq '-')
{
push @lines, GBSCMD_set_xtitle( '');
} else
{
my @text = reverse @text;
my $text = (@text) ? join ( ' - ', @text) : '-';
push @lines, GBSCMD_set_xtitle( "GBS: $text");
}
}

return @lines;
}

1;


